#include "terasic_includes.h"
//#include "i2c.h"
#include "i2c_core.h"
#include "si5340A_config.h"

//#define SI5430_DEBUG(x)    DEBUG(x)

bool SI5340A_w_reg_only(alt_u32 i2c_base, alt_u8 reg_index, alt_u8 data8);

typedef struct {
    int FreqID;
    int RX_REG;
    bool OUTX_OE;
    alt_u64 NX_NUM;
    alt_u64 NX_DEN;
    double FREQUENCY;
} Si5340A_CONFIG;

Si5340A_CONFIG gszGi5340A_ConfigTable[] = { { SI5340_644M53125,  0, 1, 0x0580000000, 0x80000000, 644.53125 },
                                            { SI5340_625M,       0, 1, 0x05AC000000, 0x80000000, 625.0 },
                                            { SI5340_322M265625, 0, 1, 0x0B00000000, 0x80000000, 322.265625 },
                                            { SI5340_312M5,      0, 1, 0x0B58000000, 0x80000000, 312.5 },
                                            { SI5340_250M,       0, 1, 0x0E2E000000, 0x80000000, 250.0 },
                                            { SI5340_184M32,     0, 1, 0x133B80AAAB, 0x80000000, 184.32 },
                                            { SI5340_156M25,     0, 1, 0x16B0000000, 0x80000000, 156.25 },
                                            { SI5340_125M,       0, 1, 0x1C5C000000, 0x80000000, 125.0 },
                                            { SI5340_100M,       0, 1, 0x2373000000, 0x80000000, 100.0 },
                                            { SI5340_NONE,       0, 0, 0x0000000000, 0x80000000, 0 }, };

bool si5340A_find_DIV_NI(SI5340A_FREQ_ID FreqID, alt_u32 *RX_REG, bool *OUT0_OE, alt_u64 *NX_NUM, alt_u64 *NX_DEN,
        double *pfFreq)
{
    bool bFind = FALSE;
    int i;

    for (i = 0; i < sizeof(gszGi5340A_ConfigTable) / sizeof(gszGi5340A_ConfigTable[0]) && !bFind; i++) {
        if (gszGi5340A_ConfigTable[i].FreqID == FreqID) {
            bFind = TRUE;
            *RX_REG = gszGi5340A_ConfigTable[i].RX_REG;
            *OUT0_OE = gszGi5340A_ConfigTable[i].OUTX_OE;
            *NX_NUM = gszGi5340A_ConfigTable[i].NX_NUM;
            *NX_DEN = gszGi5340A_ConfigTable[i].NX_DEN;
            *pfFreq = gszGi5340A_ConfigTable[i].FREQUENCY;
        }
    } // for

    return bFind;

}

bool SI5340A_Config_Init(alt_u32 i2c_base)
{
    bool bSuccess = TRUE;

    bSuccess = oc_i2c_init(i2c_base);

    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_PAGE, 0x00); //page  0
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0018, REGA_0018);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0021, REGA_0021);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_PAGE, 0x01); //page  1
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0102, REGA_0102);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0113, REGA_0113);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0114, REGA_0114);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0115, REGA_0115);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0118, REGA_0118);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0119, REGA_0119);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_011A, REGA_011A);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0127, REGA_0127);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0128, REGA_0128);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0129, REGA_0129);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_012C, REGA_012C);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_012D, REGA_012D);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_012E, REGA_012E);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0141, REGA_0141);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_PAGE, 0x02); //page  2
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0235, REGA_0235);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0236, REGA_0236);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0237, REGA_0237);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0238, REGA_0238);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0239, REGA_0239);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_023A, REGA_023A);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_023B, REGA_023B);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_023C, REGA_023C);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_023D, REGA_023D);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_023E, REGA_023E);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_PAGE, 0x09); //page  9
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_090E, REGA_090E);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_091C, REGA_091C);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0949, REGA_0949);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_094A, REGA_094A);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_PAGE, 0x0A); //page  A
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0A03, REGA_0A03);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0A04, REGA_0A04);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0A05, REGA_0A05);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_PAGE, 0x0B); //page  B
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0B44, REGA_0B44);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0B4A, REGA_0B4A);
    DEBUG(("[SI5430] SI5340A_Config_Init %s\r\n", bSuccess ? "success" : "fail"));

    return bSuccess;
}

bool SI5340A_Config(alt_u32 i2c_base, SI5340A_FREQ_ID FreqID0, SI5340A_FREQ_ID FreqID1, SI5340A_FREQ_ID FreqID2,
        SI5340A_FREQ_ID FreqID3)
{
    bool bSuccess;
    bool OUT0_OE, OUT1_OE, OUT2_OE, OUT3_OE;
    alt_u8 RX_Reg[101];
    //alt_u8 dataR1;
    alt_u8 NX_Num_Den[45];
    alt_u8 REG_0112, REG_0117, REG_0126, REG_012B;
    double fFreq0, fFreq1, fFreq2, fFreq3;
    alt_u32 R0_REG, R1_REG, R2_REG, R3_REG;
    alt_u64 N0_NUM, N1_NUM, N2_NUM, N3_NUM;
    alt_u64 N0_DEN, N1_DEN, N2_DEN, N3_DEN;
    int n;

    bSuccess = si5340A_find_DIV_NI(FreqID0, &R0_REG, &OUT0_OE, &N0_NUM, &N0_DEN, &fFreq0);

    if (!bSuccess) {
        printf("This not my preset freq id (%d)\r\n", FreqID0);
        return FALSE;
    }
    bSuccess = si5340A_find_DIV_NI(FreqID1, &R1_REG, &OUT1_OE, &N1_NUM, &N1_DEN, &fFreq1);
    if (!bSuccess) {
        printf("This not my preset freq id (%d)\r\n", FreqID1);
        return FALSE;
    }
    bSuccess = si5340A_find_DIV_NI(FreqID2, &R2_REG, &OUT2_OE, &N2_NUM, &N2_DEN, &fFreq2);
    if (!bSuccess) {
        printf("This not my preset freq id (%d)\r\n", FreqID2);
        return FALSE;
    }
    bSuccess = si5340A_find_DIV_NI(FreqID3, &R3_REG, &OUT3_OE, &N3_NUM, &N3_DEN, &fFreq3);
    if (!bSuccess) {
        printf("This not my preset freq id (%d)\r\n", FreqID3);
        return FALSE;
    }
    if (bSuccess) {

        REG_0112 = OUT0_OE ? 0x06 : 0x04;
        REG_0117 = OUT1_OE ? 0x06 : 0x04;
        REG_0126 = OUT2_OE ? 0x06 : 0x04;
        REG_012B = OUT3_OE ? 0x06 : 0x04;

        RX_Reg[83] = (R0_REG >> 0) & 0xff;
        RX_Reg[84] = (R0_REG >> 8) & 0xff;
        RX_Reg[85] = (R0_REG >> 16) & 0xff;
        RX_Reg[86] = (R1_REG >> 0) & 0xff;
        RX_Reg[87] = (R1_REG >> 8) & 0xff;
        RX_Reg[88] = (R1_REG >> 16) & 0xff;

        RX_Reg[95] = (R2_REG >> 0) & 0xff;
        RX_Reg[96] = (R2_REG >> 8) & 0xff;
        RX_Reg[97] = (R2_REG >> 16) & 0xff;
        RX_Reg[98] = (R3_REG >> 0) & 0xff;
        RX_Reg[99] = (R3_REG >> 8) & 0xff;
        RX_Reg[100] = (R3_REG >> 16) & 0xff;

        NX_Num_Den[2] = (N0_NUM >> 0) & 0xff;
        NX_Num_Den[3] = (N0_NUM >> 8) & 0xff;
        NX_Num_Den[4] = (N0_NUM >> 16) & 0xff;
        NX_Num_Den[5] = (N0_NUM >> 24) & 0xff;
        NX_Num_Den[6] = (N0_NUM >> 32) & 0xff;
        NX_Num_Den[7] = (N0_NUM >> 40) & 0x0f;
        NX_Num_Den[8] = (N0_DEN >> 0) & 0xff;
        NX_Num_Den[9] = (N0_DEN >> 8) & 0xff;
        NX_Num_Den[10] = (N0_DEN >> 16) & 0xff;
        NX_Num_Den[11] = (N0_DEN >> 24) & 0xff;

        NX_Num_Den[13] = (N1_NUM >> 0) & 0xff;
        NX_Num_Den[14] = (N1_NUM >> 8) & 0xff;
        NX_Num_Den[15] = (N1_NUM >> 16) & 0xff;
        NX_Num_Den[16] = (N1_NUM >> 24) & 0xff;
        NX_Num_Den[17] = (N1_NUM >> 32) & 0xff;
        NX_Num_Den[18] = (N1_NUM >> 40) & 0x0f;
        NX_Num_Den[19] = (N1_DEN >> 0) & 0xff;
        NX_Num_Den[20] = (N1_DEN >> 8) & 0xff;
        NX_Num_Den[21] = (N1_DEN >> 16) & 0xff;
        NX_Num_Den[22] = (N1_DEN >> 24) & 0xff;

        NX_Num_Den[24] = (N2_NUM >> 0) & 0xff;
        NX_Num_Den[25] = (N2_NUM >> 8) & 0xff;
        NX_Num_Den[26] = (N2_NUM >> 16) & 0xff;
        NX_Num_Den[27] = (N2_NUM >> 24) & 0xff;
        NX_Num_Den[28] = (N2_NUM >> 32) & 0xff;
        NX_Num_Den[29] = (N2_NUM >> 40) & 0x0f;
        NX_Num_Den[30] = (N2_DEN >> 0) & 0xff;
        NX_Num_Den[31] = (N2_DEN >> 8) & 0xff;
        NX_Num_Den[32] = (N2_DEN >> 16) & 0xff;
        NX_Num_Den[33] = (N2_DEN >> 24) & 0xff;

        NX_Num_Den[35] = (N3_NUM >> 0) & 0xff;
        NX_Num_Den[36] = (N3_NUM >> 8) & 0xff;
        NX_Num_Den[37] = (N3_NUM >> 16) & 0xff;
        NX_Num_Den[38] = (N3_NUM >> 24) & 0xff;
        NX_Num_Den[39] = (N3_NUM >> 32) & 0xff;
        NX_Num_Den[40] = (N3_NUM >> 40) & 0x0f;
        NX_Num_Den[41] = (N3_DEN >> 0) & 0xff;
        NX_Num_Den[42] = (N3_DEN >> 8) & 0xff;
        NX_Num_Den[43] = (N3_DEN >> 16) & 0xff;
        NX_Num_Den[44] = (N3_DEN >> 24) & 0xff;
    }
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_PAGE, 0x01); //page  1
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0112, REG_0112);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0117, REG_0117);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0126, REG_0126);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_012B, REG_012B);

    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_PAGE, 0x02); //page  2
    if (bSuccess) {
        // Write the new frequency configuration (R0_REG/R1_REG)
        for (n = 83; n <= 88 && bSuccess; n++) {
            bSuccess = SI5340A_w_reg(i2c_base, n, RX_Reg[n]);
        }

        // Write the new frequency configuration (R2_REG/R3_REG)
        for (n = 95; n <= 100 && bSuccess; n++) {
            bSuccess = SI5340A_w_reg(i2c_base, n, RX_Reg[n]);
        }
    }
    if (bSuccess)
        bSuccess = SI5340A_w_reg_only(i2c_base, REGA_ADDR_023F, 0x01); //M_Update
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_PAGE, 0x03); //page  3
    if (bSuccess) {
        // Write the new frequency configuration (N0_NUM/N0_DEN)
        for (n = 2; n <= 11 && bSuccess; n++) {
            bSuccess = SI5340A_w_reg(i2c_base, n, NX_Num_Den[n]);
        }
        // Write the new frequency configuration (N1_NUM/N1_DEN)
        for (n = 13; n <= 22 && bSuccess; n++) {
            bSuccess = SI5340A_w_reg(i2c_base, n, NX_Num_Den[n]);
        }
        // Write the new frequency configuration (N2_NUM/N2_DEN)
        for (n = 24; n <= 33 && bSuccess; n++) {
            bSuccess = SI5340A_w_reg(i2c_base, n, NX_Num_Den[n]);
        }
        // Write the new frequency configuration (N3_NUM/N3_DEN)
        for (n = 35; n <= 44 && bSuccess; n++) {
            bSuccess = SI5340A_w_reg(i2c_base, n, NX_Num_Den[n]);
        }
    }

    if (bSuccess)
        bSuccess = SI5340A_w_reg_only(i2c_base, REGA_ADDR_030C, 0x01); //N0_Update
    if (bSuccess)
        bSuccess = SI5340A_w_reg_only(i2c_base, REGA_ADDR_0317, 0x01); //N1_Update
    if (bSuccess)
        bSuccess = SI5340A_w_reg_only(i2c_base, REGA_ADDR_0322, 0x01); //N2_Update
    if (bSuccess)
        bSuccess = SI5340A_w_reg_only(i2c_base, REGA_ADDR_032D, 0x01); //N3_Update
    if (bSuccess)
        bSuccess = SI5340A_w_reg_only(i2c_base, REGA_ADDR_0338, 0x01); //N_Update

    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_0339, 0x1F);
    if (bSuccess)
        bSuccess = SI5340A_w_reg(i2c_base, REGA_ADDR_PAGE, 0x00); //page  0
    if (bSuccess)
        bSuccess = SI5340A_w_reg_only(i2c_base, REGA_ADDR_001C, 0x01); //soft reset

    DEBUG(("[SI5430] SI5340A_Configuration %s\r\n", bSuccess ? "success" : "fail"));

    return bSuccess;
}

bool SI5340A_w_reg(alt_u32 i2c_base, alt_u8 reg_index, alt_u8 data8)
{

    bool bSuccess = TRUE;
    alt_u8 dataR;
    if (bSuccess) {
        //bSuccess = I2C_Write(SI5340A_I2C_SCL_BASE, SI5340A_I2C_SDA_BASE, SI5340_DeviceAddr, reg_index, data8);
        bSuccess = SI5340A_w_reg_only(i2c_base, reg_index, data8);
    }
    usleep(50 * 1000);
    if (bSuccess) {
//     	bSuccess = I2C_Read(SI5340A_I2C_SCL_BASE, SI5340A_I2C_SDA_BASE, SI5340_DeviceAddr, reg_index, &dataR);
        bSuccess = OC_I2C_Read(i2c_base, SI5340A_DeviceAddr, reg_index, &dataR, sizeof(dataR));
        //printf("[SI5430] read reg[%02xh] = %02Xh\r\n", reg_index, dataR );
    }

    if (!bSuccess) {
        printf("SI5430: failed to read page register\r\n");
    } else {
        if (dataR != data8) {
            bSuccess = FALSE;
            DEBUG(("[SI5430] read  reg[%02d] = %02xh (expected:%02xh)\r\n", reg_index, dataR, data8));
        }
    }

    return bSuccess;

}

bool SI5340A_w_reg_only(alt_u32 i2c_base, alt_u8 reg_index, alt_u8 data8)
{

    bool bSuccess;

    bSuccess = OC_I2C_Write(i2c_base, SI5340A_DeviceAddr, reg_index, &data8, sizeof(data8));

    return bSuccess;

}

