// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_prs_if
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module  intel_pcie_whr_gen4x8_prs_if
 #(
     parameter intended_device_family = "Stratix 10"
  )
  (
   input              clk500,
   input              rst_clk500_n,
   input              clk250,
   input              rst_clk250_n,
   input              prs_event_valid_i,
   input  logic [2:0] prs_event_func_i,
   input  logic [1:0] prs_event_i,
   output logic       prs_event_valid_o,
   output logic [2:0] prs_event_func_o,
   output logic [1:0] prs_event_o
  );

//********************************************************************
// DEFINE VARIABLES
//********************************************************************
  typedef struct packed {
    logic       prs_event_valid;
    logic [2:0] prs_event_func;
    logic [1:0] prs_event;
  } prs_data_t;

  logic prs_fifo_rdreq, prs_fifo_rdreq_q, prs_fifo_empty;
  prs_data_t prs_data_in, prs_data_out;
  logic [2:0] prs_rd_numdata;
  logic       prs_event_valid_q;  /* synthesis preserve_syn_only */
  logic [2:0] prs_event_func_q;   /* synthesis preserve_syn_only */
  logic [1:0] prs_event_q;        /* synthesis preserve_syn_only */
//********************************************************************
// WRITE CLOCK DOMAIN
//********************************************************************
  always @ (posedge clk250) begin //negedge rst_clk250_n or 
    if (rst_clk250_n == 0) begin
      prs_event_valid_q <= '0;
    end
    else begin
      prs_event_valid_q <= prs_event_valid_i;
    end
  end

  always @ (posedge clk250) begin
    prs_event_func_q  <= prs_event_func_i;
    prs_event_q       <= prs_event_i;
  end


  assign prs_data_in = {prs_event_valid_q, prs_event_func_q, prs_event_q};

//********************************************************************
// STORAGE
//********************************************************************
    intel_pcie_whr_gen4x8_dcfifo
    #(
      .DWIDTH ($bits(prs_data_in)),
      .AWIDTH (2),
      .intended_device_family(intended_device_family)
     ) prsfifo
    (
     .wr_rst_n(rst_clk250_n),                         // Write Domain Active low Reset
     .wr_clk(clk250),                                 // Write Domain Clock
     .wr_en(prs_event_valid_q),                       // Write Data Enable
     .wr_data(prs_data_in),                           // Write Data In
     .rd_rst_n(rst_clk500_n),                         // Read Domain Active low Reset
     .rd_clk(clk500),                                 // Read Domain Clock
     .rd_en(prs_fifo_rdreq),                          // Read Data Enable
     .r_pempty(2'd2),                                 // FIFO partially empty threshold
     .r_pfull(2'd3),                                  // FIFO partially full threshold
     .rd_data(prs_data_out),                          // Read Data Out
     .rd_numdata(prs_rd_numdata),                     // Number of Data available in Read clock
     .wr_numdata(),                                   // Number of Data available in Write clock
     .wr_empty(),                                     // FIFO Empty
     .wr_pempty(),                                    // FIFO Partial Empty
     .wr_full(),                                      // FIFO Full
     .wr_pfull(),                                     // FIFO Parial Full
     .rd_empty(prs_fifo_empty),                       // FIFO Empty
     .rd_pempty(),                                    // FIFO Partial Empty
     .rd_full(),                                      // FIFO Full
     .rd_pfull()                                      // FIFO Partial Full
     );

//********************************************************************
// READ CLOCK DOMAIN
//********************************************************************
   assign prs_fifo_rdreq = ~(prs_fifo_empty | (prs_rd_numdata == 3'd1 & ~prs_fifo_empty & prs_fifo_rdreq_q));

  always @ (posedge clk500) begin //negedge rst_clk500_n or 
    if (rst_clk500_n == 0) begin
      prs_event_valid_o    <= '0;
      prs_fifo_rdreq_q     <= '0;
    end
    else begin
      prs_event_valid_o    <= '0;
      prs_fifo_rdreq_q     <= prs_fifo_rdreq;
      if (prs_fifo_rdreq_q) begin
        prs_event_valid_o    <= prs_data_out.prs_event_valid;
      end
    end
  end

  always @ (posedge clk500) begin
    prs_event_func_o     <= prs_data_out.prs_event_func;
    prs_event_o          <= prs_data_out.prs_event;
  end
endmodule