// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_avmm_cii_if
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module  intel_pcie_whr_gen4x8_avmm_cii_if
 #(
     parameter intended_device_family = "Stratix 10"
  )
  (
   input               clk500,
   input               rst_clk500_n,
   input               clk250,
   input               rst_clk250_n,
   input               cii_req_i,
   input               cii_hdr_poisoned_i,
   input  logic [3:0]  cii_hdr_first_be_i,
   input  logic [2:0]  cii_func_num_i,
   input  logic        cii_wr_i,
   input  logic [9:0]  cii_addr_i,
   input  logic [31:0] cii_dout_i,
   input  logic        cii_wr_vf_active_i,
   input  logic [10:0] cii_vf_num_i,
   input  logic        cii_override_en_i,
   input  logic [31:0] cii_override_din_i,
   input  logic        cii_halt_i,
   output logic        cii_req_o,
   output logic        cii_hdr_poisoned_o,
   output logic [3:0]  cii_hdr_first_be_o,
   output logic [2:0]  cii_func_num_o,
   output logic        cii_wr_o,
   output logic [9:0]  cii_addr_o,
   output logic [31:0] cii_dout_o,
   output logic        cii_wr_vf_active_o,
   output logic [10:0] cii_vf_num_o,
   output logic        cii_override_en_o,
   output logic [31:0] cii_override_din_o,
   output logic        cii_halt_o
  );

//********************************************************************
// SYNCHRONIZATION OF HIP -> APP SIGNALS
//********************************************************************
  //******************************************************************************
  // DEFINE VARIABLES
  //******************************************************************************
   logic        cii_req_sync;
   logic        load;

   intel_pcie_whr_gen4x8_avmm_sync_bit #(.DWIDTH(1), .RESET_VAL(0))
           req_sync (.clk(clk250), .rst_n(rst_clk250_n), .din(cii_req_i), .dout(cii_req_sync));

  assign load = (cii_req_sync != cii_req_o);

  always @(negedge rst_clk250_n or posedge clk250) begin
    if (rst_clk250_n == 0) begin
      cii_req_o <= '0;
    end
    else begin
      cii_req_o <= cii_req_sync;
      if (load == 1 & cii_req_sync == 1) begin
        cii_hdr_poisoned_o <= cii_hdr_poisoned_i;
        cii_hdr_first_be_o <= cii_hdr_first_be_i;
        cii_func_num_o     <= cii_func_num_i;
        cii_wr_o           <= cii_wr_i;
        cii_addr_o         <= cii_addr_i;
        cii_dout_o         <= cii_dout_i;
        cii_wr_vf_active_o <= cii_wr_vf_active_i;
        cii_vf_num_o       <= cii_vf_num_i;
      end
      else if (load == 1 & cii_req_sync == 0) begin
        cii_hdr_poisoned_o <= '0;
        cii_hdr_first_be_o <= '0;
        cii_func_num_o     <= '0;
        cii_wr_o           <= '0;
        cii_addr_o         <= '0;
        cii_dout_o         <= '0;
        cii_wr_vf_active_o <= '0;
        cii_vf_num_o       <= '0;
      end
     end
  end
//********************************************************************
// SYNCHRONIZATION OF APP -> HIP SIGNALS
//********************************************************************
  //******************************************************************************
  // DEFINE VARIABLES
  //******************************************************************************

  logic        cii_override_en_q;
  logic [31:0] cii_override_din_q;
  logic        cii_halt_out;

  intel_pcie_whr_gen4x8_avmm_sync_pulse cii_hlt_sync (.wr_clk(clk250), .rd_clk(clk500), .wr_rst_n(rst_clk250_n),
                                                                  .rd_rst_n(rst_clk500_n), .din(~cii_halt_i), .dout(cii_halt_out));

  always @(negedge rst_clk250_n or posedge clk250) begin
    if (rst_clk250_n == 1'b0) begin
      cii_override_en_q  <= '0;
    end
    else begin
      if (cii_halt_i == 0) begin
        cii_override_en_q  <= cii_override_en_i;
        cii_override_din_q <= cii_override_din_i;
      end
    end
  end

  always @(negedge rst_clk500_n or posedge clk500) begin
    if (rst_clk500_n == 1'b0) begin
      cii_halt_o        <= '1;
      cii_override_en_o <= '0;
    end
    else begin
      if (cii_halt_out == 1'b1) begin
        cii_halt_o         <= 1'b0;
        cii_override_en_o  <= cii_override_en_q;
        cii_override_din_o <= cii_override_din_q;
      end
      else begin
        cii_halt_o        <= 1'b1;
        cii_override_en_o <= 1'b0;
      end
    end
  end

endmodule
