// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

//-----------------------------------------------------------------------------
// Description   : WHR PCI Express CFG Requests Register Address Decoder.
//                 Creates Chip Select if register address matches.
//-----------------------------------------------------------------------------
// File          : intel_pcie_virtio_addrdc.v
// Author        : Intel Corporation
//-----------------------------------------------------------------------------
//

module intel_pcie_virtio_addrdc #
    (
        parameter REG_ADDR_WIDTH                   = 10,   // The double word register address.
        parameter VIRTIO_CMN_CONFIG_REG_ADDR       = 8'h12,   // Reg Address
        parameter VIRTIO_NOTIFICATION_REG_ADDR     = 8'h16,
        parameter VIRTIO_ISR_STATUS_REG_ADDR       = 8'h2F,
        parameter VIRTIO_DEV_SPECIFIC_REG_ADDR     = 8'h33,
        parameter VIRTIO_PCICFG_ACCESS_REG_ADDR    = 8'h37
    )
    (
        input                                 clk,
        input                                 rstn,
        // CII Interface
        input                                 cii_req_i,       // Indicates the CFG request is intercepted and all the other CII signals are valid.
        input  [REG_ADDR_WIDTH-1:0]           cii_addr_i,      // The dword register address in the received TLP header on CII.
        // VirtIO Common Config Capability
        output reg                 virtio_cmncfg_header_cs_o,
        output reg                 virtio_cmncfg_bar_cs_o,
        output reg                 virtio_cmncfg_baroffset_cs_o,
        output reg                 virtio_cmncfg_length_cs_o,
        // VirtIO Notifications Capability
        output reg                 virtio_notifications_header_cs_o,
        output reg                 virtio_notifications_bar_cs_o,
        output reg                 virtio_notifications_baroffset_cs_o,
        output reg                 virtio_notifications_length_cs_o,
        output reg                 virtio_notifications_notifyoff_cs_o,
        // VirtIO ISR Status Capability
        output reg                 virtio_isrstatus_header_cs_o,
        output reg                 virtio_isrstatus_bar_cs_o,
        output reg                 virtio_isrstatus_baroffset_cs_o,
        output reg                 virtio_isrstatus_length_cs_o,
        // VirtIO Device Specific Capability
        output reg                 virtio_devspecific_header_cs_o,
        output reg                 virtio_devspecific_bar_cs_o,
        output reg                 virtio_devspecific_baroffset_cs_o,
        output reg                 virtio_devspecific_length_cs_o,
        // VirtIO PCI Config Access Capability
        output reg                 virtio_pcicfgaccess_header_cs_o,
        output reg                 virtio_pcicfgaccess_bar_cs_o,
        output reg                 virtio_pcicfgaccess_baroffset_cs_o,
        output reg                 virtio_pcicfgaccess_length_cs_o,
        output reg                 virtio_pcicfgaccess_cfgdata_cs_o,

        output                     noreg_selected_o
    );

    // Signal Declaration
    wire  anyreg_sel;
    reg   noreg_sel_q;
    reg   cii_req_q;
    // VirtIO Cap Address
    wire [7:0] virtio_cmn_config_bir_addr;
    wire [7:0] virtio_cmn_config_bar_offset_addr;
    wire [7:0] virtio_cmn_config_length_addr;
    wire [7:0] virtio_notification_bir_addr;
    wire [7:0] virtio_notification_bar_offset_addr;
    wire [7:0] virtio_notification_length_addr;
    wire [7:0] virtio_notify_off_multiplier_addr;
    wire [7:0] virtio_isr_status_bir_addr;
    wire [7:0] virtio_isr_status_bar_offset_addr;
    wire [7:0] virtio_isr_status_length_addr;
    wire [7:0] virtio_dev_specific_bir_addr;
    wire [7:0] virtio_dev_specific_bar_offset_addr;
    wire [7:0] virtio_dev_specific_length_addr;
    wire [7:0] virtio_pcicfg_access_bir_addr;
    wire [7:0] virtio_pcicfg_access_bar_offset_addr;
    wire [7:0] virtio_pcicfg_access_length_addr;
    wire [7:0] virtio_pcicfg_access_cfgdata_addr;
    // Address Comparation Signals
    wire [1:0] virtio_cmncfg_header_cs;
    wire [1:0] virtio_cmncfg_bar_cs;
    wire [1:0] virtio_cmncfg_baroffset_cs;
    wire [1:0] virtio_cmncfg_length_cs;
    wire [1:0] virtio_notifications_header_cs;
    wire [1:0] virtio_notifications_bar_cs;
    wire [1:0] virtio_notifications_baroffset_cs;
    wire [1:0] virtio_notifications_length_cs;
    wire [1:0] virtio_notifications_notifyoff_cs;
    wire [1:0] virtio_isrstatus_header_cs;
    wire [1:0] virtio_isrstatus_bar_cs;
    wire [1:0] virtio_isrstatus_baroffset_cs;
    wire [1:0] virtio_isrstatus_length_cs;
    wire [1:0] virtio_devspecific_header_cs;
    wire [1:0] virtio_devspecific_bar_cs;
    wire [1:0] virtio_devspecific_baroffset_cs;
    wire [1:0] virtio_devspecific_length_cs;
    wire [1:0] virtio_pcicfgaccess_header_cs;
    wire [1:0] virtio_pcicfgaccess_bar_cs;
    wire [1:0] virtio_pcicfgaccess_baroffset_cs;
    wire [1:0] virtio_pcicfgaccess_length_cs;
    wire [1:0] virtio_pcicfgaccess_cfgdata_cs;

    // Common Config Capability
    assign virtio_cmn_config_bir_addr            = VIRTIO_CMN_CONFIG_REG_ADDR + 1;  // Bar Indicator
    assign virtio_cmn_config_bar_offset_addr     = VIRTIO_CMN_CONFIG_REG_ADDR + 2;
    assign virtio_cmn_config_length_addr         = VIRTIO_CMN_CONFIG_REG_ADDR + 3;
    // Notification Capability
    assign virtio_notification_bir_addr          = VIRTIO_NOTIFICATION_REG_ADDR + 1;
    assign virtio_notification_bar_offset_addr   = VIRTIO_NOTIFICATION_REG_ADDR + 2;
    assign virtio_notification_length_addr       = VIRTIO_NOTIFICATION_REG_ADDR + 3;
    assign virtio_notify_off_multiplier_addr     = VIRTIO_NOTIFICATION_REG_ADDR + 4;
    // ISR STATUS Capability
    assign virtio_isr_status_bir_addr            = VIRTIO_ISR_STATUS_REG_ADDR + 1;
    assign virtio_isr_status_bar_offset_addr     = VIRTIO_ISR_STATUS_REG_ADDR + 2;
    assign virtio_isr_status_length_addr         = VIRTIO_ISR_STATUS_REG_ADDR + 3;
    // Device Specific Capability
    assign virtio_dev_specific_bir_addr          = VIRTIO_DEV_SPECIFIC_REG_ADDR + 1;
    assign virtio_dev_specific_bar_offset_addr   = VIRTIO_DEV_SPECIFIC_REG_ADDR + 2;
    assign virtio_dev_specific_length_addr       = VIRTIO_DEV_SPECIFIC_REG_ADDR + 3;
    // PCI Config Access Capability
    assign virtio_pcicfg_access_bir_addr         = VIRTIO_PCICFG_ACCESS_REG_ADDR + 1;
    assign virtio_pcicfg_access_bar_offset_addr  = VIRTIO_PCICFG_ACCESS_REG_ADDR + 2;
    assign virtio_pcicfg_access_length_addr      = VIRTIO_PCICFG_ACCESS_REG_ADDR + 3;
    assign virtio_pcicfg_access_cfgdata_addr     = VIRTIO_PCICFG_ACCESS_REG_ADDR + 4;

    // Create Chip Select if Register Address Matches
    assign virtio_cmncfg_header_cs            = {(cii_addr_i[7:4] == VIRTIO_CMN_CONFIG_REG_ADDR[7:4]) , (cii_addr_i[3:0] == VIRTIO_CMN_CONFIG_REG_ADDR[3:0])};
    assign virtio_cmncfg_bar_cs               = {(cii_addr_i[7:4] == virtio_cmn_config_bir_addr[7:4]) , (cii_addr_i[3:0] == virtio_cmn_config_bir_addr[3:0])};
    assign virtio_cmncfg_baroffset_cs         = {(cii_addr_i[7:4] == virtio_cmn_config_bar_offset_addr[7:4]) , (cii_addr_i[3:0] == virtio_cmn_config_bar_offset_addr[3:0])};
    assign virtio_cmncfg_length_cs            = {(cii_addr_i[7:4] == virtio_cmn_config_length_addr[7:4]) , (cii_addr_i[3:0] == virtio_cmn_config_length_addr[3:0])};
    assign virtio_notifications_header_cs     = {(cii_addr_i[7:4] == VIRTIO_NOTIFICATION_REG_ADDR[7:4]) , (cii_addr_i[3:0] == VIRTIO_NOTIFICATION_REG_ADDR[3:0])};
    assign virtio_notifications_bar_cs        = {(cii_addr_i[7:4] == virtio_notification_bir_addr[7:4]) , (cii_addr_i[3:0] == virtio_notification_bir_addr[3:0])};
    assign virtio_notifications_baroffset_cs  = {(cii_addr_i[7:4] == virtio_notification_bar_offset_addr[7:4]) , (cii_addr_i[3:0] == virtio_notification_bar_offset_addr[3:0])};
    assign virtio_notifications_length_cs     = {(cii_addr_i[7:4] == virtio_notification_length_addr[7:4]) , (cii_addr_i[3:0] == virtio_notification_length_addr[3:0])};
    assign virtio_notifications_notifyoff_cs  = {(cii_addr_i[7:4] == virtio_notify_off_multiplier_addr[7:4]) , (cii_addr_i[3:0] == virtio_notify_off_multiplier_addr[3:0])};
    assign virtio_isrstatus_header_cs         = {(cii_addr_i[7:4] == VIRTIO_ISR_STATUS_REG_ADDR[7:4]) , (cii_addr_i[3:0] == VIRTIO_ISR_STATUS_REG_ADDR[3:0])};
    assign virtio_isrstatus_bar_cs            = {(cii_addr_i[7:4] == virtio_isr_status_bir_addr[7:4]) , (cii_addr_i[3:0] == virtio_isr_status_bir_addr[3:0])};
    assign virtio_isrstatus_baroffset_cs      = {(cii_addr_i[7:4] == virtio_isr_status_bar_offset_addr[7:4]) , (cii_addr_i[3:0] == virtio_isr_status_bar_offset_addr[3:0])};
    assign virtio_isrstatus_length_cs         = {(cii_addr_i[7:4] == virtio_isr_status_length_addr[7:4]) , (cii_addr_i[3:0] == virtio_isr_status_length_addr[3:0])};
    assign virtio_devspecific_header_cs       = {(cii_addr_i[7:4] == VIRTIO_DEV_SPECIFIC_REG_ADDR[7:4]) , (cii_addr_i[3:0] == VIRTIO_DEV_SPECIFIC_REG_ADDR[3:0])};
    assign virtio_devspecific_bar_cs          = {(cii_addr_i[7:4] == virtio_dev_specific_bir_addr[7:4]) , (cii_addr_i[3:0] == virtio_dev_specific_bir_addr[3:0])};
    assign virtio_devspecific_baroffset_cs    = {(cii_addr_i[7:4] == virtio_dev_specific_bar_offset_addr[7:4]) , (cii_addr_i[3:0] == virtio_dev_specific_bar_offset_addr[3:0])};
    assign virtio_devspecific_length_cs       = {(cii_addr_i[7:4] == virtio_dev_specific_length_addr[7:4]) , (cii_addr_i[3:0] == virtio_dev_specific_length_addr[3:0])};
    assign virtio_pcicfgaccess_header_cs      = {(cii_addr_i[7:4] == VIRTIO_PCICFG_ACCESS_REG_ADDR[7:4]) , (cii_addr_i[3:0] == VIRTIO_PCICFG_ACCESS_REG_ADDR[3:0])};
    assign virtio_pcicfgaccess_bar_cs         = {(cii_addr_i[7:4] == virtio_pcicfg_access_bir_addr[7:4]) , (cii_addr_i[3:0] == virtio_pcicfg_access_bir_addr[3:0])};
    assign virtio_pcicfgaccess_baroffset_cs   = {(cii_addr_i[7:4] == virtio_pcicfg_access_bar_offset_addr[7:4]) , (cii_addr_i[3:0] == virtio_pcicfg_access_bar_offset_addr[3:0])};
    assign virtio_pcicfgaccess_length_cs      = {(cii_addr_i[7:4] == virtio_pcicfg_access_length_addr[7:4]) , (cii_addr_i[3:0] == virtio_pcicfg_access_length_addr[3:0])};
    assign virtio_pcicfgaccess_cfgdata_cs     = {(cii_addr_i[7:4] == virtio_pcicfg_access_cfgdata_addr[7:4]) , (cii_addr_i[3:0] == virtio_pcicfg_access_cfgdata_addr[3:0])};

    always @ (posedge clk) begin
        //if (~rstn) begin
        //    virtio_cmncfg_header_cs_o           <= '0;
        //    virtio_cmncfg_bar_cs_o              <= '0;
        //    virtio_cmncfg_baroffset_cs_o        <= '0;
        //    virtio_cmncfg_length_cs_o           <= '0;
		//
        //    virtio_notifications_header_cs_o    <= '0;
        //    virtio_notifications_bar_cs_o       <= '0;
        //    virtio_notifications_baroffset_cs_o <= '0;
        //    virtio_notifications_length_cs_o    <= '0;
        //    virtio_notifications_notifyoff_cs_o <= '0;
		//
        //    virtio_isrstatus_header_cs_o        <= '0;
        //    virtio_isrstatus_bar_cs_o           <= '0;
        //    virtio_isrstatus_baroffset_cs_o     <= '0;
        //    virtio_isrstatus_length_cs_o        <= '0;
		//
        //    virtio_devspecific_header_cs_o      <= '0;
        //    virtio_devspecific_bar_cs_o         <= '0;
        //    virtio_devspecific_baroffset_cs_o   <= '0;
        //    virtio_devspecific_length_cs_o      <= '0;
		//
        //    virtio_pcicfgaccess_header_cs_o     <= '0;
        //    virtio_pcicfgaccess_bar_cs_o        <= '0;
        //    virtio_pcicfgaccess_baroffset_cs_o  <= '0;
        //    virtio_pcicfgaccess_length_cs_o     <= '0;
        //    virtio_pcicfgaccess_cfgdata_cs_o    <= '0;
        //end
        //else begin
            // Chip Select Hit for VirtIO CMN CFG Capability
            virtio_cmncfg_header_cs_o           <= (virtio_cmncfg_header_cs == 2'b11);
            virtio_cmncfg_bar_cs_o              <= (virtio_cmncfg_bar_cs == 2'b11);
            virtio_cmncfg_baroffset_cs_o        <= (virtio_cmncfg_baroffset_cs == 2'b11);
            virtio_cmncfg_length_cs_o           <= (virtio_cmncfg_length_cs == 2'b11);
            // Chip Select Hit for VirtIO NOTIFICATIONS Capability
            virtio_notifications_header_cs_o    <= (virtio_notifications_header_cs == 2'b11);
            virtio_notifications_bar_cs_o       <= (virtio_notifications_bar_cs == 2'b11);
            virtio_notifications_baroffset_cs_o <= (virtio_notifications_baroffset_cs == 2'b11);
            virtio_notifications_length_cs_o    <= (virtio_notifications_length_cs == 2'b11);
            virtio_notifications_notifyoff_cs_o <= (virtio_notifications_notifyoff_cs == 2'b11);
            // Chip Select Hit for VirtIO ISR Capability
            virtio_isrstatus_header_cs_o        <= (virtio_isrstatus_header_cs == 2'b11);
            virtio_isrstatus_bar_cs_o           <= (virtio_isrstatus_bar_cs == 2'b11);
            virtio_isrstatus_baroffset_cs_o     <= (virtio_isrstatus_baroffset_cs == 2'b11);
            virtio_isrstatus_length_cs_o        <= (virtio_isrstatus_length_cs == 2'b11);
            // Chip Select Hit for VirtIO DEVICE SPECIFIC Capability
            virtio_devspecific_header_cs_o      <= (virtio_devspecific_header_cs == 2'b11);
            virtio_devspecific_bar_cs_o         <= (virtio_devspecific_bar_cs == 2'b11);
            virtio_devspecific_baroffset_cs_o   <= (virtio_devspecific_baroffset_cs == 2'b11);
            virtio_devspecific_length_cs_o      <= (virtio_devspecific_length_cs == 2'b11);
            // Chip Select Hit for VirtIO PCI CFG ACCESS Capability
            virtio_pcicfgaccess_header_cs_o     <= (virtio_pcicfgaccess_header_cs == 2'b11);
            virtio_pcicfgaccess_bar_cs_o        <= (virtio_pcicfgaccess_bar_cs == 2'b11);
            virtio_pcicfgaccess_baroffset_cs_o  <= (virtio_pcicfgaccess_baroffset_cs == 2'b11);
            virtio_pcicfgaccess_length_cs_o     <= (virtio_pcicfgaccess_length_cs == 2'b11);
            virtio_pcicfgaccess_cfgdata_cs_o    <= (virtio_pcicfgaccess_cfgdata_cs == 2'b11);
        //end
    end

    assign anyreg_sel  = virtio_cmncfg_header_cs_o | virtio_cmncfg_bar_cs_o |
                         virtio_cmncfg_baroffset_cs_o | virtio_cmncfg_length_cs_o |
                         virtio_notifications_header_cs_o | virtio_notifications_bar_cs_o | virtio_notifications_baroffset_cs_o |
                         virtio_notifications_length_cs_o | virtio_notifications_notifyoff_cs_o |
                         virtio_isrstatus_header_cs_o | virtio_isrstatus_bar_cs_o |
                         virtio_isrstatus_baroffset_cs_o| virtio_isrstatus_length_cs_o |
                         virtio_devspecific_header_cs_o | virtio_devspecific_bar_cs_o |
                         virtio_devspecific_baroffset_cs_o | virtio_devspecific_length_cs_o |
                         virtio_pcicfgaccess_header_cs_o | virtio_pcicfgaccess_bar_cs_o | virtio_pcicfgaccess_baroffset_cs_o |
                         virtio_pcicfgaccess_length_cs_o | virtio_pcicfgaccess_cfgdata_cs_o;

    // If No Reg is selected always return Zero.
    // By default return data is set to "Zero"
    always @ (posedge clk) begin
        if (~rstn) begin
            noreg_sel_q <= 1'b0;
            cii_req_q   <= '0; 
        end
        else begin
            cii_req_q   <= cii_req_i;
            noreg_sel_q <= cii_req_q & ~anyreg_sel;
        end
    end

    assign noreg_selected_o = noreg_sel_q;

endmodule
