// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

//-----------------------------------------------------------------------------
// Description   : WHR PCI Express VirtIO CFG Read Requests returned 
//                 data flow control and CII outputs State Machine
//-----------------------------------------------------------------------------
// File          : intel_pcie_virtio_dataflow.v
// Author        : Intel Corporation
//-----------------------------------------------------------------------------
//

module intel_pcie_virtio_dataflow
    (
        input                      clk,
        input                      rstn,
        // Read Data from VirtIO Common Config Capability
        input  [31:0]              virtio_cmncfg_header_data_i,
        input  [31:0]              virtio_cmncfg_bar_data_i,
        input  [31:0]              virtio_cmncfg_baroffset_data_i,
        input  [31:0]              virtio_cmncfg_length_data_i,
        // Read Data from VirtIO Notifications Capability
        input  [31:0]              virtio_notifications_header_data_i,
        input  [31:0]              virtio_notifications_bar_data_i,
        input  [31:0]              virtio_notifications_baroffset_data_i,
        input  [31:0]              virtio_notifications_length_data_i,
        input  [31:0]              virtio_notifications_notifyoff_data_i,
        // Read Data from VirtIO ISR Status Capability
        input  [31:0]              virtio_isrstatus_header_data_i,
        input  [31:0]              virtio_isrstatus_bar_data_i,
        input  [31:0]              virtio_isrstatus_baroffset_data_i,
        input  [31:0]              virtio_isrstatus_length_data_i,
        // Read Data from VirtIO Device Specific Capability
        input  [31:0]              virtio_devspecific_header_data_i,
        input  [31:0]              virtio_devspecific_bar_data_i,
        input  [31:0]              virtio_devspecific_baroffset_data_i,
        input  [31:0]              virtio_devspecific_length_data_i,
        // Read Data from VirtIO PCI Config Access Capability
        input  [31:0]              virtio_pcicfgaccess_header_data_i,
        input  [31:0]              virtio_pcicfgaccess_bar_data_i,
        input  [31:0]              virtio_pcicfgaccess_baroffset_data_i,
        input  [31:0]              virtio_pcicfgaccess_length_data_i,
        input  [31:0]              virtio_pcicfgaccess_cfgdata_data_i,
        // CFG Req Ack
        input                      virtio_cfgwr_req_ack_i,
        input                      virtio_cfgrd_req_ack_i,
        // CII Interface
        output                     cii_override_en_o,     // When your application logic asserts this output, the HIP overrides the CfgWr payload 
                                                          // or CfgRd completion using the data supplied by your application logic on cii_override_din.
        output [31:0]              cii_override_din_o,
        output                     cii_halt_o,            // When cii_halt_p0/1 is asserted, the HIP halts the processing of CFG reqs for the PCIe config space regs.
        // Address Decoder Interface
        input                      noregbank_selected_i   // ~address hit, halt=0, override=0, no change to cfgreq
        
    );
    
    localparam IDLE                 = 2'd0;
    localparam CFGREQ               = 2'd1;
    localparam NOHIT                = 2'd2;
    
    // Signal Declaration
    genvar i;

    reg  [1:0 ]          cii_ctrl_state;
    reg                  cii_override_en_reg;
    wire                 cii_override_en;
    reg  [31:0]          cii_override_din_reg;
    wire [31:0]          cii_override_din;
    reg                  cii_halt_reg;
    wire                 cii_halt;
    
    reg                  virtio_cfgwr_req_ack_reg;  // used for edge detect
    reg                  virtio_cfgrd_req_ack_reg;
    reg                  noregbank_selected_reg;
        
    // Any Read Data from Capabilities to override CFG RD CPLD payload
    assign cii_override_din       = virtio_cmncfg_header_data_i | virtio_cmncfg_bar_data_i | virtio_cmncfg_baroffset_data_i | virtio_cmncfg_length_data_i |
                                    virtio_notifications_header_data_i | virtio_notifications_bar_data_i | virtio_notifications_baroffset_data_i | 
                                    virtio_notifications_length_data_i | virtio_notifications_notifyoff_data_i |
                                    virtio_isrstatus_header_data_i | virtio_isrstatus_bar_data_i | virtio_isrstatus_baroffset_data_i | virtio_isrstatus_length_data_i |
                                    virtio_devspecific_header_data_i | virtio_devspecific_bar_data_i | virtio_devspecific_baroffset_data_i | virtio_devspecific_length_data_i |
                                    virtio_pcicfgaccess_header_data_i | virtio_pcicfgaccess_bar_data_i | virtio_pcicfgaccess_baroffset_data_i | 
                                    virtio_pcicfgaccess_length_data_i | virtio_pcicfgaccess_cfgdata_data_i;                             
    
    always @ (posedge clk or negedge rstn) begin
        if (~rstn) begin 
            virtio_cfgwr_req_ack_reg        <= 1'b0;
            virtio_cfgrd_req_ack_reg        <= 1'b0;
            noregbank_selected_reg          <= 1'b0;
        end    
        else begin
            virtio_cfgwr_req_ack_reg        <= virtio_cfgwr_req_ack_i;
            virtio_cfgrd_req_ack_reg        <= virtio_cfgrd_req_ack_i;
            noregbank_selected_reg          <= noregbank_selected_i;
        end
    end  
     
    always @ (posedge clk or negedge rstn) begin
        if (~rstn) begin 
            cii_override_din_reg        <= {32{1'b0}};
        end    
        else if (virtio_cfgrd_req_ack_i) begin
            cii_override_din_reg        <= cii_override_din;
        end
    end  
    
    always @ (posedge clk or negedge rstn) begin
        // Initialize override_en and halt registers
        if (~rstn) begin
            cii_override_en_reg  <= 1'b0; 
            cii_halt_reg         <= 1'b1;   // tie to 1 as long as CII is enabled
            cii_ctrl_state       <= IDLE;
        end
        // State Machine
        else begin
            case (cii_ctrl_state)
                IDLE :
                begin
                    cii_override_en_reg  <= 1'b0; 
                    cii_halt_reg         <= 1'b1;  // this will hold req if there comes a req
                    // Detect rising edge
                    if (noregbank_selected_i & !noregbank_selected_reg) begin
                        cii_ctrl_state       <= NOHIT;
                    end
                    // If there's a virtio reg hit, and it's wr req to a RO reg, same as NOHIT
                    // Will do nothing and release the halt. On HIP side, SC will be sent back.
                    else if (!noregbank_selected_i & (virtio_cfgwr_req_ack_i & !virtio_cfgwr_req_ack_reg)) begin  // rising edge
                        cii_ctrl_state       <= NOHIT;
                    end
                    // If there's a legal virtio req hit, then detect ack rising edge
                    else if (!noregbank_selected_i & ((virtio_cfgwr_req_ack_i & !virtio_cfgwr_req_ack_reg) | (virtio_cfgrd_req_ack_i &  !virtio_cfgrd_req_ack_reg))) begin  // rising edge
                        cii_ctrl_state       <= CFGREQ;
                    end
                    // Wait for next req
                    else begin
                        cii_ctrl_state       <= IDLE;
                    end
                end
                CFGREQ:
                begin
                    if (virtio_cfgwr_req_ack_i) begin  
                        // De-assert halt for one cycle, but won't assert override_en
                        cii_override_en_reg  <= 1'b0; 
                        cii_halt_reg         <= 1'b0;
                    end
                    else if (virtio_cfgrd_req_ack_i) begin  
                        // De-assert halt for one cycle, and also assert override_en to override payload of cfgrd cpld
                        cii_override_en_reg  <= 1'b1; 
                        cii_halt_reg         <= 1'b0;
                    end
                    cii_ctrl_state       <= IDLE;  // Wait for next req
                end
                NOHIT:
                begin
                    // De-assert halt for one cycle to release non-virtio reqs
                    cii_override_en_reg  <= 1'b0; 
                    cii_halt_reg         <= 1'b0;
                    cii_ctrl_state       <= IDLE;  // Wait for next req
                end
                default:
                begin
                    cii_override_en_reg  <= 1'b0; 
                    cii_halt_reg         <= 1'b1;  // tie to 1 as long as CII is enabled
                    cii_ctrl_state       <= IDLE;
                end
            endcase    
        end
    end

        
    assign cii_override_en_o            = cii_override_en_reg;
    assign cii_override_din_o           = cii_override_din_reg;
    assign cii_halt_o                   = cii_halt_reg;
    
endmodule
