// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_credit_if
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module intel_pcie_whr_gen4x8_credit_if
 #(
     parameter intended_device_family = "Stratix 10"
  )
  (
   input                clk500,
   input                rst_clk500_n,
   input                clk250,
   input                rst_clk250_n,
   input  logic [15:0]  tx_cdts_limit_i,
   input  logic [2:0]   tx_cdts_limit_tdm_idx_i,
   input  logic [11:0]  rx_buffer_limit_i,
   input  logic [1:0]   rx_buffer_limit_tdm_idx_i,
   output logic [15:0]  tx_cdts_limit_o,
   output logic [2:0]   tx_cdts_limit_tdm_idx_o,
   output logic [11:0]  rx_buffer_limit_o,
   output logic [1:0]   rx_buffer_limit_tdm_idx_o
  );

//********************************************************************
// TX CREDIT CDC
//********************************************************************
  //******************************************************************************
  // DEFINE VARIABLES
  //******************************************************************************
  localparam PH     = 3'b000,
             NPH    = 3'b001,
             CPLH   = 3'b010,
             PD     = 3'b100,
             NPD    = 3'b101,
             CPLD   = 3'b110,
             IDLE   = 3'b111;
 
  logic [15:0] tx_cdts_limit_q, tx_cdts_limit_qq;                 
  logic [2:0]  tx_cdts_limit_tdm_idx_q, tx_cdts_limit_tdm_idx_qq; 
  logic tx_credit_fifo_empty;
  logic tx_credit_fifo_wrreq;
  logic tx_credit_fifo_rdreq, tx_credit_fifo_rdreq_q;
  logic [2:0] tx_state;

  typedef struct packed {
    logic [15:0]  tx_cdts_limit;
    logic [2:0]   tx_cdts_limit_tdm_idx;
  } tx_credit_data_t;

  tx_credit_data_t tx_credit_in, tx_credit_out;
  logic [3:0] tx_credit_rd_numdata;

  //********************************************************************
  // TX CREDIT CDC
  //********************************************************************
  always @(posedge clk500) begin
    tx_cdts_limit_q          <= tx_cdts_limit_i;
    tx_cdts_limit_tdm_idx_q  <= tx_cdts_limit_tdm_idx_i;
    tx_cdts_limit_qq         <= tx_cdts_limit_q;
    tx_cdts_limit_tdm_idx_qq <= tx_cdts_limit_tdm_idx_q;
  end

  always @(posedge clk500) begin
    if(rst_clk500_n == 0) begin
      tx_state <= IDLE;
      tx_credit_fifo_wrreq <= 1'b0;
    end
    else begin
      case(tx_state) 
      IDLE: begin
        if(tx_cdts_limit_tdm_idx_q == PH) begin
          tx_state <= PH;
          tx_credit_fifo_wrreq <= 1'b1;
        end
        else begin
          tx_state <= IDLE;
          tx_credit_fifo_wrreq <= 1'b0;
        end
      end
      PH: begin
        if(tx_cdts_limit_tdm_idx_q == CPLH) begin
          tx_state <= CPLH;
          tx_credit_fifo_wrreq <= 1'b1;
        end
        else begin
          tx_state <= PH;
          tx_credit_fifo_wrreq <= 1'b0;
        end
      end
      CPLH: begin
        if(tx_cdts_limit_tdm_idx_q == NPD) begin
          tx_state <= NPD;
          tx_credit_fifo_wrreq <= 1'b1;
        end
        else begin
          tx_state <= CPLH;
          tx_credit_fifo_wrreq <= 1'b0;
        end
      end
      NPD: begin
        if(tx_cdts_limit_tdm_idx_q == NPH) begin
          tx_state <= NPH;
          tx_credit_fifo_wrreq <= 1'b1;
        end
        else begin
          tx_state <= NPD;
          tx_credit_fifo_wrreq <= 1'b0;
        end
      end
      NPH: begin
        if(tx_cdts_limit_tdm_idx_q == PD) begin
          tx_state <= PD;
          tx_credit_fifo_wrreq <= 1'b1;
        end
        else begin
          tx_state <= NPH;
          tx_credit_fifo_wrreq <= 1'b0;
        end
      end
      PD: begin
        if(tx_cdts_limit_tdm_idx_q == CPLD) begin
          tx_state <= CPLD;
          tx_credit_fifo_wrreq <= 1'b1;
        end
        else begin
          tx_state <= PD;
          tx_credit_fifo_wrreq <= 1'b0;
        end
      end
      CPLD: begin
        if(tx_cdts_limit_tdm_idx_q == PH) begin
          tx_state <= PH;
          tx_credit_fifo_wrreq <= 1'b1;
        end
        else begin
          tx_state <= CPLD;
          tx_credit_fifo_wrreq <= 1'b0;
        end
      end
      default: begin
        tx_state <= IDLE;
        tx_credit_fifo_wrreq <= 1'b0;
      end
      endcase
    end
  end

  assign tx_credit_in = {tx_cdts_limit_qq, tx_cdts_limit_tdm_idx_qq};

  intel_pcie_whr_gen4x8_dcfifo
  #(
    .DWIDTH ($bits(tx_credit_in)),
    .AWIDTH (3),
    .intended_device_family(intended_device_family)
   ) tx_credit_data_fifo
  (
   .wr_rst_n(rst_clk500_n),                         // Write Domain Active low Reset
   .wr_clk(clk500),                                 // Write Domain Clock
   .wr_en(tx_credit_fifo_wrreq),                    // Write Data Enable
   .wr_data(tx_credit_in),                          // Write Data In
   .rd_rst_n(rst_clk250_n),                         // Read Domain Active low Reset
   .rd_clk(clk250),                                 // Read Domain Clock
   .rd_en(tx_credit_fifo_rdreq),                    // Read Data Enable
   .r_pempty(3'd2),                                 // FIFO partially empty threshold
   .r_pfull(3'd6),                                  // FIFO partially full threshold
   .rd_data(tx_credit_out),                         // Read Data Out
   .rd_numdata(tx_credit_rd_numdata),               // Number of Data available in Read clock
   .wr_numdata(),                                   // Number of Data available in Write clock
   .wr_empty(),                                     // FIFO Empty
   .wr_pempty(),                                    // FIFO Partial Empty
   .wr_full(),                                      // FIFO Full
   .wr_pfull(),                                     // FIFO Parial Full
   .rd_empty(tx_credit_fifo_empty),                 // FIFO Empty
   .rd_pempty(),                                    // FIFO Partial Empty
   .rd_full(),                                      // FIFO Full
   .rd_pfull()                                      // FIFO Partial Full
   );

  //********************************************************************
  // READ CLOCK DOMAIN
  //********************************************************************
  always @ (posedge clk250) begin //negedge rst_clk500_n or 
    if (rst_clk250_n == 0) begin
      tx_credit_fifo_rdreq   <= '0;
      tx_credit_fifo_rdreq_q <= '0;
      tx_cdts_limit_o         <= '0;
      tx_cdts_limit_tdm_idx_o <= '0;
    end
    else begin
      tx_credit_fifo_rdreq    <= ~(tx_credit_fifo_empty | (tx_credit_rd_numdata == 4'd1 & ~tx_credit_fifo_empty & tx_credit_fifo_rdreq_q));
      tx_credit_fifo_rdreq_q  <= tx_credit_fifo_rdreq;
      tx_cdts_limit_o         <= tx_credit_out.tx_cdts_limit;
      tx_cdts_limit_tdm_idx_o <= tx_credit_out.tx_cdts_limit_tdm_idx;
    end
  end

//********************************************************************
// RX CREDIT CDC
//********************************************************************
  //******************************************************************************
  // DEFINE VARIABLES
  //******************************************************************************
  localparam PB   = 2'b00,
             NPB  = 2'b01,
             CPLB = 2'b10,
             RSVD = 2'b11;

  logic [11:0] rx_buffer_limit_q; /* synthesis preserve_syn_only */
  logic [1:0]  rx_buffer_limit_tdm_idx_q; /* synthesis preserve_syn_only */
  logic rx_credit_fifo_empty;
  logic rx_credit_fifo_wrreq;
  logic rx_credit_fifo_rdreq, rx_credit_fifo_rdreq_q;

  typedef struct packed {
    logic [11:0]  rx_buffer_limit;
    logic [1:0]   rx_buffer_limit_tdm_idx;
  } rx_credit_data_t;

  rx_credit_data_t rx_credit_in, rx_credit_out;
  logic [2:0] rx_credit_rd_numdata;
  //******************************************************************************
  // FIFO TO SYNCHRONIZE RX CREDIT VALUES
  //******************************************************************************
  always @ (posedge clk250) begin
    rx_buffer_limit_q         <= rx_buffer_limit_i;
    rx_buffer_limit_tdm_idx_q <= rx_buffer_limit_tdm_idx_i;
  end

  assign rx_credit_in = {rx_buffer_limit_q, rx_buffer_limit_tdm_idx_q};
  assign rx_credit_fifo_wrreq = (rx_buffer_limit_tdm_idx_q == PB | rx_buffer_limit_tdm_idx_q == NPB | rx_buffer_limit_tdm_idx_q == CPLB) ? 1'b1 : 1'b0;

  intel_pcie_whr_gen4x8_dcfifo
  #(
    .DWIDTH ($bits(rx_credit_in)),
    .AWIDTH (2),
    .intended_device_family(intended_device_family)
   ) rx_credit_data_fifo
  (
   .wr_rst_n(rst_clk250_n),                         // Write Domain Active low Reset
   .wr_clk(clk250),                                 // Write Domain Clock
   .wr_en(rx_credit_fifo_wrreq),                    // Write Data Enable
   .wr_data(rx_credit_in),                          // Write Data In
   .rd_rst_n(rst_clk500_n),                         // Read Domain Active low Reset
   .rd_clk(clk500),                                 // Read Domain Clock
   .rd_en(rx_credit_fifo_rdreq),                    // Read Data Enable
   .r_pempty(2'd1),                                 // FIFO partially empty threshold
   .r_pfull(2'd2),                                  // FIFO partially full threshold
   .rd_data(rx_credit_out),                         // Read Data Out
   .rd_numdata(rx_credit_rd_numdata),               // Number of Data available in Read clock
   .wr_numdata(),                                   // Number of Data available in Write clock
   .wr_empty(),                                     // FIFO Empty
   .wr_pempty(),                                    // FIFO Partial Empty
   .wr_full(),                                      // FIFO Full
   .wr_pfull(),                                     // FIFO Parial Full
   .rd_empty(rx_credit_fifo_empty),                 // FIFO Empty
   .rd_pempty(),                                    // FIFO Partial Empty
   .rd_full(),                                      // FIFO Full
   .rd_pfull()                                      // FIFO Partial Full
   );

  assign rx_credit_fifo_rdreq = ~(rx_credit_fifo_empty | (rx_credit_rd_numdata == 3'd1 & ~rx_credit_fifo_empty & rx_credit_fifo_rdreq_q));

  always_ff @ (posedge clk500) begin //negedge rst_clk500_n or 
    if (rst_clk500_n == 0) begin
      rx_credit_fifo_rdreq_q    <= '0;
      rx_buffer_limit_o         <= '0;
      rx_buffer_limit_tdm_idx_o <= '0;
    end
    else begin
      rx_credit_fifo_rdreq_q    <= rx_credit_fifo_rdreq;
      rx_buffer_limit_o         <= rx_credit_out.rx_buffer_limit;
      rx_buffer_limit_tdm_idx_o <= rx_credit_out.rx_buffer_limit_tdm_idx;
    end
  end
endmodule
