// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_rx_st_if
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module  intel_pcie_whr_gen4x8_rx_st_if
  #(
    parameter adap_x8_or_x4 = 1,
    parameter intended_device_family = "Stratix 10"
   )
   (
    input  logic                                         clk500,
    input  logic                                         rst_clk500_n,
    input  logic                                         clk250,
    input  logic                                         rst_clk250_n,
    input  logic [(adap_x8_or_x4 ? 255:127) : 0]         rx_st_data_i,
    input  logic [(adap_x8_or_x4 ? 2:1) : 0]             rx_st_empty_i,
    input  logic                                         rx_st_ready_i,
    input  logic                                         rx_st_sop_i,
    input  logic                                         rx_st_eop_i,
    input  logic                                         rx_st_valid_i,
    input  logic [127:0]                                 rx_st_hdr_i,
    input  logic [ 31:0]                                 rx_st_tlp_prfx_i,
    input  logic                                         rx_st_vf_active_i,
    input  logic [  2:0]                                 rx_st_func_num_i,
    input  logic [ 10:0]                                 rx_st_vf_num_i,
    input  logic [  2:0]                                 rx_st_bar_range_i,
    input  logic [(adap_x8_or_x4 ? 31:15) : 0]           rx_st_data_par_i,
    input  logic [ 15:0]                                 rx_st_hdr_par_i,
    input  logic [  3:0]                                 rx_st_tlp_prfx_par_i,
    input  logic                                         rx_par_err_i,
    output logic [(adap_x8_or_x4 ? 256*2-1:128*2-1) : 0] rx_st_data_o,
    output logic [(adap_x8_or_x4 ? 3*2-1:2*2-1) : 0]     rx_st_empty_o,
    output logic                                         rx_st_ready_o,
    output logic [      1:0]                             rx_st_sop_o,
    output logic [      1:0]                             rx_st_eop_o,
    output logic [      1:0]                             rx_st_valid_o,
    output logic [128*2-1:0]                             rx_st_hdr_o,
    output logic [ 32*2-1:0]                             rx_st_tlp_prfx_o,
    output logic [      1:0]                             rx_st_vf_active_o,
    output logic [  3*2-1:0]                             rx_st_func_num_o,
    output logic [ 11*2-1:0]                             rx_st_vf_num_o,
    output logic [  3*2-1:0]                             rx_st_bar_range_o,
    output logic [(adap_x8_or_x4 ? 32*2-1:16*2-1) : 0]   rx_st_data_par_o,
    output logic [ 16*2-1:0]                             rx_st_hdr_par_o,
    output logic [  4*2-1:0]                             rx_st_tlp_prfx_par_o,
    output logic                                         rx_par_err_o
    );

//********************************************************************
// DEFINE VARIABLES
//********************************************************************

  localparam data_width     = adap_x8_or_x4 ? 256*2:128*2;
  localparam empty_width    = adap_x8_or_x4 ? 3*2  :2*2;
  localparam data_par_width = adap_x8_or_x4 ? 32*2 :16*2;

  typedef struct packed {
    logic [data_width/2-1:0]     data;
    logic [empty_width/2-1:0]    empty;
    logic                        sop;
    logic                        eop;
    logic                        valid;
    logic [127:0]                hdr;
    logic [ 31:0]                tlp_prfx;
    logic                        vf_active;
    logic [  2:0]                func_num;
    logic [ 10:0]                vf_num;
    logic [  2:0]                bar_range;
    logic [data_par_width/2-1:0] data_par;
    logic [ 15:0]                hdr_par;
    logic [  3:0]                tlp_prfx_par;
  } rx_data_t;

  rx_data_t   hip_data; /* synthesis preserve_syn_only */
  rx_data_t   high_data;
  rx_data_t   low_data;

  rx_data_t   fifo_wr_high_data;
  rx_data_t   fifo_wr_low_data;

  rx_data_t   fifo_rd_high_data;
  rx_data_t   fifo_rd_low_data;

 // logic       low_data_valid_dup, low_data_eop_dup, high_data_valid_dup; /* synthesis preserve_syn_only */
  logic       rx_fifo_wrreq;
  logic       rx_fifo_rdreq;
  logic       rx_fifo_empty;
  logic       rx_fifo_empty2;
  logic       rx_fifo_full;
  logic [3:0] rx_fifo_rusedw;
  logic       rx_fifo_rdreq_q;
  logic [4:0] rx_rd_numdata;
  logic       rx_st_ready_q, rx_st_ready_i_q;
//********************************************************************
// Two stage synchronizer for rx_ready
//********************************************************************

  intel_pcie_whr_gen4x8_sync_bit #(.DWIDTH(1)) rx_ready_sync (.clk(clk500), .rst_n(rst_clk500_n), .din(rx_st_ready_i_q), .dout(rx_st_ready_q));

always @ (posedge clk250) begin
  if(rst_clk250_n == 0)
    rx_st_ready_i_q <= '0;
  else
    rx_st_ready_i_q <= rx_st_ready_i;
end

always @ (posedge clk500) begin
  if(rst_clk500_n == 0)
    rx_st_ready_o <= '0;
  else
    rx_st_ready_o <= rx_st_ready_q;
end
//********************************************************************
// WRITE CLOCK DOMAIN
//********************************************************************

  always @ (posedge clk500) begin //negedge rst_clk500_n or 
    if (rst_clk500_n == 0) begin
      rx_fifo_wrreq           <= '0;
      hip_data.valid          <= '0;
      low_data.valid          <= '0;
      //low_data_valid_dup      <= '0;
      high_data.valid         <= '0;
      //high_data_valid_dup     <= '0;
      fifo_wr_low_data.valid  <= '0;
      fifo_wr_high_data.valid <= '0;
    end
    else begin
      hip_data.valid        <= rx_st_valid_i;
      high_data.valid       <= hip_data.valid;
      //high_data_valid_dup   <= hip_data.valid;
      rx_fifo_wrreq         <= 1'b0;

      if (!low_data.valid) begin
        low_data.valid     <= high_data.valid;
        //low_data_valid_dup <= high_data.valid;
      end

      if (low_data.valid && (low_data.eop || high_data.valid)) begin
        // Only send data to the application layer when:
        // - We have something in low part
        // AND
        // - Low part is EOP or we have data in high part
        fifo_wr_low_data.valid  <= low_data.valid;
        fifo_wr_high_data.valid <= high_data.valid;
        rx_fifo_wrreq      <= 1'b1;
        low_data.valid     <= 1'b0;
        //low_data_valid_dup <= 1'b0;
      end
    end
  end

  always @ (posedge clk500) begin
      hip_data.data         <= rx_st_data_i;
      hip_data.empty        <= rx_st_empty_i;
      hip_data.sop          <= rx_st_sop_i;
      hip_data.eop          <= rx_st_eop_i;
      hip_data.hdr          <= rx_st_hdr_i;
      hip_data.tlp_prfx     <= rx_st_tlp_prfx_i;
      hip_data.vf_active    <= rx_st_vf_active_i;
      hip_data.func_num     <= rx_st_func_num_i;
      hip_data.vf_num       <= rx_st_vf_num_i;
      hip_data.bar_range    <= rx_st_bar_range_i;
      hip_data.data_par     <= rx_st_data_par_i;
      hip_data.hdr_par      <= rx_st_hdr_par_i;
      hip_data.tlp_prfx_par <= rx_st_tlp_prfx_par_i;

      high_data.data         <= hip_data.data;
      high_data.empty        <= hip_data.empty;
      high_data.sop          <= hip_data.sop;
      high_data.eop          <= hip_data.eop;
      high_data.hdr          <= hip_data.hdr;
      high_data.tlp_prfx     <= hip_data.tlp_prfx;
      high_data.vf_active    <= hip_data.vf_active;
      high_data.func_num     <= hip_data.func_num;
      high_data.vf_num       <= hip_data.vf_num;
      high_data.bar_range    <= hip_data.bar_range;
      high_data.data_par     <= hip_data.data_par;
      high_data.hdr_par      <= hip_data.hdr_par;
      high_data.tlp_prfx_par <= hip_data.tlp_prfx_par;

      if (!low_data.valid) begin
        low_data.data         <= high_data.data;
        low_data.empty        <= high_data.empty;
        low_data.sop          <= high_data.sop;
        low_data.eop          <= high_data.eop;
        //low_data_eop_dup      <= high_data.eop;
        low_data.hdr          <= high_data.hdr;
        low_data.tlp_prfx     <= high_data.tlp_prfx;
        low_data.vf_active    <= high_data.vf_active;
        low_data.func_num     <= high_data.func_num;
        low_data.vf_num       <= high_data.vf_num;
        low_data.bar_range    <= high_data.bar_range;
        low_data.data_par     <= high_data.data_par;
        low_data.hdr_par      <= high_data.hdr_par;
        low_data.tlp_prfx_par <= high_data.tlp_prfx_par;
      end

      if (low_data.valid && (low_data.eop || high_data.valid)) begin
        // Only send data to the application layer when:
        // - We have something in low part
        // AND
        // - Low part is EOP or we have data in high part
        fifo_wr_high_data.data         <= high_data.data;
        fifo_wr_high_data.empty        <= high_data.empty;
        fifo_wr_high_data.sop          <= high_data.sop;
        fifo_wr_high_data.eop          <= high_data.eop;
        fifo_wr_high_data.hdr          <= high_data.hdr;
        fifo_wr_high_data.tlp_prfx     <= high_data.tlp_prfx;
        fifo_wr_high_data.vf_active    <= high_data.vf_active;
        fifo_wr_high_data.func_num     <= high_data.func_num;
        fifo_wr_high_data.vf_num       <= high_data.vf_num;
        fifo_wr_high_data.bar_range    <= high_data.bar_range;
        fifo_wr_high_data.data_par     <= high_data.data_par;
        fifo_wr_high_data.hdr_par      <= high_data.hdr_par;
        fifo_wr_high_data.tlp_prfx_par <= high_data.tlp_prfx_par;
      //end 

      //if (low_data_valid_dup && (low_data_eop_dup || high_data_valid_dup)) begin
        fifo_wr_low_data.data         <= low_data.data;
        fifo_wr_low_data.empty        <= low_data.empty;
        fifo_wr_low_data.sop          <= low_data.sop;
        fifo_wr_low_data.eop          <= low_data.eop;
        fifo_wr_low_data.hdr          <= low_data.hdr;
        fifo_wr_low_data.tlp_prfx     <= low_data.tlp_prfx;
        fifo_wr_low_data.vf_active    <= low_data.vf_active;
        fifo_wr_low_data.func_num     <= low_data.func_num;
        fifo_wr_low_data.vf_num       <= low_data.vf_num;
        fifo_wr_low_data.bar_range    <= low_data.bar_range;
        fifo_wr_low_data.data_par     <= low_data.data_par;
        fifo_wr_low_data.hdr_par      <= low_data.hdr_par;
        fifo_wr_low_data.tlp_prfx_par <= low_data.tlp_prfx_par;
      end
  end

//********************************************************************
// STORAGE
//********************************************************************

  intel_pcie_whr_gen4x8_dcfifo
    #(
      .DWIDTH ($bits(fifo_wr_high_data) + $bits(fifo_wr_low_data)),
      .AWIDTH (4),
      .intended_device_family(intended_device_family)
     ) rxfifo
    (
     .wr_rst_n(rst_clk500_n),                         // Write Domain Active low Reset
     .wr_clk(clk500),                                 // Write Domain Clock
     .wr_en(rx_fifo_wrreq),                           // Write Data Enable
     .wr_data({fifo_wr_high_data, fifo_wr_low_data}), // Write Data In
     .rd_rst_n(rst_clk250_n),                         // Read Domain Active low Reset
     .rd_clk(clk250),                                 // Read Domain Clock
     .rd_en(rx_fifo_rdreq),                           // Read Data Enable
     .r_pempty(4'd2),                                 // FIFO partially empty threshold
     .r_pfull(4'd14),                                 // FIFO partially full threshold
     .rd_data({fifo_rd_high_data, fifo_rd_low_data}), // Read Data Out
     .rd_numdata(rx_rd_numdata),                      // Number of Data available in Read clock
     .wr_numdata(),                                   // Number of Data available in Write clock
     .wr_empty(),                                     // FIFO Empty
     .wr_pempty(),                                    // FIFO Partial Empty
     .wr_full(),                                      // FIFO Full
     .wr_pfull(),                                     // FIFO Parial Full
     .rd_empty(rx_fifo_empty),                        // FIFO Empty
     .rd_pempty(),                                    // FIFO Partial Empty
     .rd_full(),                                      // FIFO Full
     .rd_pfull()                                      // FIFO Partial Full
     );

//********************************************************************
// READ CLOCK DOMAIN
//********************************************************************

  assign rx_fifo_rdreq   = ~(rx_fifo_empty | (rx_rd_numdata == 5'd1 & ~rx_fifo_empty & rx_fifo_rdreq_q));
  // Async FIFO read interface
  always @ (posedge clk250) begin //negedge rst_clk250_n or
    if (rst_clk250_n == 0) begin
      rx_st_valid_o     <= '0;
      rx_fifo_rdreq_q   <= '0;
    end
    else begin
      rx_st_valid_o     <= '0;
      rx_fifo_rdreq_q   <= rx_fifo_rdreq;
      if (rx_fifo_rdreq_q) begin
        rx_st_valid_o   <= {fifo_rd_high_data.valid,        fifo_rd_low_data.valid};
      end
    end
  end

  always @ (posedge clk250) begin
        rx_st_data_o         <= {fifo_rd_high_data.data,         fifo_rd_low_data.data};
        rx_st_empty_o        <= {fifo_rd_high_data.empty,        fifo_rd_low_data.empty};
        rx_st_sop_o          <= {fifo_rd_high_data.sop,          fifo_rd_low_data.sop};
        rx_st_eop_o          <= {fifo_rd_high_data.eop,          fifo_rd_low_data.eop};
        rx_st_hdr_o          <= {fifo_rd_high_data.hdr,          fifo_rd_low_data.hdr};
        rx_st_tlp_prfx_o     <= {fifo_rd_high_data.tlp_prfx,     fifo_rd_low_data.tlp_prfx};
        rx_st_vf_active_o    <= {fifo_rd_high_data.vf_active,    fifo_rd_low_data.vf_active};
        rx_st_func_num_o     <= {fifo_rd_high_data.func_num,     fifo_rd_low_data.func_num};
        rx_st_vf_num_o       <= {fifo_rd_high_data.vf_num,       fifo_rd_low_data.vf_num};
        rx_st_bar_range_o    <= {fifo_rd_high_data.bar_range,    fifo_rd_low_data.bar_range};
        rx_st_data_par_o     <= {fifo_rd_high_data.data_par,     fifo_rd_low_data.data_par};
        rx_st_hdr_par_o      <= {fifo_rd_high_data.hdr_par,      fifo_rd_low_data.hdr_par};
        rx_st_tlp_prfx_par_o <= {fifo_rd_high_data.tlp_prfx_par, fifo_rd_low_data.tlp_prfx_par};
  end

  intel_pcie_whr_gen4x8_sync_pulse rx_par_err_sync (.wr_clk(clk500), .rd_clk(clk250), .wr_rst_n(rst_clk500_n),
                                                  .rd_rst_n(rst_clk250_n), .din(rx_par_err_i), .dout(rx_par_err_o));

endmodule

