// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_tx_st_if
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------

module  intel_pcie_whr_gen4x8_tx_st_if
  #(
    parameter adap_x8_or_x4 = 1,
    parameter intended_device_family = "Stratix 10"
   )
   (
    input  logic                                         clk500,
    input  logic                                         rst_clk500_n,
    input  logic                                         clk250,
    input  logic                                         rst_clk250_n,
    input  logic [(adap_x8_or_x4 ? 256*2-1:128*2-1) : 0] tx_st_data_i,
    input  logic [      1:0]                             tx_st_sop_i,
    input  logic [      1:0]                             tx_st_eop_i,
    input  logic [      1:0]                             tx_st_valid_i,
    input  logic                                         tx_st_ready_i,
    input  logic [      1:0]                             tx_st_err_i,
    input  logic [128*2-1:0]                             tx_st_hdr_i,
    input  logic [ 32*2-1:0]                             tx_st_tlp_prfx_i,
    input  logic [(adap_x8_or_x4 ? 32*2-1:16*2-1) : 0]   tx_st_data_par_i,
    input  logic [ 16*2-1:0]                             tx_st_hdr_par_i,
    input  logic [  4*2-1:0]                             tx_st_tlp_prfx_par_i,
    input  logic                                         tx_par_err_i,
    output logic [(adap_x8_or_x4 ? 255:127) : 0]         tx_st_data_o,
    output logic                                         tx_st_sop_o,
    output logic                                         tx_st_eop_o,
    output logic                                         tx_st_valid_o,
    output logic                                         tx_st_ready_o,
    output logic                                         tx_st_err_o,
    output logic [127:0]                                 tx_st_hdr_o,
    output logic [ 31:0]                                 tx_st_tlp_prfx_o,
    output logic [(adap_x8_or_x4 ? 31:15) : 0]           tx_st_data_par_o,
    output logic [ 15:0]                                 tx_st_hdr_par_o,
    output logic [  3:0]                                 tx_st_tlp_prfx_par_o,
    output logic                                         tx_par_err_o
    );

//******************************************************************************
// DEFINE VARIABLES
//******************************************************************************
  localparam data_width     = adap_x8_or_x4 ? 256*2:128*2;
  localparam data_par_width = adap_x8_or_x4 ? 32*2 :16*2;

  logic [data_width-1:0]     tx_st_data_q;                       /* synthesis preserve_syn_only */ 
  logic [      1:0]          tx_st_sop_q;                        /* synthesis preserve_syn_only */
  logic [      1:0]          tx_st_eop_q;                        /* synthesis preserve_syn_only */
  logic [      1:0]          tx_st_valid_q;                      /* synthesis preserve_syn_only */
  logic [      1:0]          tx_st_err_q;                        /* synthesis preserve_syn_only */
  logic [128*2-1:0]          tx_st_hdr_q;                        /* synthesis preserve_syn_only */
  logic [ 32*2-1:0]          tx_st_tlp_prfx_q;                   /* synthesis preserve_syn_only */
  logic [data_par_width-1:0] tx_st_data_par_q;                   /* synthesis preserve_syn_only */
  logic [ 16*2-1:0]          tx_st_hdr_par_q;                    /* synthesis preserve_syn_only */ 
  logic [  4*2-1:0]          tx_st_tlp_prfx_par_q;               /* synthesis preserve_syn_only */
  logic                      tx_fifo_empty_lo;
  logic                      pending_hi_read;
  logic                      tx_fifo_wrreq;
  logic                      tx_fifo_rdreq_lo_q;
  logic                      tx_fifo_rdreq_hi_q;
  logic                      tx_fifo_rdreq_lo;
  logic                      tx_fifo_rdreq_hi;
  logic [      5:0]          rd_numdata_lo;
  logic                      fifo_almost_full;

  typedef struct packed {
    logic [data_width/2-1:0]     tx_st_data;
    logic                        tx_st_sop;
    logic                        tx_st_eop;
    logic                        tx_st_valid;
    logic                        tx_st_err;
    logic [127:0]                tx_st_hdr;
    logic [ 31:0]                tx_st_tlp_prfx;
    logic [data_par_width/2-1:0] tx_st_data_par;
    logic [ 15:0]                tx_st_hdr_par;
    logic [  3:0]                tx_st_tlp_prfx_par;
    } tx_data_t;

  tx_data_t tx_fifo_wrdata_lo, tx_fifo_wrdata_hi;
  tx_data_t tx_fifo_rddata_lo, tx_fifo_rddata_hi;

//********************************************************************
// WRITE CLOCK DOMAIN
//********************************************************************
  assign tx_st_ready_o = !fifo_almost_full;

  always @ (posedge clk250)  begin //negedge rst_clk250_n or 
    if (rst_clk250_n == 0) begin
      tx_st_valid_q     <= '0;
    end
    else begin
      tx_st_valid_q        <= tx_st_valid_i;
    end
  end

  always @ (posedge clk250)  begin
      tx_st_data_q         <= tx_st_data_i;
      tx_st_sop_q          <= tx_st_sop_i;
      tx_st_eop_q          <= tx_st_eop_i;
      tx_st_err_q          <= tx_st_err_i;
      tx_st_hdr_q          <= tx_st_hdr_i;
      tx_st_tlp_prfx_q     <= tx_st_tlp_prfx_i;
      tx_st_data_par_q     <= tx_st_data_par_i;
      tx_st_hdr_par_q      <= tx_st_hdr_par_i;
      tx_st_tlp_prfx_par_q <= tx_st_tlp_prfx_par_i;
  end


  assign tx_fifo_wrreq     = |tx_st_valid_q;
  assign tx_fifo_wrdata_lo = {tx_st_data_q[data_width/2-1:0],tx_st_sop_q[0], tx_st_eop_q[0], tx_st_valid_q[0], tx_st_err_q[0], tx_st_hdr_q[127:0],
                              tx_st_tlp_prfx_q[31:0], tx_st_data_par_q[data_par_width/2-1:0], tx_st_hdr_par_q[15:0], tx_st_tlp_prfx_par_q[3:0]};
      
  assign tx_fifo_wrdata_hi = {tx_st_data_q[data_width-1:data_width/2], tx_st_sop_q[1], tx_st_eop_q[1], tx_st_valid_q[1], tx_st_err_q[1], tx_st_hdr_q[255:128],
                              tx_st_tlp_prfx_q[63:32], tx_st_data_par_q[data_par_width-1:data_par_width/2], tx_st_hdr_par_q[31:16], tx_st_tlp_prfx_par_q[7:4]};

//********************************************************************
// STORAGE
//********************************************************************

  intel_pcie_whr_gen4x8_dcfifo
  #(
    .DWIDTH ($bits(tx_fifo_wrdata_lo)),
    .AWIDTH (5),
    .intended_device_family(intended_device_family)
   ) tx_fifo_lo
  (
   .wr_rst_n(rst_clk250_n),                         // Write Domain Active low Reset
   .wr_clk(clk250),                                 // Write Domain Clock
   .wr_en(tx_fifo_wrreq),                           // Write Data Enable
   .wr_data(tx_fifo_wrdata_lo),                     // Write Data In
   .rd_rst_n(rst_clk500_n),                         // Read Domain Active low Reset
   .rd_clk(clk500),                                 // Read Domain Clock
   .rd_en(tx_fifo_rdreq_lo),                        // Read Data Enable
   .r_pempty(5'd2),                                 // FIFO partially empty threshold
   .r_pfull(5'd24),                                 // FIFO partially full threshold
   .rd_data(tx_fifo_rddata_lo),                     // Read Data Out
   .rd_numdata(rd_numdata_lo),                      // Number of Data available in Read clock
   .wr_numdata(),                                   // Number of Data available in Write clock
   .wr_empty(),                                     // FIFO Empty
   .wr_pempty(),                                    // FIFO Partial Empty
   .wr_full(),                                      // FIFO Full
   .wr_pfull(fifo_almost_full),                     // FIFO Parial Full
   .rd_empty(tx_fifo_empty_lo),                     // FIFO Empty
   .rd_pempty(),                                    // FIFO Partial Empty
   .rd_full(),                                      // FIFO Full
   .rd_pfull()                                      // FIFO Partial Full
   );
   
  intel_pcie_whr_gen4x8_dcfifo
  #(
    .DWIDTH ($bits(tx_fifo_wrdata_hi)),
    .AWIDTH (5),
    .intended_device_family(intended_device_family)
   ) tx_fifo_hi
  (
   .wr_rst_n(rst_clk250_n),                         // Write Domain Active low Reset
   .wr_clk(clk250),                                 // Write Domain Clock
   .wr_en(tx_fifo_wrreq),                           // Write Data Enable
   .wr_data(tx_fifo_wrdata_hi),                     // Write Data In
   .rd_rst_n(rst_clk500_n),                         // Read Domain Active low Reset
   .rd_clk(clk500),                                 // Read Domain Clock
   .rd_en(tx_fifo_rdreq_hi),                        // Read Data Enable
   .r_pempty(5'd2),                                 // FIFO partially empty threshold
   .r_pfull(5'd24),                                 // FIFO partially full threshold
   .rd_data(tx_fifo_rddata_hi),                     // Read Data Out
   .rd_numdata(),                                   // Number of Data available in Read clock
   .wr_numdata(),                                   // Number of Data available in Write clock
   .wr_empty(),                                     // FIFO Empty
   .wr_pempty(),                                    // FIFO Partial Empty
   .wr_full(),                                      // FIFO Full
   .wr_pfull(),                                     // FIFO Parial Full
   .rd_empty(),                                     // FIFO Empty
   .rd_pempty(),                                    // FIFO Partial Empty
   .rd_full(),                                      // FIFO Full
   .rd_pfull()                                      // FIFO Partial Full
   );

//********************************************************************
// READ CLOCK DOMAIN
//********************************************************************

  always @ (posedge clk500) begin //negedge rst_clk500_n or 
    if (rst_clk500_n == 0) begin 
      tx_fifo_rdreq_lo   <= '0;
      tx_fifo_rdreq_hi   <= '0;
      tx_fifo_rdreq_lo_q <= '0;
      tx_fifo_rdreq_hi_q <= '0;
      pending_hi_read    <= '0;
    end
    else begin
      tx_fifo_rdreq_lo_q <= tx_fifo_rdreq_lo;
      tx_fifo_rdreq_hi_q <= tx_fifo_rdreq_hi;
      tx_fifo_rdreq_lo   <= (~(tx_fifo_empty_lo | (rd_numdata_lo == 6'd1 & ~tx_fifo_empty_lo & tx_fifo_rdreq_lo_q))) & tx_st_ready_i & ~pending_hi_read & ~tx_fifo_rdreq_lo;
      tx_fifo_rdreq_hi   <=  tx_st_ready_i &  (tx_fifo_rdreq_lo | pending_hi_read);
      pending_hi_read    <= ~tx_st_ready_i ? (tx_fifo_rdreq_lo | pending_hi_read) : '0;
    end
  end

  always @ (posedge clk500) begin
    tx_st_data_o         <= tx_fifo_rdreq_hi_q ? tx_fifo_rddata_hi.tx_st_data         : tx_fifo_rddata_lo.tx_st_data;
    tx_st_hdr_o          <= tx_fifo_rdreq_hi_q ? tx_fifo_rddata_hi.tx_st_hdr          : tx_fifo_rddata_lo.tx_st_hdr;
    tx_st_tlp_prfx_o     <= tx_fifo_rdreq_hi_q ? tx_fifo_rddata_hi.tx_st_tlp_prfx     : tx_fifo_rddata_lo.tx_st_tlp_prfx;
    tx_st_data_par_o     <= tx_fifo_rdreq_hi_q ? tx_fifo_rddata_hi.tx_st_data_par     : tx_fifo_rddata_lo.tx_st_data_par;
    tx_st_hdr_par_o      <= tx_fifo_rdreq_hi_q ? tx_fifo_rddata_hi.tx_st_hdr_par      : tx_fifo_rddata_lo.tx_st_hdr_par;
    tx_st_tlp_prfx_par_o <= tx_fifo_rdreq_hi_q ? tx_fifo_rddata_hi.tx_st_tlp_prfx_par : tx_fifo_rddata_lo.tx_st_tlp_prfx_par;
    tx_st_sop_o          <= ((tx_fifo_rddata_lo.tx_st_sop & tx_fifo_rdreq_lo_q)   | (tx_fifo_rddata_hi.tx_st_sop & tx_fifo_rdreq_hi_q));
    tx_st_eop_o          <= ((tx_fifo_rddata_lo.tx_st_eop & tx_fifo_rdreq_lo_q)   | (tx_fifo_rddata_hi.tx_st_eop & tx_fifo_rdreq_hi_q));
    tx_st_err_o          <= ((tx_fifo_rddata_lo.tx_st_err & tx_fifo_rdreq_lo_q)   | (tx_fifo_rddata_hi.tx_st_err & tx_fifo_rdreq_hi_q));
  end

  always @ (posedge clk500) begin //negedge rst_clk500_n or 
    if (rst_clk500_n == 0) begin               
      tx_st_valid_o <= '0;
    end
    else begin
      tx_st_valid_o <= ((tx_fifo_rddata_lo.tx_st_valid & tx_fifo_rdreq_lo_q) | (tx_fifo_rddata_hi.tx_st_valid & tx_fifo_rdreq_hi_q));
    end
  end
  
  intel_pcie_whr_gen4x8_sync_pulse tx_par_err_sync (.wr_clk(clk500), .rd_clk(clk250), .wr_rst_n(rst_clk500_n),
                                                  .rd_rst_n(rst_clk250_n), .din(tx_par_err_i), .dout(tx_par_err_o));
endmodule 
