// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_avmm_credit_if
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module intel_pcie_whr_gen4x8_avmm_credit_if
 #(
     parameter intended_device_family = "Stratix 10"
  )
  (
   input                clk500,
   input                rst_clk500_n,
   input                clk250,
   input                rst_clk250_n,
   input  logic [15:0]  tx_cdts_limit_i,
   input  logic [2:0]   tx_cdts_limit_tdm_idx_i,
   input  logic [11:0]  rx_buffer_limit_i,
   input  logic [1:0]   rx_buffer_limit_tdm_idx_i,
   output logic [15:0]  tx_cdts_limit_o,
   output logic [2:0]   tx_cdts_limit_tdm_idx_o,
   output logic [11:0]  rx_buffer_limit_o,
   output logic [1:0]   rx_buffer_limit_tdm_idx_o
  );

//********************************************************************
// TX CREDIT CDC
//********************************************************************
  //******************************************************************************
  // DEFINE VARIABLES
  //******************************************************************************
  localparam PH     = 3'b000,
             NPH    = 3'b001,
             CPLH   = 3'b010,
             RSVD1  = 3'b011,
             PD     = 3'b100,
             NPD    = 3'b101,
             CPLD   = 3'b110,
             RSVD2  = 3'b111;

  logic [2:0]  tx_credit_indx_500;
  logic [2:0]  tx_credit_indx_250;
  logic [15:0] credit_storage_500 [5:0];
  logic [15:0] credit_storage_250 [5:0];

//********************************************************************
// TX CREDIT CDC
//********************************************************************
  assign tx_credit_indx_500 = tx_cdts_limit_tdm_idx_i;

  always @(posedge clk500 or negedge rst_clk500_n) begin
   if(rst_clk500_n == 0) begin
     credit_storage_500 <= {'0,'0,'0,'0,'0,'0};
   end
   else begin
     case(tx_credit_indx_500)
       PH: begin
           credit_storage_500[0] <= tx_cdts_limit_i;
       end
       NPH: begin
           credit_storage_500[1] <= tx_cdts_limit_i;
       end
       CPLH: begin
           credit_storage_500[2] <= tx_cdts_limit_i;
       end
       PD: begin
           credit_storage_500[3] <= tx_cdts_limit_i;
       end
       NPD: begin
           credit_storage_500[4] <= tx_cdts_limit_i;
       end
       CPLD: begin
           credit_storage_500[5] <= tx_cdts_limit_i;
       end
     endcase
   end
  end

  genvar i;
  generate
    for (i = 0; i < 6; i=i+1) begin: credit_storage
          intel_pcie_whr_gen4x8_avmm_sync_vec #(.DWIDTH(16)) credit_storage_sync (.wr_clk(clk500), .rd_clk(clk250), .wr_rst_n(rst_clk500_n),
                              .rd_rst_n(rst_clk250_n), .din(credit_storage_500[i]), .dout(credit_storage_250[i]));
    end
  endgenerate

  always @(posedge clk250 or negedge rst_clk250_n) begin
   if(rst_clk250_n == 0) begin
     tx_credit_indx_250 <= '0;
   end
   else begin
     if (tx_credit_indx_250 == 3'd5) begin
       tx_credit_indx_250 <= '0;
     end
     else begin
       tx_credit_indx_250 <= tx_credit_indx_250 + 1'b1;
     end
   end
  end

  always @(posedge clk250) begin
     case(tx_credit_indx_250)
       3'b000: begin
           tx_cdts_limit_o          <= credit_storage_250[0];
           tx_cdts_limit_tdm_idx_o <= PH;
       end
       3'b001: begin
           tx_cdts_limit_o          <= credit_storage_250[1];
           tx_cdts_limit_tdm_idx_o <= NPH;
       end
       3'b010: begin
           tx_cdts_limit_o          <= credit_storage_250[2];
           tx_cdts_limit_tdm_idx_o <= CPLH;
       end
       3'b011: begin
           tx_cdts_limit_o          <= credit_storage_250[3];
           tx_cdts_limit_tdm_idx_o <= PD;
       end
       3'b100: begin
           tx_cdts_limit_o          <= credit_storage_250[4];
           tx_cdts_limit_tdm_idx_o <= NPD;
       end
       3'b101: begin
           tx_cdts_limit_o          <= credit_storage_250[5];
           tx_cdts_limit_tdm_idx_o <= CPLD;
       end
     endcase
   end

//********************************************************************
// RX CREDIT CDC
//********************************************************************
  //******************************************************************************
  // DEFINE VARIABLES
  //******************************************************************************
  localparam PB   = 2'b00,
             NPB  = 2'b01,
             CPLB = 2'b10,
             RSVD = 2'b11;

  logic rx_credit_fifo_empty;
  logic rx_credit_fifo_wrreq;
  logic rx_credit_fifo_rdreq, rx_credit_fifo_rdreq_q;

  typedef struct packed {
    logic [11:0]  rx_buffer_limit;
    logic [1:0]   rx_buffer_limit_tdm_idx;
  } rx_credit_data_t;

  rx_credit_data_t rx_credit_in, rx_credit_out;
  logic [2:0] rx_credit_rd_numdata;
  //******************************************************************************
  // FIFO TO SYNCHRONIZE RX CREDIT VALUES
  //******************************************************************************
  assign rx_credit_in = {rx_buffer_limit_i, rx_buffer_limit_tdm_idx_i};
  assign rx_credit_fifo_wrreq = (rx_buffer_limit_tdm_idx_i == PB | rx_buffer_limit_tdm_idx_i == NPB | rx_buffer_limit_tdm_idx_i == CPLB) ? 1'b1 : 1'b0;

  intel_pcie_whr_gen4x8_avmm_dcfifo
  #(
    .DWIDTH ($bits(rx_credit_in)),
    .AWIDTH (2),
    .intended_device_family(intended_device_family)
   ) rx_credit_data_fifo
  (
   .wr_rst_n(rst_clk250_n),                         // Write Domain Active low Reset
   .wr_clk(clk250),                                 // Write Domain Clock
   .wr_en(rx_credit_fifo_wrreq),                    // Write Data Enable
   .wr_data(rx_credit_in),                          // Write Data In
   .rd_rst_n(rst_clk500_n),                         // Read Domain Active low Reset
   .rd_clk(clk500),                                 // Read Domain Clock
   .rd_en(rx_credit_fifo_rdreq),                    // Read Data Enable
   .r_pempty(2'd1),                                 // FIFO partially empty threshold
   .r_pfull(2'd2),                                  // FIFO partially full threshold
   .rd_data(rx_credit_out),                         // Read Data Out
   .rd_numdata(rx_credit_rd_numdata),               // Number of Data available in Read clock
   .wr_numdata(),                                   // Number of Data available in Write clock
   .wr_empty(),                                     // FIFO Empty
   .wr_pempty(),                                    // FIFO Partial Empty
   .wr_full(),                                      // FIFO Full
   .wr_pfull(),                                     // FIFO Parial Full
   .rd_empty(rx_credit_fifo_empty),                 // FIFO Empty
   .rd_pempty(),                                    // FIFO Partial Empty
   .rd_full(),                                      // FIFO Full
   .rd_pfull()                                      // FIFO Partial Full
   );

  assign rx_credit_fifo_rdreq = ~(rx_credit_fifo_empty | (rx_credit_rd_numdata == 3'd1 & ~rx_credit_fifo_empty & rx_credit_fifo_rdreq_q));

  always_ff @ (negedge rst_clk500_n or posedge clk500) begin
    if (rst_clk500_n == 0) begin
      rx_buffer_limit_o           <= '0;
      rx_buffer_limit_tdm_idx_o  <= '0;
      rx_credit_fifo_rdreq_q    <= '0;
    end
    else begin
      rx_credit_fifo_rdreq_q    <= rx_credit_fifo_rdreq;
      if (rx_credit_fifo_rdreq_q) begin
        rx_buffer_limit_o           <= rx_credit_out.rx_buffer_limit;
        rx_buffer_limit_tdm_idx_o  <= rx_credit_out.rx_buffer_limit_tdm_idx;
      end
    end
  end
endmodule
