// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_avmm_dcfifo
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module  intel_pcie_whr_gen4x8_avmm_dcfifo #(

  parameter DWIDTH      = 8,// FIFO Input data width
  parameter AWIDTH      = 4,// FIFO Depth (address width)
  parameter SIM_EMULATE = 0,
  parameter intended_device_family = "Stratix 10"
  )
  (
   input  logic                   wr_rst_n,     // Write Domain Active low Reset
   input  logic                   wr_clk,       // Write Domain Clock
   input  logic                   wr_en,        // Write Data Enable
   input  logic [DWIDTH-1:0]      wr_data,      // Write Data In
   input  logic                   rd_rst_n,     // Read Domain Active low Reset
   input  logic                   rd_clk,       // Read Domain Clock
   input  logic                   rd_en,        // Read Data Enable
   input  logic [AWIDTH-1:0]      r_pempty,     // FIFO partially empty threshold
   input  logic [AWIDTH-1:0]      r_pfull,      // FIFO partially full threshold

   output logic [DWIDTH-1:0]      rd_data,      // Read Data Out
   output logic [AWIDTH:0]        rd_numdata,   // Number of Data available in Read clock
   output logic [AWIDTH:0]        wr_numdata,   // Number of Data available in Write clock

   output logic                    wr_empty,     // FIFO Empty
   output logic                    wr_pempty,    // FIFO Partial Empty
   output logic                    wr_full,      // FIFO Full
   output logic                    wr_pfull,     // FIFO Parial Full
   output logic                    rd_empty,     // FIFO Empty
   output logic                    rd_pempty,    // FIFO Partial Empty
   output logic                    rd_full,      // FIFO Full
   output logic                    rd_pfull      // FIFO Partial Full
  );

//********************************************************************
// DEFINE VARIABLES
//********************************************************************
  logic [AWIDTH:0]           wr_addr_bin;
  logic [AWIDTH:0]           rd_addr_bin;
  logic [AWIDTH-1:0]         wr_addr_mem;
  logic [AWIDTH-1:0]         rd_addr_mem;
  
  logic [AWIDTH:0]           wr_addr_bin_nxt;
  logic [AWIDTH:0]           rd_addr_bin_nxt;
  logic [AWIDTH:0]           wr_addr_bin_sync;
  logic [AWIDTH:0]           rd_addr_bin_sync;
  logic [AWIDTH:0]           rd_numdata_comb;
  logic [DWIDTH-1:0]         rd_data_out; 
//********************************************************************
// STORAGE
//********************************************************************
  alt_mlab m0 (
               .wclk(wr_clk),
               .wena(1'b1),
               .waddr_reg(wr_addr_mem),
               .wdata_reg(wr_data),
               .raddr(rd_addr_mem),
               .rdata(rd_data_out)
               );
  defparam m0 .SIM_EMULATE = SIM_EMULATE;
  defparam m0 .WIDTH = DWIDTH;
  defparam m0 .ADDR_WIDTH = AWIDTH;
  defparam m0 .intended_device_family = intended_device_family;

  always @(posedge rd_clk) begin
    rd_data <= rd_data_out;
  end
//********************************************************************
// WRITE CLOCK DOMAIN: Generate WRITE Address & WRITE Address GREY
//********************************************************************
  // Memory write-address pointer
  assign wr_addr_mem = wr_addr_bin[AWIDTH-1:0];
  always @(negedge wr_rst_n or posedge wr_clk) begin
  if (wr_rst_n == 1'b0) begin
     wr_addr_bin <= {(AWIDTH+1){1'b0}};
  end
  else begin
     wr_addr_bin <= wr_addr_bin_nxt;
  end
  end
  // Binary Next Write Address
  assign wr_addr_bin_nxt = wr_addr_bin + (wr_en);

//********************************************************************
// WRITE CLOCK DOMAIN: Synchronize Read Address to Write Clock
//********************************************************************

  intel_pcie_whr_gen4x8_avmm_gpx3 #(
    .DWIDTH (AWIDTH+1)
    )
      gx1 (
           .din_clk(rd_clk),
           .din_rst_n  (rd_rst_n),
           .din(rd_addr_bin_nxt),
           .dout_clk(wr_clk),
           .dout_rst_n (wr_rst_n),
           .dout(rd_addr_bin_sync)
           );

  assign wr_numdata = (wr_addr_bin_nxt - rd_addr_bin_sync);

//********************************************************************
// WRITE CLOCK DOMAIN: Generate Fifo Number of Data Present
// using Write Address and Synchronized Read Address
//********************************************************************
  always @(negedge wr_rst_n or posedge wr_clk) begin
     if (wr_rst_n == 1'b0) begin
        wr_full  <= 1'b0;
        wr_pfull <= 1'b0;
        wr_empty <= 1'b1;
        wr_pempty<= 1'b1;
     end
     else begin
     // Generate FIFO Empty
        wr_empty <= (wr_numdata <= 0) ? 1'b1 : 1'b0;
        // Generate FIFO Almost Empty
        wr_pempty <= (wr_numdata <= r_pempty) ? 1'b1 : 1'b0;
        // Generate FIFO Full
        wr_full  <= (wr_numdata >= (1<<AWIDTH)) ? 1'b1 : 1'b0;
        // Generate FIFO Almost Full
        wr_pfull  <= (wr_numdata >= r_pfull) ? 1'b1 : 1'b0;
     end
  end
//********************************************************************
// READ CLOCK DOMAIN: Generate READ Address & READ Address GREY
//********************************************************************
// Memory read-address pointer
  assign rd_addr_mem = rd_addr_bin[AWIDTH-1:0];

  always @(negedge rd_rst_n or posedge rd_clk) begin
     if (rd_rst_n == 1'b0) begin
        rd_addr_bin <= {(AWIDTH+1){1'b0}};
     end
     else begin
        rd_addr_bin <= rd_addr_bin_nxt;
     end
  end
  // Binary Next Read Address
  assign rd_addr_bin_nxt = rd_addr_bin + (rd_en & ~rd_empty);

//********************************************************************
// READ CLOCK DOMAIN: Synchronize Write Address to Read Clock
//********************************************************************


  intel_pcie_whr_gen4x8_avmm_gpx3 #(
    .DWIDTH (AWIDTH+1)
    ) gx0 (
           .din_clk(wr_clk),
           .din_rst_n  (wr_rst_n),
           .din(wr_addr_bin_nxt),
           .dout_clk(rd_clk),
           .dout_rst_n (rd_rst_n),
           .dout(wr_addr_bin_sync)
           );

  assign rd_numdata_comb       = (wr_addr_bin_sync - rd_addr_bin);

//********************************************************************
// READ CLOCK DOMAIN: Generate Fifo Number of Data Present
// using Read Address and Synchronized Write Address
//********************************************************************
  always @(negedge rd_rst_n or posedge rd_clk) begin
     if (rd_rst_n == 1'b0) begin
        rd_empty    <= 1'b1;
        rd_pempty   <= 1'b1;
        rd_full     <= 1'b0;
        rd_pfull    <= 1'b0;
        rd_numdata  <= 'b0;
     end
     else begin
     rd_numdata <= rd_numdata_comb;
     // Generate FIFO Empty
     rd_empty <= (rd_numdata_comb <= 0) ? 1'b1 : 1'b0;
     // Generate FIFO Almost Empty
     rd_pempty <= (rd_numdata_comb <= r_pempty) ? 1'b1 : 1'b0;
     // Generate FIFO Almost Full
     rd_pfull <= (rd_numdata_comb >= r_pfull) ? 1'b1 : 1'b0;
     // Generate FIFO Full
     rd_full <= (rd_numdata_comb >= (1<<AWIDTH)) ? 1'b1 : 1'b0;
     end
  end
endmodule // wrlib_async_fifo
