// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ns/1ps

module  intel_pcie_adapter_rx_if #(
  parameter          UI_WIDTH              = 128,
  parameter          HIP_WIDTH             = 256,
  parameter          PF_ENABLE             = 1,
  parameter          VF_ENABLE             = 1,
  parameter          HEADER_PREFIX_ENABLE  = 1,
  parameter          ROOT_PORT_MODE        = 0,
  parameter          MPS                   = 256,
  parameter          EMPTY_WIDTH           = (UI_WIDTH==256)  ? 3 : (UI_WIDTH==128)  ? 2 : 1,
  parameter          HIP_EMPTY_WIDTH       = (HIP_WIDTH==256) ? 3 : (HIP_WIDTH==128) ? 2 : 1,
  parameter          SIM_EMULATE           = 0,
  parameter          ECC_ENABLE            = 0,
  parameter          FAMILY                = "Stratix 10"
) (

  input logic                       clk,
  input logic                       rst,

  input logic                       rx_st_ready,
  output logic [UI_WIDTH-1:0]       rx_st_data      ,
  output logic [UI_WIDTH/8-1:0]     rx_st_data_par  ,
  output logic [127:0]              rx_st_hdr       ,
  output logic [15:0]               rx_st_hdr_par   ,
  output logic [31:0]               rx_st_tlp_prfx  ,
  output logic [3:0]                rx_st_tlp_prfx_par,
  output logic                      rx_st_vf_active ,
  output logic [2:0]                rx_st_func_num  ,
  output logic [10:0]               rx_st_vf_num    ,
  output logic                      rx_st_sop       ,
  output logic                      rx_st_eop       ,
  output logic                      rx_st_valid     ,
  output logic                      rx_st_tlp_abort ,
  output logic [EMPTY_WIDTH-1:0]    rx_st_empty     ,
  output logic [2:0]                rx_st_bar_range ,

  output logic                       rx_st_ready_hip     ,
  input  logic [HIP_WIDTH-1:0]       rx_st_data_hip      ,
  input  logic [HIP_WIDTH/8-1:0]     rx_st_data_par_hip  ,
  input  logic [127:0]               rx_st_hdr_hip       ,
  input  logic [15:0]                rx_st_hdr_par_hip   ,
  input  logic [31:0]                rx_st_tlp_prfx_hip  ,
  input  logic [3:0]                 rx_st_tlp_prfx_par_hip,
  input  logic                       rx_st_vf_active_hip ,
  input  logic [2:0]                 rx_st_func_num_hip  ,
  input  logic [10:0]                rx_st_vf_num_hip    ,
  input  logic                       rx_st_sop_hip       ,
  input  logic                       rx_st_eop_hip       ,
  input  logic                       rx_st_valid_hip     ,
  input  logic                       rx_st_tlp_abort_hip ,
  input  logic [HIP_EMPTY_WIDTH-1:0] rx_st_empty_hip     ,
  input  logic [2:0]                 rx_st_bar_range_hip
);
  localparam  NUM_DATA_MEM         = HIP_WIDTH/32;
  localparam  HDR_MEM_WIDTH        = HEADER_PREFIX_ENABLE ? 163 : 131;
  localparam  HDR_PAR_MEM_WIDTH    = HEADER_PREFIX_ENABLE ? 20 : 16;
  localparam  MEM_SIZE             = 64;
  localparam  MEM_TYPE             = "MLAB";
  localparam  WIDTH_RATIO          = HIP_WIDTH/UI_WIDTH;
  localparam  MEM_SIZE_WIDTH       = $clog2(MEM_SIZE);
  localparam  PF_WIDTH             = ROOT_PORT_MODE ? 0 : PF_ENABLE ? 3:0;
  localparam  VF_WIDTH             = ROOT_PORT_MODE ? 0 : VF_ENABLE ? 11+1:0;
  localparam  CONTROL_WIDTH        = VF_WIDTH + PF_WIDTH + EMPTY_WIDTH + WIDTH_RATIO + 2 ;


  logic                             data_present;
  logic                             header_write;
  logic [WIDTH_RATIO-1:0]           eop_location;
  logic [3:0]                       st_empty;

  logic [UI_WIDTH-1:0]              rx_st_data_out_wire  [WIDTH_RATIO-1:0];
  logic [UI_WIDTH/8-1:0]            rx_st_data_par_out_wire [WIDTH_RATIO-1:0];

  //logic [0:(UI_WIDTH/64)-1][WIDTH_RATIO-1] rx_st_empty_hip_wire;
  logic [EMPTY_WIDTH-1 : 0]         rx_st_empty_hip_wire;
  logic [WIDTH_RATIO-1:0]           rx_st_eop_hip_wire;
  logic [CONTROL_WIDTH-1:0]         rx_st_control_in_wire,rx_st_control_out_wire;

  logic  [MEM_SIZE_WIDTH-1:0]   wr_ptr,rd_ptr;
  logic  [MEM_SIZE_WIDTH-1:0]   fillcount,readycount;
  logic                         fillcount_lessthan_4;
  logic                         read_req,read_req_r,read_req_rr;
  logic                         fifo_empty;
  logic  [WIDTH_RATIO-1:0]      select_line;

  logic [9:0]                   tlp_length;
  logic [WIDTH_RATIO-1:0]       eop_reg;
  logic [(UI_WIDTH/64)-1 : 0]                 empty_reg;

  logic [WIDTH_RATIO-1:0]                     rx_st_eop_wire;
  logic [EMPTY_WIDTH-1:0]                     rx_st_empty_wire;
  logic                                       rx_st_sop_wire  ;
  logic                                       rx_st_valid_wire;
  logic                                       rx_st_tlp_abort_wire;
  logic [HIP_WIDTH-1:0]                       rx_st_data_wire     ;
  logic [HIP_WIDTH/8-1:0]                     rx_st_data_par_wire ;
  logic [HDR_MEM_WIDTH-1:0]                   rx_st_hdr_wire      ;
  logic                                       rx_st_vf_act_wire   ;
  logic [10:0]                                rx_st_vf_wire       ;
  logic [2:0]                                 rx_st_func_wire     ;
  logic                                       rx_st_ready_r       ,rx_st_ready_rr,    rx_st_ready_rrr;

  logic [HEADER_PREFIX_ENABLE ? 162 :130:0]   rx_st_hdr_hip_r;
  logic [HDR_PAR_MEM_WIDTH-1:0]               rx_st_hdr_par_hip_r,rx_st_hdr_par_wire;
  logic [HIP_WIDTH-1:0]                       rx_st_data_hip_r;
  logic [HIP_WIDTH/8-1:0]                     rx_st_data_par_hip_r;
  logic                                       rx_st_vf_act_hip_r;
  logic [10:0]                                rx_st_vf_hip_r;
  logic [2:0]                                 rx_st_func_hip_r;
  logic                                       rx_st_sop_hip_r;
  logic                                       rx_st_tlp_abort_hip_r;
  logic                                       rx_st_eop_hip_r;
  logic                                       rx_st_valid_hip_r,rx_st_valid_hip_rr;

  logic                                       rx_st_valid_pre;
  logic                                       pre_eop_at_0;
  logic                                       eop_reg1_detected;
  logic                                       pre_read_req;
  logic                                       tlp_length_is0;


  assign tlp_length             = rx_st_hdr_hip[105:96];
  assign rx_st_control_in_wire  = !PF_ENABLE ? {rx_st_empty_hip_wire,rx_st_eop_hip_wire,rx_st_tlp_abort_hip_r,rx_st_sop_hip_r} :
                                  !VF_ENABLE ? {rx_st_func_hip_r,rx_st_empty_hip_wire,rx_st_eop_hip_wire,rx_st_tlp_abort_hip_r,rx_st_sop_hip_r}
                                  : {rx_st_vf_act_hip_r,rx_st_vf_hip_r,rx_st_func_hip_r,rx_st_empty_hip_wire,
                                  rx_st_eop_hip_wire,rx_st_tlp_abort_hip_r,rx_st_sop_hip_r};
  //assign rx_st_empty_hip_wire = eop_location & st_empty;
/*   assign read_req = !fifo_empty & (!rx_st_packet_present |
                                  (rx_st_valid_pre & (select_line[WIDTH_RATIO-1])) |
                                  (rx_st_ready_rr & ((rx_st_eop_wire[0] & eop_reg[0])|eop_reg[0])));
  assign read_req = !fifo_empty & ((rx_st_valid_pre & (select_line[WIDTH_RATIO-1])) |
                                  (rx_st_ready_rr & ((rx_st_eop_wire[0] & eop_reg[0])|eop_reg[0])));*/

  assign read_req = !fifo_empty & ((rx_st_valid_pre & (select_line[WIDTH_RATIO-1])) |
                                  (rx_st_valid_pre & rx_st_eop_wire[0]) |
                                  (rx_st_valid_pre & rx_st_valid & eop_reg[1]) |
                                  (((rx_st_valid & !pre_read_req ) | (rx_st_valid_pre & !rx_st_valid)) & eop_reg[0] & !pre_eop_at_0));

  genvar i;
  generate
    if (!PF_ENABLE) begin
      assign {rx_st_empty_wire,rx_st_eop_wire,rx_st_tlp_abort_wire,rx_st_sop_wire} = rx_st_control_out_wire;
    end
    else begin
      if (!VF_ENABLE)
        assign {rx_st_func_wire,rx_st_empty_wire,rx_st_eop_wire,rx_st_tlp_abort_wire,rx_st_sop_wire} = rx_st_control_out_wire;
      else
        assign {rx_st_vf_act_wire,rx_st_vf_wire,rx_st_func_wire,rx_st_empty_wire,
               rx_st_eop_wire,rx_st_tlp_abort_wire,rx_st_sop_wire} = rx_st_control_out_wire;
    end

    always_ff @(posedge clk) begin
      if (rst) begin
        eop_location   <= '0;
        tlp_length_is0 <= '0;
      end
      else begin
        if (rx_st_sop_hip & rx_st_valid_hip) begin
          tlp_length_is0 <= tlp_length == '0;
          if (UI_WIDTH == 64) begin
            eop_location[0] <= (tlp_length[1:0] == 2'd1) |(tlp_length[1:0] == 2'd2);
            eop_location[1] <= (tlp_length[1:0] == 2'd3) |(tlp_length[2:0] == 3'd4) | (tlp_length[2:0] == 3'd0);
            if (WIDTH_RATIO > 2) begin
              eop_location[0] <= (tlp_length[2:0] == 3'd1) | (tlp_length[2:0] == 3'd2);
              eop_location[1] <= (tlp_length[2:0] == 3'd3) | (tlp_length[2:0] == 3'd4);
              eop_location[2] <= (tlp_length[2:0] == 3'd5) | (tlp_length[2:0] == 3'd6);
              eop_location[3] <= (tlp_length[2:0] == 3'd7) | (tlp_length[3:0] == 4'd8) | (tlp_length[3:0] == 4'd0);
            end
          end
          if (UI_WIDTH == 128) begin
            eop_location[0] <= (tlp_length[2:0] >= 3'd1) & (tlp_length[2:0] <= 3'd4);
            eop_location[1] <= ((tlp_length[2:0] >= 3'd5) & (tlp_length[2:0] <= 3'd7)) | (tlp_length[2:0] == 3'd0);
          end
        end
      end
    end

    assign rx_st_empty_hip_wire = st_empty[EMPTY_WIDTH-1:0];
    for (i=0; i < WIDTH_RATIO; i=i+1) begin: st_empty_gen
      //assign rx_st_empty_hip_wire[i] = eop_location[i] & st_empty;
      if (i == 0)
        assign rx_st_eop_hip_wire[i] = tlp_length_is0 | (!data_present ? rx_st_eop_hip_r : rx_st_eop_hip_r & eop_location[i]);
      else
        assign rx_st_eop_hip_wire[i] = !data_present ? 1'b0 : rx_st_eop_hip_r & eop_location[i];
        //assign rx_st_eop_hip_wire[i] = rx_st_eop_hip_r & eop_location[i];
    end
  endgenerate


  always_ff @(posedge clk) begin
    if (rst) begin
      st_empty <= '0;
    end
    else begin
      st_empty <= 4'd8 - tlp_length[3:0];
    end
  end



  always_ff @(posedge clk) begin
    if (rst) begin
      data_present <= '0;
    end
    else begin
      if (rx_st_sop_hip & rx_st_valid_hip)
        //data_present <= rx_st_hdr_hip[30];
        data_present <= rx_st_hdr_hip[126];
    end
  end

  always_ff @(posedge clk) begin
    if (rst) begin
      wr_ptr <= '0;
    end
    else begin
      if (rx_st_valid_hip_r) begin
        wr_ptr <= wr_ptr + 1'b1;
      end
      else begin
        wr_ptr <= wr_ptr;
      end
    end
  end

  always_ff @(posedge clk) begin
    if (rst) begin
      rd_ptr <= '0;
    end
    else begin
      if (read_req) begin
        rd_ptr <=  rd_ptr +  1'b1;
      end
      else begin
        rd_ptr <= rd_ptr;
      end
    end
  end

  always_ff @(posedge clk) begin
    if (rst) begin
      fillcount            <= '0;
      readycount           <= '0;
      rx_st_ready_hip      <= '0;
      fifo_empty           <= 1'b1;
      fillcount_lessthan_4 <= 1'b1;
    end
    else begin
      fifo_empty <= fillcount_lessthan_4 & ((read_req & !rx_st_valid_hip_rr & (!fillcount[1] & fillcount[0])) | !(fillcount[0] | fillcount[1] | fillcount[2]));
      fillcount_lessthan_4 <= fillcount < 4;
      rx_st_ready_hip <= (readycount < 6'd04);
      if (read_req & !rx_st_valid_hip_rr)
        fillcount <= fillcount - 1'b1;
      if (!read_req & rx_st_valid_hip_rr)
        fillcount <= fillcount + 1'b1;
      if (read_req & !rx_st_valid_hip)
        readycount <= readycount - 1'b1;
      if (!read_req & rx_st_valid_hip)
        readycount <= readycount + 1'b1;
    end
  end

  always_ff @(posedge clk) begin
    rx_st_data_hip_r     <= rx_st_data_hip;
    rx_st_data_par_hip_r <=rx_st_data_par_hip;
    if (rst) begin
      rx_st_valid_hip_r <= '0;
      rx_st_sop_hip_r   <= '0;
      rx_st_eop_hip_r   <= '0;
      rx_st_hdr_hip_r   <= '0;
      rx_st_valid_hip_rr <= '0;
      rx_st_vf_act_hip_r <= '0;
      rx_st_vf_hip_r     <= '0;
      rx_st_func_hip_r   <= '0;
    end
    else begin
      if (rx_st_sop_hip & rx_st_valid_hip) begin
        rx_st_vf_act_hip_r    <= rx_st_vf_active_hip;
        rx_st_vf_hip_r        <= rx_st_vf_num_hip;
        rx_st_func_hip_r      <= rx_st_func_num_hip;
      end
      rx_st_valid_hip_rr    <= rx_st_valid_hip_r;
      rx_st_eop_hip_r       <= rx_st_eop_hip & rx_st_valid_hip;
      rx_st_sop_hip_r       <= rx_st_sop_hip & rx_st_valid_hip;
      rx_st_tlp_abort_hip_r <= rx_st_tlp_abort_hip  & rx_st_valid_hip;
      rx_st_valid_hip_r <= rx_st_valid_hip;
      if (rx_st_valid_hip & rx_st_sop_hip) begin
        rx_st_hdr_hip_r  <= HEADER_PREFIX_ENABLE ? {rx_st_bar_range_hip,rx_st_tlp_prfx_hip,rx_st_hdr_hip} : {rx_st_bar_range_hip,rx_st_hdr_hip};
        rx_st_hdr_par_hip_r <= HEADER_PREFIX_ENABLE ? {rx_st_tlp_prfx_par_hip,rx_st_hdr_par_hip} : {rx_st_hdr_par_hip};
      end
    end
  end

  always_ff @(posedge clk) begin
    if (rst) begin
      select_line <= {{{WIDTH_RATIO-2}{1'b0}},1'b1};
      eop_reg1_detected <= '0;
    end
    else begin
      if (rx_st_valid & eop_reg[1] & !eop_reg[0] & !rx_st_valid_pre) //(eop_reg[0] is condition for 0length MrWr)
        eop_reg1_detected <= 1'b1;
      if (rx_st_valid_pre & eop_reg1_detected)
        eop_reg1_detected <= 1'b0;

      if (rx_st_valid_pre)
        select_line <= {select_line[WIDTH_RATIO-2:0],select_line[WIDTH_RATIO-1]};
     /*  if (rx_st_valid_pre & (eop_reg[0] | rx_st_eop_wire[0]))
        select_line <= {{(WIDTH_RATIO-2){1'b0}},1'b1}; */
      if ((rx_st_valid & eop_reg[1] & rx_st_valid_pre) | (rx_st_valid_pre & rx_st_eop_wire[0]) | (rx_st_valid_pre & eop_reg1_detected))
        select_line <= {{(WIDTH_RATIO-2){1'b0}},1'b1};

      //if (rx_st_valid_pre & rx_st_eop)
        //select_line <= {{(WIDTH_RATIO-2){1'b0}},1'b1};
    end
  end

  always_ff @(posedge clk) begin
    if (rst) begin
      eop_reg   <= '0;
      //empty_reg <= '0;
    end
    else begin
      eop_reg   <= (fifo_empty | read_req_r) ? rx_st_eop_wire   : rx_st_valid ? {1'b0,eop_reg  [WIDTH_RATIO-1:1]} : eop_reg;
      //empty_reg <= (fifo_empty | read_req_r) ? rx_st_empty_wire : rx_st_valid ? {1'b0,empty_reg[WIDTH_RATIO-1:1]} : empty_reg;
    end
  end



  always_ff @(posedge clk) begin
    if (rst) begin
      read_req_r         <= '0;
      read_req_rr        <= '0;
      rx_st_valid        <= '0;
      rx_st_sop          <= '0;
      rx_st_hdr          <= '0;
      rx_st_tlp_abort    <= '0;
      rx_st_valid_pre    <= '0;
      rx_st_ready_r      <= '0;
      rx_st_ready_rr     <= '0;
      rx_st_ready_rrr    <= '0;
      pre_eop_at_0       <= '0;
      pre_read_req       <= '0;
      rx_st_bar_range    <= '0;
      rx_st_tlp_prfx     <= '0;
      rx_st_empty        <= '0;
      if (PF_ENABLE) begin
        rx_st_func_num   <= '0;
      end
      if (VF_ENABLE) begin
        rx_st_vf_active  <= '0;
        rx_st_vf_num     <= '0;
      end
    end
    else begin
      pre_read_req       <= !fifo_empty  & ((rx_st_valid_pre & !rx_st_valid &  eop_reg[0] & !pre_eop_at_0) | (rx_st_valid_pre & rx_st_valid &  eop_reg[1]));
      pre_eop_at_0       <= !fifo_empty  & rx_st_valid_pre & rx_st_eop_wire[0];
      if (pre_eop_at_0 & !rx_st_eop_wire[0])
        pre_eop_at_0 <= 1'b0;

      read_req_r         <= read_req;
      read_req_rr        <= read_req_r;
      if (PF_ENABLE) begin
        rx_st_func_num   <= rx_st_func_wire;
      end
      if (VF_ENABLE) begin
        rx_st_vf_active   <= rx_st_vf_act_wire;
        rx_st_vf_num      <= rx_st_vf_wire;
      end
      rx_st_tlp_prfx     <= rx_st_sop_wire & select_line[0] ? (HEADER_PREFIX_ENABLE ? rx_st_hdr_wire[159:128] : '0) : '0;
      rx_st_tlp_prfx_par <= rx_st_sop_wire & select_line[0] ? (HEADER_PREFIX_ENABLE ? rx_st_hdr_par_wire[19:16] : '0) : '0;
      rx_st_bar_range    <= rx_st_sop_wire & select_line[0] ? (HEADER_PREFIX_ENABLE ? rx_st_hdr_wire[162:160] : rx_st_hdr_wire[130:128]) : '0;
      rx_st_hdr          <= rx_st_sop_wire & select_line[0] ? rx_st_hdr_wire[127:0] : '0;
      rx_st_hdr_par      <= rx_st_sop_wire & select_line[0] ? rx_st_hdr_par_wire[15:0] : '0;
      rx_st_sop          <= rx_st_sop_wire & select_line[0];
      rx_st_tlp_abort    <= rx_st_tlp_abort_wire;
      rx_st_empty        <= rx_st_empty_wire;

      rx_st_ready_r      <= rx_st_ready;
      rx_st_ready_rr     <= rx_st_ready_r;
      rx_st_ready_rrr    <= rx_st_ready_rr;
      rx_st_valid        <= rx_st_valid_pre;
      rx_st_valid_pre    <= !(fillcount_lessthan_4 & ((read_req & !rx_st_valid_hip_rr & (!fillcount[1] & fillcount[0])) | !(fillcount[0] | fillcount[1] | fillcount[2]))) & rx_st_ready_r;

    end
  end

  //assign rx_st_sop      = rx_st_sop_wire & select_line[0];
  assign rx_st_eop      = eop_reg[0];
  //assign rx_st_empty    = empty_reg[0];

  generate
    always_ff @(posedge clk) begin
      if (rst) begin
        rx_st_data     <= '0;
        rx_st_data_par <=  '0;
      end
      else begin
        for (int i=0; i < WIDTH_RATIO; i=i+1 ) begin : rx_st_data_UI
          if(select_line == (1 << i)) begin
            rx_st_data     <= rx_st_data_out_wire[i];
            rx_st_data_par <= rx_st_data_par_out_wire[i];
          end
        end
      end
    end
  endgenerate

  genvar ig;

  generate
    for(ig=0; ig < WIDTH_RATIO; ig=ig+1) begin: data_mux_array
      assign rx_st_data_out_wire[ig] = rx_st_data_wire[((ig+1)*UI_WIDTH)-1:(ig*UI_WIDTH)];
      assign rx_st_data_par_out_wire[ig] = rx_st_data_par_wire[((ig+1)*(UI_WIDTH/8))-1:(ig*(UI_WIDTH/8))];
    end
  endgenerate




/*   always_ff @(posedge clk) begin
    rx_st_data <= rx_st_data_out_wire;
  end
  int j;
  always_comb begin
    for ( j=0; j < WIDTH_RATIO-1; j=j+1 ) begin : rx_st_data_UI
      if (select_line == (1 << j)) begin
        rx_st_data_out_wire = rx_st_data_wire[(UI_WIDTH*(j+1))-1:j*UI_WIDTH];
      end
    end
  end */

  generate
    for (i=0; i < NUM_DATA_MEM; i=i+1 ) begin : data_fifo_memory
      alt_mlab data_memory (
        .wclk               (clk                               ),
        .wena               (rx_st_valid_hip_r                 ),
        .waddr_reg          (wr_ptr                            ),
        .wdata_reg          (rx_st_data_hip_r[32*i+31:32*i]    ),
        .raddr              (rd_ptr                            ),
        .rdata              (rx_st_data_wire[32*i+31:32*i]     )
        );
      defparam data_memory .SIM_EMULATE = SIM_EMULATE;
      defparam data_memory .WIDTH = 32;
      defparam data_memory .ADDR_WIDTH = MEM_SIZE_WIDTH;
    end

    alt_mlab hdr_memory (
      .wclk               (clk                                ),
      .wena               (rx_st_valid_hip_r & rx_st_sop_hip_r),
      .waddr_reg          (wr_ptr                             ),
      .wdata_reg          (rx_st_hdr_hip_r                    ),
      .raddr              (rd_ptr                             ),
      .rdata              (rx_st_hdr_wire                     )
     );
    defparam hdr_memory .SIM_EMULATE = SIM_EMULATE;
    defparam hdr_memory .WIDTH = HDR_MEM_WIDTH;
    defparam hdr_memory .ADDR_WIDTH = MEM_SIZE_WIDTH;

    if (ECC_ENABLE == 0) begin

      alt_mlab data_memory (
        .wclk               (clk                               ),
        .wena               (rx_st_valid_hip_r                 ),
        .waddr_reg          (wr_ptr                            ),
        .wdata_reg          (rx_st_data_par_hip_r              ),
        .raddr              (rd_ptr                            ),
        .rdata              (rx_st_data_par_wire               )
        );
      defparam data_memory .SIM_EMULATE = SIM_EMULATE;
      defparam data_memory .WIDTH = HIP_WIDTH/8;
      defparam data_memory .ADDR_WIDTH = MEM_SIZE_WIDTH;

      alt_mlab hdr_par_memory (
        .wclk               (clk                                ),
        .wena               (rx_st_valid_hip_r & rx_st_sop_hip_r),
        .waddr_reg          (wr_ptr                             ),
        .wdata_reg          (rx_st_hdr_par_hip_r                ),
        .raddr              (rd_ptr                             ),
        .rdata              (rx_st_hdr_par_wire                 )
       );
      defparam hdr_par_memory .SIM_EMULATE = SIM_EMULATE;
      defparam hdr_par_memory .WIDTH = HDR_PAR_MEM_WIDTH;
      defparam hdr_par_memory .ADDR_WIDTH = MEM_SIZE_WIDTH;
    end

    alt_mlab control_memory (
      .wclk               (clk                               ),
      .wena               (rx_st_valid_hip_r                 ),
      .waddr_reg          (wr_ptr                            ),
      .wdata_reg          ( rx_st_control_in_wire            ),
      .raddr              (rd_ptr                            ),
      .rdata              (rx_st_control_out_wire            )
      );
    defparam control_memory .SIM_EMULATE = SIM_EMULATE;
    defparam control_memory .WIDTH = CONTROL_WIDTH;
    defparam control_memory .ADDR_WIDTH = MEM_SIZE_WIDTH;


    /*for (i=0; i < NUM_DATA_MEM; i=i+1 ) begin : data_fifo_memory
      altera_syncram  data_memory
      (
        .clock0         ( clk                              ),
        .wren_a         ( rx_st_valid_hip_r                ),
        .address_a      ( wr_ptr                           ),
        .data_a         ( rx_st_data_hip_r[32*i+31:32*i]   ),
        .rden_b         ( 1'b1                             ),
        .address_b      ( rd_ptr                           ),
        .q_b            ( rx_st_data_wire[32*i+31:32*i]    ),
        .aclr0          ( 1'b0                 ),
        .aclr1          ( 1'b0                 ),
        .sclr           ( 1'b0                 ), // Not used.
        .addressstall_a ( 1'b0                 ),
        .addressstall_b ( 1'b0                 ),
        .byteena_a      ( 1'b1                 ),
        .byteena_b      ( 1'b1                 ),
        .clock1         ( 1'b1                 ),
        .clocken0       ( 1'b1                 ),
        .clocken1       ( 1'b1                 ),
        .clocken2       ( 1'b1                 ),
        .clocken3       ( 1'b1                 ),
        .data_b         ( {32 {1'b1}}          ), // Not used.
        .eccencbypass   ( 1'b0                 ), // Not used.
        .eccencparity   ( 8'b0                 ), // Not used.
        .q_a            (                      ), // Not used.
        .rden_a         ( 1'b1                 ), // Not used.
        .wren_b         ( 1'b0                 ), // Not used.
        .eccstatus      (                      )  // Not used.
        );
      defparam
       data_memory.address_aclr_b              = "NONE",
       data_memory.address_reg_b               = "CLOCK0",
       data_memory.clock_enable_input_a        = "BYPASS",
       data_memory.clock_enable_input_b        = "BYPASS",
       data_memory.clock_enable_output_b       = "BYPASS",
       data_memory.enable_ecc                  = "FALSE",
       data_memory.ecc_pipeline_stage_enabled  = "FALSE",
       data_memory.intended_device_family      = FAMILY,
       data_memory.lpm_type                    = "altera_syncram",
       data_memory.maximum_depth               = 512,
       data_memory.numwords_a                  = MEM_SIZE,
       data_memory.numwords_b                  = MEM_SIZE,
       data_memory.operation_mode              = "DUAL_PORT",
       data_memory.outdata_aclr_b              = "NONE",
       data_memory.outdata_reg_b               = "UNREGISTERED",
       data_memory.power_up_uninitialized      = "FALSE",
       data_memory.ram_block_type              = MEM_TYPE,
       data_memory.widthad_a                   = MEM_SIZE_WIDTH,
       data_memory.widthad_b                   = MEM_SIZE_WIDTH,
       data_memory.width_a                     = 32,
       data_memory.width_b                     = 32,
       data_memory.width_byteena_a             = 1;
    end

      altera_syncram  hdr_memory
      (
        .clock0         ( clk                              ),
        .wren_a         ( rx_st_valid_hip_r                ),
        .address_a      ( wr_ptr                           ),
        .data_a         ( rx_st_hdr_hip_r                  ),
        .rden_b         ( 1'b1                             ),
        .address_b      ( rd_ptr                           ),
        .q_b            ( rx_st_hdr_wire                   ),
        .aclr0          ( 1'b0                 ),
        .aclr1          ( 1'b0                 ),
        .sclr           ( 1'b0                 ), // Not used.
        .addressstall_a ( 1'b0                 ),
        .addressstall_b ( 1'b0                 ),
        .byteena_a      ( 1'b1                 ),
        .byteena_b      ( 1'b1                 ),
        .clock1         ( 1'b1                 ),
        .clocken0       ( 1'b1                 ),
        .clocken1       ( 1'b1                 ),
        .clocken2       ( 1'b1                 ),
        .clocken3       ( 1'b1                 ),
        .data_b         ( {32 {1'b1}}          ), // Not used.
        .eccencbypass   ( 1'b0                 ), // Not used.
        .eccencparity   ( 8'b0                 ), // Not used.
        .q_a            (                      ), // Not used.
        .rden_a         ( 1'b1                 ), // Not used.
        .wren_b         ( 1'b0                 ), // Not used.
        .eccstatus      (                      )  // Not used.
        );
      defparam
       hdr_memory.address_aclr_b              = "NONE",
       hdr_memory.address_reg_b               = "CLOCK0",
       hdr_memory.clock_enable_input_a        = "BYPASS",
       hdr_memory.clock_enable_input_b        = "BYPASS",
       hdr_memory.clock_enable_output_b       = "BYPASS",
       hdr_memory.enable_ecc                  = "FALSE",
       hdr_memory.ecc_pipeline_stage_enabled  = "FALSE",
       hdr_memory.intended_device_family      = FAMILY,
       hdr_memory.lpm_type                    = "altera_syncram",
       hdr_memory.maximum_depth               = 512,
       hdr_memory.numwords_a                  = MEM_SIZE,
       hdr_memory.numwords_b                  = MEM_SIZE,
       hdr_memory.operation_mode              = "DUAL_PORT",
       hdr_memory.outdata_aclr_b              = "NONE",
       hdr_memory.outdata_reg_b               = "UNREGISTERED",
       hdr_memory.power_up_uninitialized      = "FALSE",
       hdr_memory.ram_block_type              = MEM_TYPE,
       hdr_memory.widthad_a                   = MEM_SIZE_WIDTH,
       hdr_memory.widthad_b                   = MEM_SIZE_WIDTH,
       hdr_memory.width_a                     = HDR_MEM_WIDTH,
       hdr_memory.width_b                     = HDR_MEM_WIDTH,
       hdr_memory.width_byteena_a             = 1;

    altera_syncram  control_memory
      (
        .clock0         ( clk                              ),
        .wren_a         ( rx_st_valid_hip_r                ),
        .address_a      ( wr_ptr                           ),
        .data_a         ( rx_st_control_in_wire            ),
        .rden_b         ( 1'b1                             ),
        .address_b      ( rd_ptr                           ),
        .q_b            ( rx_st_control_out_wire           ),
        .aclr0          ( 1'b0                 ),
        .aclr1          ( 1'b0                 ),
        .sclr           ( 1'b0                 ), // Not used.
        .addressstall_a ( 1'b0                 ),
        .addressstall_b ( 1'b0                 ),
        .byteena_a      ( 1'b1                 ),
        .byteena_b      ( 1'b1                 ),
        .clock1         ( 1'b1                 ),
        .clocken0       ( 1'b1                 ),
        .clocken1       ( 1'b1                 ),
        .clocken2       ( 1'b1                 ),
        .clocken3       ( 1'b1                 ),
        .data_b         ( {32 {1'b1}}          ), // Not used.
        .eccencbypass   ( 1'b0                 ), // Not used.
        .eccencparity   ( 8'b0                 ), // Not used.
        .q_a            (                      ), // Not used.
        .rden_a         ( 1'b1                 ), // Not used.
        .wren_b         ( 1'b0                 ), // Not used.
        .eccstatus      (                      )  // Not used.
        );
      defparam
       control_memory.address_aclr_b              = "NONE",
       control_memory.address_reg_b               = "CLOCK0",
       control_memory.clock_enable_input_a        = "BYPASS",
       control_memory.clock_enable_input_b        = "BYPASS",
       control_memory.clock_enable_output_b       = "BYPASS",
       control_memory.enable_ecc                  = "FALSE",
       control_memory.ecc_pipeline_stage_enabled  = "FALSE",
       control_memory.intended_device_family      = FAMILY,
       control_memory.lpm_type                    = "altera_syncram",
       control_memory.maximum_depth               = 512,
       control_memory.numwords_a                  = MEM_SIZE,
       control_memory.numwords_b                  = MEM_SIZE,
       control_memory.operation_mode              = "DUAL_PORT",
       control_memory.outdata_aclr_b              = "NONE",
       control_memory.outdata_reg_b               = "UNREGISTERED",
       control_memory.power_up_uninitialized      = "FALSE",
       control_memory.ram_block_type              = MEM_TYPE,
       control_memory.widthad_a                   = MEM_SIZE_WIDTH,
       control_memory.widthad_b                   = MEM_SIZE_WIDTH,
       control_memory.width_a                     = CONTROL_WIDTH,
       control_memory.width_b                     = CONTROL_WIDTH,
       control_memory.width_byteena_a             = 1;
    */

  endgenerate

   initial begin
     // VCS dump for "dve"
  // $vcdplusfile("wave.vpd");
   $vcdplusmemon;
   end


endmodule
