// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ns/1ps

module intel_pcie_adapter_stos #(
  parameter          BYPASS_ADAPTER        = 0,
  parameter          UI_WIDTH              = 128,
  parameter          HIP_WIDTH             = 256,
  parameter          PF_ENABLE             = 0,
  parameter          VF_ENABLE             = 0,
  parameter          HEADER_PREFIX_ENABLE  = 1,
  parameter          ROOT_PORT_MODE        = 0,
  parameter          ECC_ENABLE            = 0,
  parameter          MPS                   = 1024,
  parameter          EMPTY_WIDTH           = (UI_WIDTH>=256)  ? 3 : (UI_WIDTH==128)  ? 2 : 1,
  parameter          HIP_EMPTY_WIDTH       = (HIP_WIDTH>=256) ? 3 : (HIP_WIDTH==128) ? 2 : 1,
  parameter          FAMILY                = "Stratix 10"
) (

  input logic                       clk,
  input logic                       rst_n,  
  
  // RX Interface
  input  logic                       rx_st_ready,
  output logic [UI_WIDTH-1:0]        rx_st_data,
  output logic [UI_WIDTH/8-1:0]      rx_st_data_par,
  output logic [127:0]               rx_st_hdr,
  output logic [15:0]                rx_st_hdr_par,
  output logic [31:0]                rx_st_tlp_prfx,
  output logic [3:0]                 rx_st_tlp_prfx_par,
  output logic                       rx_st_vf_active,
  output logic [2:0]                 rx_st_func_num,
  output logic [10:0]                rx_st_vf_num,
  output logic                       rx_st_sop,
  output logic                       rx_st_eop,
  output logic                       rx_st_valid,
  output logic                       rx_st_tlp_abort,
  output logic [EMPTY_WIDTH-1:0]     rx_st_empty,
  output logic [2:0]                 rx_st_bar_range,
  
  output logic                       rx_st_ready_hip,
  input  logic [HIP_WIDTH-1:0]       rx_st_data_hip,
  input  logic [HIP_WIDTH/8-1:0]     rx_st_data_par_hip,
  input  logic [127:0]               rx_st_hdr_hip,
  input  logic [15:0]                rx_st_hdr_par_hip,
  input  logic [31:0]                rx_st_tlp_prfx_hip,
  input  logic [3:0]                 rx_st_tlp_prfx_par_hip,
  input  logic                       rx_st_vf_active_hip,
  input  logic [2:0]                 rx_st_func_num_hip,
  input  logic [10:0]                rx_st_vf_num_hip,
  input  logic                       rx_st_sop_hip,
  input  logic                       rx_st_eop_hip,
  input  logic                       rx_st_valid_hip,
  input  logic                       rx_st_tlp_abort_hip,
  input  logic [HIP_EMPTY_WIDTH-1:0] rx_st_empty_hip,
  input  logic [2:0]                 rx_st_bar_range_hip,
  
  
  //TX Interface
  output logic                       tx_st_ready,
  input  logic [UI_WIDTH-1:0]        tx_st_data,
  input  logic [UI_WIDTH/8-1:0]      tx_st_data_par,
  input  logic [127:0]               tx_st_hdr,
  input  logic [15:0]                tx_st_hdr_par,
  input  logic [31:0]                tx_st_tlp_prfx,
  input  logic [3:0]                 tx_st_tlp_prfx_par,
  input  logic                       tx_st_sop,
  input  logic                       tx_st_eop,
  input  logic                       tx_st_valid,
  input  logic                       tx_st_err,
  
  input  logic                       tx_st_ready_hip,
  output logic [HIP_WIDTH-1:0]       tx_st_data_hip,
  output logic [HIP_WIDTH/8-1:0]     tx_st_data_par_hip,
  output logic [127:0]               tx_st_hdr_hip,
  output logic [15:0]                tx_st_hdr_par_hip,
  output logic [31:0]                tx_st_tlp_prfx_hip,
  output logic [3:0]                 tx_st_tlp_prfx_par_hip,
  output logic                       tx_st_sop_hip,
  output logic                       tx_st_eop_hip,
  output logic                       tx_st_valid_hip,
  output logic                       tx_st_err_hip
  
);
generate
  if (BYPASS_ADAPTER == 1) begin
    assign rx_st_data            =    rx_st_data_hip;
    assign rx_st_data_par        =    rx_st_data_par_hip;
    assign rx_st_hdr             =    rx_st_hdr_hip;
    assign rx_st_hdr_par         =    rx_st_hdr_par_hip;
    assign rx_st_tlp_prfx        =    rx_st_tlp_prfx_hip;
    assign rx_st_tlp_prfx_par    =    rx_st_tlp_prfx_par_hip;
    assign rx_st_vf_active       =    rx_st_vf_active_hip;
    assign rx_st_func_num        =    rx_st_func_num_hip;
    assign rx_st_vf_num          =    rx_st_vf_num_hip;
    assign rx_st_sop             =    rx_st_sop_hip;
    assign rx_st_eop             =    rx_st_eop_hip;
    assign rx_st_valid           =    rx_st_valid_hip;
    assign rx_st_tlp_abort       =    rx_st_tlp_abort_hip;
    assign rx_st_empty           =    rx_st_empty_hip;
    assign rx_st_bar_range       =    rx_st_bar_range_hip;
    assign rx_st_ready_hip       =    rx_st_ready;
    
    assign tx_st_ready              =  tx_st_ready_hip;
    assign tx_st_data_hip           =  tx_st_data;
    assign tx_st_data_par_hip       =  tx_st_data_par;
    assign tx_st_hdr_hip            =  tx_st_hdr;
    assign tx_st_hdr_par_hip        =  tx_st_hdr_par;
    assign tx_st_tlp_prfx_hip       =  tx_st_tlp_prfx;
    assign tx_st_tlp_prfx_par_hip   =  tx_st_tlp_prfx_par;
    assign tx_st_sop_hip            =  tx_st_sop;
    assign tx_st_eop_hip            =  tx_st_eop;
    assign tx_st_valid_hip          =  tx_st_valid;
    assign tx_st_err_hip            =  tx_st_err;  
  
  end
  else begin

    intel_pcie_adapter_rx_if #(
      .UI_WIDTH              (UI_WIDTH              ),
      .HIP_WIDTH             (HIP_WIDTH             ),
      .PF_ENABLE             (PF_ENABLE             ),
      .VF_ENABLE             (VF_ENABLE             ),
      .HEADER_PREFIX_ENABLE  (HEADER_PREFIX_ENABLE  ),
      .ROOT_PORT_MODE        (ROOT_PORT_MODE        ),
      .MPS                   (MPS                   ),
      .ECC_ENABLE            (ECC_ENABLE            ),
      .EMPTY_WIDTH           (EMPTY_WIDTH           ),
      .HIP_EMPTY_WIDTH       (HIP_EMPTY_WIDTH       ),
      .FAMILY                (FAMILY                )  
    ) rx_if (
    
    .clk                     (clk ),
    .rst                     (~rst_n ),
    
    .rx_st_ready             (rx_st_ready         ),
    .rx_st_data              (rx_st_data          ),
    .rx_st_data_par          (rx_st_data_par      ),
    .rx_st_hdr               (rx_st_hdr           ),
    .rx_st_hdr_par           (rx_st_hdr_par       ),
    .rx_st_tlp_prfx          (rx_st_tlp_prfx      ),
    .rx_st_tlp_prfx_par      (rx_st_tlp_prfx_par  ),
    .rx_st_vf_active         (rx_st_vf_active     ),
    .rx_st_func_num          (rx_st_func_num      ),
    .rx_st_vf_num            (rx_st_vf_num        ),
    .rx_st_sop               (rx_st_sop           ),
    .rx_st_eop               (rx_st_eop           ),
    .rx_st_valid             (rx_st_valid         ),
    .rx_st_tlp_abort         (rx_st_tlp_abort     ),
    .rx_st_empty             (rx_st_empty         ),
    .rx_st_bar_range         (rx_st_bar_range     ),
    
    .rx_st_ready_hip         (rx_st_ready_hip         ),
    .rx_st_data_hip          (rx_st_data_hip          ),
    .rx_st_data_par_hip      (rx_st_data_par_hip      ),
    .rx_st_hdr_hip           (rx_st_hdr_hip           ),
    .rx_st_hdr_par_hip       (rx_st_hdr_par_hip       ),
    .rx_st_tlp_prfx_hip      (rx_st_tlp_prfx_hip      ),
    .rx_st_tlp_prfx_par_hip  (rx_st_tlp_prfx_par_hip  ),
    .rx_st_vf_active_hip     (rx_st_vf_active_hip     ),
    .rx_st_func_num_hip      (rx_st_func_num_hip      ),
    .rx_st_vf_num_hip        (rx_st_vf_num_hip        ),
    .rx_st_sop_hip           (rx_st_sop_hip           ),
    .rx_st_eop_hip           (rx_st_eop_hip           ),
    .rx_st_valid_hip         (rx_st_valid_hip         ),
    .rx_st_tlp_abort_hip     (rx_st_tlp_abort_hip     ),
    .rx_st_empty_hip         (rx_st_empty_hip         ),
    .rx_st_bar_range_hip     (rx_st_bar_range_hip     )  
    );
    
    intel_pcie_adapter_tx_if #(
      .UI_WIDTH              (UI_WIDTH              ),
      .HIP_WIDTH             (HIP_WIDTH             ),
      .PF_ENABLE             (PF_ENABLE             ),
      .VF_ENABLE             (VF_ENABLE             ),
      .HEADER_PREFIX_ENABLE  (HEADER_PREFIX_ENABLE  ),
      .ROOT_PORT_MODE        (ROOT_PORT_MODE        ),
      .MPS                   (MPS                   ),
      .ECC_ENABLE            (ECC_ENABLE            ),
      .FAMILY                (FAMILY                )  
    ) tx_if (
      .clk                   (clk ),
      .rst                   (~rst_n ),
      
      .tx_st_ready           (tx_st_ready       ),
      .tx_st_data            (tx_st_data        ),
      .tx_st_data_par        (tx_st_data_par    ),
      .tx_st_hdr             (tx_st_hdr         ),
      .tx_st_hdr_par         (tx_st_hdr_par     ),
      .tx_st_tlp_prfx        (tx_st_tlp_prfx    ),
      .tx_st_tlp_prfx_par    (tx_st_tlp_prfx_par),
      .tx_st_sop             (tx_st_sop         ),
      .tx_st_eop             (tx_st_eop         ),
      .tx_st_valid           (tx_st_valid       ),
      .tx_st_err             (tx_st_err         ),
    
      .tx_st_ready_hip        (tx_st_ready_hip       ),
      .tx_st_data_hip         (tx_st_data_hip        ),
      .tx_st_data_par_hip     (tx_st_data_par_hip    ),
      .tx_st_hdr_hip          (tx_st_hdr_hip         ),
      .tx_st_hdr_par_hip      (tx_st_hdr_par_hip     ),
      .tx_st_tlp_prfx_hip     (tx_st_tlp_prfx_hip    ),
      .tx_st_tlp_prfx_par_hip (tx_st_tlp_prfx_par_hip),
      .tx_st_sop_hip          (tx_st_sop_hip         ),
      .tx_st_eop_hip          (tx_st_eop_hip         ),
      .tx_st_valid_hip        (tx_st_valid_hip       ),
      .tx_st_err_hip          (tx_st_err_hip         )
    );
  end
  endgenerate


endmodule
