// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//==============================================================================
//
// (C) 2011-2017 Altera Corporation. All rights reserved.
// Your use of Altera Corporation's design tools, logic functions and other
// software and tools, and its AMPP partner logic functions, and any output
// files any of the foregoing (including device programming or simulation
// files), and any associated documentation or information are expressly subject
// to the terms and conditions of the Altera Program License Subscription
// Agreement, Altera MegaCore Function License Agreement, or other applicable
// license agreement, including, without limitation, that your use is for the
// sole purpose of programming logic devices manufactured by Altera and sold by
// Altera or its authorized distributors.  Please refer to the applicable
// agreement for further details.
//
//------------------------------------------------------------------------------
//
// Filename         : ptp_ref_ts_capture.sv
// Author           : Tholu Kiran Kumar <kktholu@intel.com>
// Created On       : Tue Jul 04/07/2017 11:27:24 AM MYT
//
//------------------------------------------------------------------------------
//
// $File: $
// $Revision: $
// $Date: $
// $Author: $
//
//==============================================================================

//------------------------------------------------------------------------------
//
// Description :-
// Submodules:
//            ptp_ckt_cal
//            ptp_tx_tam_capture
//            ptp_rx_tam_capture
//
//------------------------------------------------------------------------------

`timescale 1ns / 1ns
module alt_ehipc3_fm_sl_ptp_ref_ts_capture_sn  #(
    parameter SIM_MODE = 0
)
(
    // Configuration
    input  logic        i_enable_rsfec,
    input  logic        i_rx_sn_dn,
    input  logic [15:0] i_rx_ap_roll_dly,

    input  logic [31:0] i_apulse_period,
    input  logic [31:0] i_apulse_cnt_pow2,
    input  logic [31:0] i_apulse_cnt,
    input  logic [31:0] i_tx_ui,
    input  logic [31:0] i_rx_ui,
    input  logic [31:0] i_sn_pll_sclk_period,

    input  logic        i_ptp_sn_sclk,    
    input  logic        i_clk,
    input  logic        i_tx_clk,
    input  logic        i_rx_clk,
    input  logic        i_tx_rst_n,
    input  logic        i_rx_rst_n,
    input  logic        i_tx_rst_n1,
    input  logic        i_rx_rst_n1,
    
    input  logic        i_all_ready,

    input  logic [95:0] i_ptp_tx_tod,
    input  logic [95:0] i_ptp_rx_tod,
    input  logic        i_ptp_update,

    output logic        o_tx_error_valid,
    output logic [31:0] o_tx_error,

    output logic [31:0] o_rx_error,
    output logic        o_rx_error_valid,

    output logic [95:0] o_rx_tam,
    output logic [95:0] o_tx_tam,

    output logic        o_req_tx_tam_load,
    output logic        o_req_rx_tam_load,

    output logic        o_tx_dw_err,
    output logic        o_tx_dw_err2,
    output logic        o_rx_dw_err,
    output logic        o_rx_dw_err2,

    output logic        o_sclk,
    input  logic        i_tx_sclk_return,
    input  logic        i_tx_am,
    input  logic        i_rx_sclk_return,
    input  logic        i_rx_am,

    output logic        o_tx_calibrate,
    output logic        o_rx_calibrate
  );

  localparam  SYNC_FLOPS       = 4'd3;


  logic        rx_dw_err;
  logic        rx_dw_err2;
  logic        rx_calibrate;
  logic        txrx_calibrate;
  logic [31:0] rx_error;

  logic        trx_rst_n;
  logic  txrx_reset_n /* synthesis preserve dont_replicate dont_retime */;


  always @(posedge i_tx_clk) begin
    if (!i_tx_rst_n)
      txrx_reset_n <= 1'b0;
    else if (!trx_rst_n)
      txrx_reset_n <= 1'b0;
    else
      txrx_reset_n <= 1'b1;
  end 
 
  assign txrx_calibrate = (o_tx_calibrate || rx_calibrate);

  alt_ehipc3_fm_sl_ptp_sn_wire_delay #(
             .nTXRX(0)
	)    tx_ptp_sn_wire_delay  (
    .i_ptp_clk          (i_tx_clk),
    .i_ptp_sn_sclk      (i_ptp_sn_sclk),
    .i_reset_n          (txrx_reset_n),

    .i_calibrate        (txrx_calibrate),
    .i_sn_clb_trig      (o_sclk),
    .i_async_triger     (i_tx_sclk_return),
    .i_apulse_period    (i_apulse_period),
    .i_apulse_skip      ((SIM_MODE) ? {16'hFFFF, 16'hf} : {16'hFFFF, 16'hff}),
    .i_apulse_roll_dly  ({8'hf, 8'hff, 16'd8}),
    .i_apulse_cnt_pow2  (i_apulse_cnt_pow2),
    .i_apulse_cnt       (i_apulse_cnt),

    .i_all_ready       (i_all_ready),
    .i_ui              (i_tx_ui),
    .i_sn_pll_sclk_period    (i_sn_pll_sclk_period),

    .o_err             (o_tx_dw_err), 
    .o_err2            (o_tx_dw_err2),
    .o_sclk            (o_sclk),
    .o_calibrate       (o_tx_calibrate),
    .o_wd_error        (o_tx_error),
    .o_wd_error_valid  (o_tx_error_valid)

  );

  alt_ehipc3_fm_sl_ptp_sn_wire_delay #(
             .nTXRX(1)
	)    rx_ptp_sn_wire_delay  (
    .i_ptp_clk          (i_tx_clk),    
    .i_ptp_sn_sclk      (i_ptp_sn_sclk),
    .i_reset_n          (txrx_reset_n),

    .i_calibrate        (txrx_calibrate),
    .i_sn_clb_trig      (o_sclk),
    .i_async_triger     (i_rx_sclk_return),
    .i_apulse_period    (i_apulse_period),
    .i_apulse_skip      ((SIM_MODE) ? {16'hFFFF, 16'hf} : {16'hFFFF, 16'hff}),
    .i_apulse_roll_dly  ({8'hf, 8'hff, 16'd8}),
    .i_apulse_cnt_pow2  (i_apulse_cnt_pow2),
    .i_apulse_cnt       (i_apulse_cnt),

    .i_all_ready       (i_all_ready),
    .i_ui              (i_tx_ui),       
    .i_sn_pll_sclk_period    (i_sn_pll_sclk_period),

    .o_err             (rx_dw_err), 
    .o_err2            (rx_dw_err2),
    .o_sclk            (),
    .o_calibrate       (rx_calibrate),
    .o_wd_error        (rx_error),
    .o_wd_error_valid  (o_rx_error_valid)

  );

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) trx_rst_n_2ds (
        .clk                (i_tx_clk),
        .reset              (1'b0),
        .d                  (i_rx_rst_n),
        .q                  (trx_rst_n)
    );

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) rx_dw_err_2ds (
        .clk                (i_rx_clk),
        .reset              (1'b0),
        .d                  (rx_dw_err),
        .q                  (o_rx_dw_err)
    );

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) rx_dw_err2_2ds (
        .clk                (i_rx_clk),
        .reset              (1'b0),
        .d                  (rx_dw_err2),
        .q                  (o_rx_dw_err2)
    );

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) rx_calibrate_2ds (
        .clk                (i_rx_clk),
        .reset              (1'b0),
        .d                  (rx_calibrate),
        .q                  (o_rx_calibrate)
    ); 

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) rx_error_2ds (
        .clk                (i_rx_clk),
        .reset              (1'b0),
        .d                  (rx_error),
        .q                  (o_rx_error)
    ); 

  alt_ehipc3_fm_sl_ptp_tx_tam_capture_sn tx_tam_capture_sn (
    .i_enable_rsfec     (i_enable_rsfec),
    .i_clk              (i_tx_clk),
    .i_ptp_clk          (i_clk),
    .i_rst_n            (i_tx_rst_n),
    .i_ptp_tx_tod       (i_ptp_tx_tod),
    .i_ptp_update       (i_ptp_update), // unused
    .i_calibrate        (o_tx_calibrate),
    .i_tx_sclk_return   (i_tx_sclk_return),
    .i_tx_am            (i_tx_am),
    .o_tx_tam           (o_tx_tam),
    .o_req_tx_tam_load  (o_req_tx_tam_load)
  );

    alt_ehipc3_fm_sl_ptp_rx_tam_capture_sn rx_tam_capture_sn (
    .i_enable_rsfec     (i_enable_rsfec),
    .i_rx_sn_dn         (i_rx_sn_dn),
    .i_rx_ap_roll_dly   (i_rx_ap_roll_dly),
    .i_clk              (i_rx_clk),
    .i_ptp_clk          (i_clk),
    .i_rst_n            (i_rx_rst_n),
    .i_ptp_rx_tod       (i_ptp_rx_tod),
    .i_ptp_update       (i_ptp_update), // unused
    .i_calibrate        (o_rx_calibrate),
    .i_rx_sclk_return   (i_rx_sclk_return),
    .i_rx_am            (i_rx_am),
    .o_rx_tam           (o_rx_tam),
    .o_req_rx_tam_load  (o_req_rx_tam_load)
  );

  endmodule : alt_ehipc3_fm_sl_ptp_ref_ts_capture_sn

//============================================================================//
//                           E N D   O F   F I L E                            //
//============================================================================//
