// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//==============================================================================
//
// (C) 2011-2017 Altera Corporation. All rights reserved.
// Your use of Altera Corporation's design tools, logic functions and other
// software and tools, and its AMPP partner logic functions, and any output
// files any of the foregoing (including device programming or simulation
// files), and any associated documentation or information are expressly subject
// to the terms and conditions of the Altera Program License Subscription
// Agreement, Altera MegaCore Function License Agreement, or other applicable
// license agreement, including, without limitation, that your use is for the
// sole purpose of programming logic devices manufactured by Altera and sold by
// Altera or its authorized distributors.  Please refer to the applicable
// agreement for further details.
//
//------------------------------------------------------------------------------
//
// Filename         : alt_ehipc3_ptp.sv
// Author           : Tholu Kiran Kumar <kktholu@intel.com>
// Created On       : Tue Jun 13/06/2017 09:52:41 AM MYT
//
//------------------------------------------------------------------------------
//
// $File: $
// $Revision: $
// $Date: $
// $Author: $
//
//==============================================================================

//------------------------------------------------------------------------------
//
// Description :-
//               ptp_opcode_writer
//               ptp_ref_ts_capture
//               ptp_ts_converter
//------------------------------------------------------------------------------

`timescale 1ns / 1ns
module alt_ehipc3_fm_sl_ptp_sn #(
    parameter ENHANCED_PTP_ACCURACY = 0,
    parameter SIM_MODE = 0
) (
  // Configuration
  input  logic        i_enable_rsfec,
  input  logic        i_tx_preamble_pass,

 //1ns
  input  logic [0:0] i_ptp_sn_sclk,

  input logic [31:0] i_tx_apulse_period,
  input logic [31:0] i_tx_apulse_skip,
  input logic [31:0] i_tx_apulse_roll_dly,
  input logic [31:0] i_tx_apulse_cnt_pow2,
  input logic [31:0] i_tx_apulse_cnt,
  input logic [31:0] i_tx_sn_config,

  input logic [31:0] i_rx_apulse_period,
  input logic [31:0] i_rx_apulse_skip,
  input logic [31:0] i_rx_apulse_roll_dly,
  input logic [31:0] i_rx_apulse_cnt_pow2,
  input logic [31:0] i_rx_apulse_cnt,
  input logic [31:0] i_rx_sn_config,

  input logic [31:0] i_sn_pll_sclk_period,

  output logic [31:0]  o_txrx_dt_status,
  output logic [31:0]  o_rx_dt_reg,
  output logic [31:0]  o_tx_dt_reg,

  output logic [31:0]  o_tx_apulse_rt_period,  
  output logic [31:0]  o_tx_apulse_sn_period,  
  output logic [31:0]  o_tx_apulse_err_period,  
  output logic [31:0]  o_tx_apulse_err2_latency,  
  output logic [31:0]  o_rx_apulse_rt_period,  
  output logic [31:0]  o_rx_apulse_sn_period, 
  output logic [31:0]  o_rx_apulse_err_period, 
  output logic [31:0]  o_rx_apulse_err2_latency, 
  // System Interface
  input  logic        i_ptp_clk,
  input  logic        i_ptp_tx_clk,
  input  logic        i_ptp_rx_clk,
  input  logic        i_tx_rst_n,
  input  logic        i_rx_rst_n,

  input  logic        i_tx_rst_n1,
  input  logic        i_rx_rst_n1,
  
  input  logic        i_tx_rst_n2,
  input  logic        i_rx_rst_n2,
  
  input  logic        i_tx_rst_n3,
  input  logic        i_rx_rst_n3,
  
  input  logic        i_rx_lock,
  input  logic        i_pcs_fully_aligned,
  input  logic        i_all_ready,

  // AVMM CSR Bank Interface
  output logic        o_tx_error_valid,
  output logic [31:0] o_tx_error,
  output logic        o_rx_error_valid,
  output logic [31:0] o_rx_error,
  input  logic        i_ts_format,
  input  logic [31:0] i_tx_ui,
  input  logic [31:0] i_rx_ui,
  input  logic        i_req_tx_ui_load,
  input  logic        i_req_rx_ui_load,
  input  logic [31:0] i_tpiso,
  input  logic [31:0] i_rsipo,
  input  logic [31:0] i_rx_align_offset,

  // User Interface: TOD Interface
  input  logic [95:0] i_ptp_tx_tod,
  input  logic [95:0] i_ptp_rx_tod,
  input  logic        i_ptp_update,

  // User Interface: PTP Status
  output logic        o_tx_ptp_ready,
  output logic        o_rx_ptp_ready,

  // User Interface: TX Framing Interface
  input  logic        i_tx_valid,
  input  logic        i_tx_sop,
  input  logic        i_tx_eop,
  input  logic        i_tx_inframe,
  input  logic        i_tx_skip_crc,

  // User Interface: TX 1-step Command
  input  logic        i_ptp_ins_ets,
  input  logic        i_ptp_ins_cf,
  input  logic        i_ptp_zero_csum,
  input  logic        i_ptp_update_eb,
  input  logic        i_ptp_ts_format,
  input  logic        i_ptp_asym,
  input  logic [15:0] i_ptp_ts_offset,
  input  logic [15:0] i_ptp_cf_offset,
  input  logic [15:0] i_ptp_csum_offset,
  input  logic [15:0] i_ptp_eb_offset,
  input  logic [95:0] i_ptp_tx_its,

  // User Interface: TX 2-step Timestamp Request
  input  logic        i_ptp_ts_req,
  input  logic [7:0]  i_ptp_fp,

  // User Interface: TX 2-step Timestamp Return
  output logic        o_ptp_ets_valid,
  output logic [95:0] o_ptp_ets,
  output logic [7:0]  o_ptp_ets_fp,

  // User Interface: RX Timestamp
  output logic [95:0] o_ptp_rx_its,
  output logic        o_ptp_rx_its_valid,

  // EHIP Interface: TX 1-step and 2-step Command
  output logic [1:0]  o_ptp_tx_fp,
  output logic [2:0]  o_ptp_ins_type,
  output logic [2:0]  o_ptp_byte_offset,
  output logic [23:0] o_ptp_ts,
  
  // EHIP Interface: TX 2-step Timestamp
  input  logic        i_ptp_tx_ts_valid,
  input  logic [11:0] i_ptp_tx_ts,
  input  logic [1:0]  i_ptp_tx_fp,

  // EHIP Interface: RX Timestamp
  input  logic        i_rx_valid,
  input  logic        i_rx_inframe,
  input  logic [20:0] i_ptp_rx_ts,

  // EHIP Interface: Reference Timestamp
  output logic        o_sclk,
  output logic        o_sl_tx_sel_sclk_return,
  output logic        o_sl_rx_sel_sclk_return,
  input  logic        i_tx_sclk_return,
  input  logic        i_rx_sclk_return,
  input  logic        i_tx_am,
  input  logic        i_rx_am,
  input  logic        i_rx_am_dsk,
  
  // TAM capture and AM counts
  output logic [47:0] o_tx_tam_capture,
  output logic [15:0] o_tx_tam_count,
  output logic        o_tx_tam_cap_valid,
  output logic [47:0] o_rx_tam_capture,
  output logic [15:0] o_rx_tam_count,
  output logic        o_rx_tam_cap_valid,
    
  // Deskew status
  input  logic        i_ptp_rx_aib_dsk_done,
  output logic        o_ptp_rx_aib_dsk_done

  );


  localparam  SYNC_FLOPS       = 4'd3;

  localparam CLK_DIV =   66;

  localparam TX_TAM_ADJUST_CONST_DT_UI  = (CLK_DIV * (3 + 1)); // Subtract: 3 stages of synchronizer for i_tx_sclk_return, 1 stage of TOD sampling delay to meet timing

  localparam RX_TAM_ADJUST_CONST_DT_UI  = (CLK_DIV * (3 ));    // Subtract: 3 stages of synchronizer for i_rx_sclk_return

  localparam RX_FEC_ERR_ADJUST_CONST_UI = 0; // (CLK_DIV >> 1 );       // Subtract: Half cycle for constant error adujstment
  localparam RX_10G_ERR_ADJUST_CONST_UI = 0; // ((CLK_DIV >> 1 ) + (CLK_DIV >> 2 )); // Subtract: 75% cycle for constant error adujstment
  localparam RX_25G_ERR_ADJUST_CONST_UI = 0;       // Subtract: Half cycle for constant error adujstment

  localparam TX_TAM_ADJUST_CONST_UI  = (CLK_DIV * (3 + 1 - 1)) // Subtract: 3 stages of synchronizer for i_tx_sclk_return, 1 stage of TOD sampling delay to meet timing, add 1 to get latest TOD instead of previous cycle
                                    + (CLK_DIV >> 1);       // Subtract: Half cycle for sampling uncertainty

  localparam RX_TAM_ADJUST_CONST_UI  = (CLK_DIV * (3 - 1))     // Subtract: 3 stages of synchronizer for i_rx_sclk_return, add 1 to get latest TOD instead of previous cycle
                                    + (CLK_DIV >> 1);       // Subtract: Half cycle for sampling uncertainty


  localparam TX_TAM_ADJUST_UI     = (32 * 2) // Subtract: 2 div32 clock period due to sclk pulse pipeline delay
                                    - (2)    // Add: Block sync header
                                    + TX_TAM_ADJUST_CONST_UI;   

  localparam RX_TAM_ADJUST_UI     = (32 * 2) // Subtract: 2 div32 clock period due to sclk pulse pipeline delay
                                    + (32)   // Subtract: 1 div32 clock period due to serial data to parallel data delay
                                    - (2);   // Add: Block sync header
  localparam RX_TAM_ADJUST_UI_FEC = (32 * 2) // Subtract: 2 div32 clock period due to sclk pulse pipeline delay
                                    + (32)   // Subtract: 1 div32 clock period due to serial data to parallel data delay
                                    + (32)   // Subtract: 1 div32 clock period in RSFEC mode
                                    - (2);   // Add: Block sync header

  localparam RX_TAM_ADJUST_UI_ADJ = RX_TAM_ADJUST_UI + 66  // Subtract: Adjustment in RSFEC mode when AM position shifted due to EOP
                                    + 32;                  // Subtract: 1 div32 clock period in RSFEC mode

  localparam TX_TAM_ADJUST_UI2    = (32 * 2) // Subtract: 2 div32 clock period due to sclk pulse pipeline delay
                                    - (2)    // Add: Block sync header
                                     + TX_TAM_ADJUST_CONST_DT_UI; 

  localparam RX_TAM_ADJUST_UI2     = RX_TAM_ADJUST_UI + RX_TAM_ADJUST_CONST_DT_UI; 
  localparam RX_TAM_ADJUST_UI_FEC2 = RX_TAM_ADJUST_UI_FEC + RX_TAM_ADJUST_CONST_DT_UI; 

  localparam RX_TAM_ADJUST_UI_ADJ2 = RX_TAM_ADJUST_UI2 + 66  // Subtract: Adjustment in RSFEC mode when AM position shifted due to EOP
                                    + 32;                    // Subtract: 1 div32 clock period in RSFEC mode

  localparam PIPE1 = 8;

  logic [95:0] tx_tam;
  logic [95:0] rx_tam;
  logic        req_tx_tam_load;
  logic        req_rx_tam_load;

  (* altera_attribute = {" -name SYNCHRONIZER_IDENTIFICATION OFF "} *) logic [PIPE1*32-1:0] tx_ui_r;
  (* altera_attribute = {" -name SYNCHRONIZER_IDENTIFICATION OFF "} *) logic [PIPE1*32-1:0] rx_ui_r;
  logic [PIPE1*12-1:0] tx_tam_adjust_ui_r;
  logic [PIPE1*12-1:0] rx_tam_adjust_ui_r;
  logic [PIPE1*12-1:0] rx_tam_adjust_ui_adj_r;

  logic [PIPE1*32-1:0] tx_tam_adjust_avg_r;
  logic [39:0] tx_tam_adjust_avg_f24;
  logic [39:0] tx_tam_adjust_avg_f24r;
  logic [39:0] tx_tam_adjust_avg_f24rr;

  logic [PIPE1*32-1:0] rx_tam_adjust_avg_r;
  logic [39:0] rx_tam_adjust_avg_f24;
  logic [39:0] rx_tam_adjust_avg_f24r;
  logic [39:0] rx_tam_adjust_avg_f24rr;

  logic [39:0] tx_tam_adjust;
  logic [39:0] rx_tam_adjust_no_adj;
  logic [39:0] rx_tam_adjust_adj;

  logic [39:0] tx_tam_adjust_r;
  logic [39:0] rx_tam_adjust_no_adj_r;
  logic [39:0] rx_tam_adjust_adj_r;

  logic [39:0] tx_tam_adjust_rr;
  logic [39:0] rx_tam_adjust_no_adj_rr;
  logic [39:0] rx_tam_adjust_adj_rr;

  logic [40:0] tx_tam_adjust_total;
  logic [40:0] rx_tam_adjust_no_adj_total;
  logic [40:0] rx_tam_adjust_adj_total;

  logic [40:0] tx_tam_adjust_total_r;
  logic [40:0] rx_tam_adjust_no_adj_total_r;
  logic [40:0] rx_tam_adjust_adj_total_r;

  logic [40:0] tx_tam_adjust_total_rr;
  logic [40:0] rx_tam_adjust_no_adj_total_rr;
  logic [40:0] rx_tam_adjust_adj_total_rr;

  logic [41:0] tx_tam_adjust_w_err;
  logic [41:0] rx_tam_adjust_no_adj_w_err;
  logic [41:0] rx_tam_adjust_adj_w_err;

  logic [41:0] tx_tam_adjust_w_err_r;
  logic [41:0] rx_tam_adjust_no_adj_w_err_r;
  logic [41:0] rx_tam_adjust_adj_w_err_r;

  logic [41:0] tx_tam_adjust_final;
  logic [41:0] rx_tam_adjust_final;

  logic [31:0] tx_tam_adj;
  logic [31:0] rx_tam_adj;

  logic        tx_calibrate;
  logic        rx_calibrate;
  logic        tx_calibrate_p;
  logic        rx_calibrate_p;

  logic        modify_ts;

/////// sub-nano signals /////////
   logic [15:0] tx_dt_status;
   logic [31:0] tx_tam_adjust_avg;
   logic [31:0] tx_tam_adjust_avg_p;
   logic        tx_measure_done;
   logic        tx_measure_done_p;
   logic        tx_roll_over_trig;

   logic [15:0] rx_dt_status;
   logic [31:0] rx_tam_adjust_avg;
   logic [31:0] rx_tam_adjust_avg_p;
   logic        rx_measure_done;
   logic        rx_measure_done_p;
   logic        rx_roll_over_trig;

   logic        rx_sn_dn;
   logic        rx_am;
   logic        tx_am;

   logic        tx_roll_over_sclk_return;
   logic        rx_roll_over_sclk_return;

   logic        tx_sn_en;
   logic        rx_sn_en;

   logic        tx_sn_ready;
   logic        rx_sn_ready;

   logic        tx_sn_ready_p;
   logic        rx_sn_ready_p;

   logic        tx_ptp_ready;
   logic        rx_ptp_ready;
   logic        tx_ptp_ready_x;
   logic        rx_ptp_ready_x;

   logic        tx_sn_err;
   logic        rx_sn_err;

   logic        tx_sn_err2;
   logic        rx_sn_err2;

   logic        tx_tam_valid;
   logic        rx_tam_valid;

   logic        tx_dw_err;
   logic        tx_dw_err2;
   logic        rx_dw_err;
   logic        rx_dw_err2;
   logic [31:0] tx_error_p;
   logic [31:0] rx_error_p;

   logic        rx_rst_rxx_n;
   logic        tx_rst_txx_n;

  assign o_sl_tx_sel_sclk_return = ~tx_calibrate_p;
  assign o_sl_rx_sel_sclk_return = ~rx_calibrate_p;

  always @(posedge i_ptp_clk) begin
    if(!i_tx_rst_n) begin
        tx_ui_r                 <= {PIPE1{32'h0}};
        tx_tam_adjust_ui_r      <= {PIPE1{12'h0}};
        tx_tam_adjust_avg_r     <= {PIPE1{32'h0}};
        tx_tam_adjust_avg_f24   <= 40'h0;
        tx_tam_adjust_avg_f24r  <= 40'h0;
        tx_tam_adjust_avg_f24rr <= 40'h0;
        tx_tam_adjust           <= 40'h0;
        tx_tam_adjust_r         <= 40'h0;
        tx_tam_adjust_rr        <= 40'h0;
        tx_tam_adjust_total     <= 41'h0;
        tx_tam_adjust_total_r   <= 41'h0;
        tx_tam_adjust_total_rr  <= 41'h0;
        tx_tam_adjust_w_err     <= 42'h0;
        tx_tam_adjust_w_err_r   <= 42'h0;
    end
    else begin
        tx_ui_r                 <= {tx_ui_r[(PIPE1-1)*32-1:0], i_tx_ui};

        tx_tam_adjust_ui_r      <= {tx_tam_adjust_ui_r[(PIPE1-1)*12-1:0], tx_calibrate_p ? {12{1'b1}} : (tx_measure_done_p && tx_sn_en) ? TX_TAM_ADJUST_UI2[12-1:0] : TX_TAM_ADJUST_UI[12-1:0]};
        tx_tam_adjust           <= tx_tam_adjust_ui_r[(PIPE1)*12-1:(PIPE1-1)*12] * tx_ui_r[(PIPE1)*32-1:(PIPE1-1)*32];

        tx_tam_adjust_avg_r     <= {tx_tam_adjust_avg_r[(PIPE1-1)*32-1:0], (tx_measure_done_p && tx_sn_en) ? tx_tam_adjust_avg_p : 32'd0};
        tx_tam_adjust_avg_f24   <= {tx_tam_adjust_avg_r[(PIPE1)*32-1:(PIPE1-1)*32], 8'h00};
        tx_tam_adjust_avg_f24r  <= tx_tam_adjust_avg_f24;
        tx_tam_adjust_avg_f24rr <= tx_tam_adjust_avg_f24r;

        tx_tam_adjust_r         <= tx_tam_adjust;
        tx_tam_adjust_rr        <= tx_tam_adjust_r;

        tx_tam_adjust_total     <= tx_tam_adjust_avg_f24rr + tx_tam_adjust_rr;

        tx_tam_adjust_total_r   <= tx_tam_adjust_total;
        tx_tam_adjust_total_rr  <= tx_tam_adjust_total_r;
        tx_tam_adjust_w_err     <= {tx_error_p, 8'h00} + tx_tam_adjust_total_rr;
        tx_tam_adjust_w_err_r   <= tx_tam_adjust_w_err;
    end
  end

  always @(posedge i_ptp_clk) begin
    if(!i_rx_rst_n) begin
        rx_ui_r                         <= {PIPE1{32'h0}};
        rx_tam_adjust_ui_r              <= {PIPE1{8'h0}};
        rx_tam_adjust_ui_adj_r          <= {PIPE1{8'h0}};

        rx_tam_adjust_avg_r             <= {PIPE1{32'h0}};
        rx_tam_adjust_avg_f24           <= 40'h0;
        rx_tam_adjust_avg_f24r          <= 40'h0;
        rx_tam_adjust_avg_f24rr         <= 40'h0;

        rx_tam_adjust_no_adj            <= 40'h0;
        rx_tam_adjust_adj               <= 40'h0;

        rx_tam_adjust_no_adj_r          <= 40'h0;
        rx_tam_adjust_adj_r             <= 40'h0;

        rx_tam_adjust_no_adj_rr         <= 40'h0;
        rx_tam_adjust_adj_rr            <= 40'h0;

        rx_tam_adjust_no_adj_total      <= 41'h0;
        rx_tam_adjust_adj_total         <= 41'h0;

        rx_tam_adjust_no_adj_total_r    <= 41'h0;
        rx_tam_adjust_adj_total_r       <= 41'h0;

        rx_tam_adjust_no_adj_total_rr   <= 41'h0;
        rx_tam_adjust_adj_total_rr      <= 41'h0;

        rx_tam_adjust_no_adj_w_err      <= 42'h0;
        rx_tam_adjust_adj_w_err         <= 42'h0;

        rx_tam_adjust_no_adj_w_err_r    <= 42'h0;
        rx_tam_adjust_adj_w_err_r       <= 42'h0;
    end
    else begin
        rx_ui_r                         <= {rx_ui_r[(PIPE1-1)*32-1:0], i_rx_ui};

        rx_tam_adjust_ui_r              <= {rx_tam_adjust_ui_r[(PIPE1-1)*12-1:0], rx_calibrate_p ? {12{1'b1}} 
                                            :(rx_measure_done_p && rx_sn_en) ? ( ( i_enable_rsfec ? RX_TAM_ADJUST_UI_FEC2[12-1:0] : RX_TAM_ADJUST_UI2[12-1:0] ) - 
                                                                               ( (i_rx_sn_config[17:16] == 2'b01) ? RX_FEC_ERR_ADJUST_CONST_UI[12-1:0]  :
                                                                                 (i_rx_sn_config[17:16] == 2'b10) ? RX_10G_ERR_ADJUST_CONST_UI[12-1:0]  :
                                                                                                                    RX_25G_ERR_ADJUST_CONST_UI[12-1:0] ) )
                                            : ( i_enable_rsfec ? RX_TAM_ADJUST_UI_FEC[12-1:0] : RX_TAM_ADJUST_UI[12-1:0] ) };

        rx_tam_adjust_ui_adj_r          <= {rx_tam_adjust_ui_adj_r[(PIPE1-1)*12-1:0], rx_calibrate_p ? {12{1'b1}} 
                                            :(rx_measure_done_p && rx_sn_en) ? ( RX_TAM_ADJUST_UI_ADJ2[12-1:0] - 
                                                                               ( (i_rx_sn_config[17:16] == 2'b01) ? RX_FEC_ERR_ADJUST_CONST_UI[12-1:0]  :
                                                                                 (i_rx_sn_config[17:16] == 2'b10) ? RX_10G_ERR_ADJUST_CONST_UI[12-1:0]  :
                                                                                                                    RX_25G_ERR_ADJUST_CONST_UI[12-1:0] ) )
                                            : RX_TAM_ADJUST_UI_ADJ[12-1:0]};

        rx_tam_adjust_no_adj            <= rx_tam_adjust_ui_r[(PIPE1)*12-1:(PIPE1-1)*12] * rx_ui_r[(PIPE1)*32-1:(PIPE1-1)*32];
        rx_tam_adjust_adj               <= rx_tam_adjust_ui_adj_r[(PIPE1)*12-1:(PIPE1-1)*12] * rx_ui_r[(PIPE1)*32-1:(PIPE1-1)*32];

        rx_tam_adjust_avg_r             <= {rx_tam_adjust_avg_r[(PIPE1-1)*32-1:0], (rx_measure_done_p && rx_sn_en) ? rx_tam_adjust_avg_p : 32'd0};
        rx_tam_adjust_avg_f24           <= {rx_tam_adjust_avg_r[(PIPE1)*32-1:(PIPE1-1)*32], 8'h00};
        rx_tam_adjust_avg_f24r          <= rx_tam_adjust_avg_f24;
        rx_tam_adjust_avg_f24rr         <= rx_tam_adjust_avg_f24r;

        rx_tam_adjust_no_adj_r          <= rx_tam_adjust_no_adj;
        rx_tam_adjust_adj_r             <= rx_tam_adjust_adj;

        rx_tam_adjust_no_adj_rr         <= rx_tam_adjust_no_adj_r;
        rx_tam_adjust_adj_rr            <= rx_tam_adjust_adj_r;

        rx_tam_adjust_no_adj_total      <= rx_tam_adjust_avg_f24rr + rx_tam_adjust_no_adj_rr;
        rx_tam_adjust_adj_total         <= rx_tam_adjust_avg_f24rr + rx_tam_adjust_adj_rr;

        rx_tam_adjust_no_adj_total_r    <= rx_tam_adjust_no_adj_total;
        rx_tam_adjust_adj_total_r       <= rx_tam_adjust_adj_total;

        rx_tam_adjust_no_adj_total_rr   <= rx_tam_adjust_no_adj_total_r;
        rx_tam_adjust_adj_total_rr      <= rx_tam_adjust_adj_total_r;

        rx_tam_adjust_no_adj_w_err      <= {rx_error_p, 8'h00} + rx_tam_adjust_no_adj_total_rr;
        rx_tam_adjust_adj_w_err         <= {rx_error_p, 8'h00} + rx_tam_adjust_adj_total_rr;

        rx_tam_adjust_no_adj_w_err_r    <= rx_tam_adjust_no_adj_w_err;
        rx_tam_adjust_adj_w_err_r       <= rx_tam_adjust_adj_w_err;
    end
  end

  assign tx_tam_adjust_final = tx_tam_adjust_w_err_r;
  assign rx_tam_adjust_final = modify_ts ? rx_tam_adjust_adj_w_err_r : rx_tam_adjust_no_adj_w_err_r;

// ---------------------------------------------------------------------------------------------------------------------------------
// 10/Dec/2018 : slim35 : https://hsdes.intel.com/appstore/article/#/2205692107 : Crete 3 hard logic costs too much soft logic.
// Reduce alt_ehipc3_fm_sl_ptp_compute_ts pipeline from 12 to 4 and hence the param needs to be revised.
// In addition to pipeline reduction, the new code is using bit31 of correction_value as part of the value instead of signed bit previously.
// Therefore, the bit31 of input correction_value has to be tied to 1'b0 instead of 1'b1.
// The bit31 is previously tied to 1'b1 to always do a subtraction.
// ---------------------------------------------------------------------------------------------------------------------------------
//  assign tx_tam_adj = {1'b1, tx_tam_adjust_final[38:8]};
//  assign rx_tam_adj = {1'b1, rx_tam_adjust_final[38:8]};
  assign tx_tam_adj = {1'b0, tx_tam_adjust_final[38:8]};
  assign rx_tam_adj = {1'b0, rx_tam_adjust_final[38:8]};
// ---------------------------------------------------------------------------------------------------------------------------------

  alt_ehipc3_fm_sl_ptp_tam_adj_detect tam_adj_detect_u(
    .i_enable_rsfec     (i_enable_rsfec),
    .i_clk              (i_ptp_clk),
    .i_rst_n            (i_rx_rst_n),
    .i_rx_valid         (i_rx_valid),
    .i_rx_am            (i_rx_am_dsk),
    .i_pcs_fully_aligned(i_pcs_fully_aligned),
    .o_modify_ts        (modify_ts)
  );

  alt_ehipc3_fm_sl_ptp_ref_ts_capture_sn #(
    .SIM_MODE            (SIM_MODE)
  ) ptp_ref_ts_capture_u_sn (
    .i_enable_rsfec     (i_enable_rsfec),
    .i_rx_sn_dn         (rx_sn_dn),
    .i_rx_ap_roll_dly   (i_rx_apulse_roll_dly[15:0]),
    .i_apulse_period    (i_rx_apulse_period),
    .i_apulse_cnt_pow2  (i_rx_apulse_cnt_pow2),
    .i_apulse_cnt       (i_rx_apulse_cnt),
    .i_tx_ui            (i_tx_ui),
    .i_rx_ui            (i_rx_ui),
    .i_sn_pll_sclk_period   (i_sn_pll_sclk_period),
    .i_ptp_sn_sclk      (i_ptp_sn_sclk),
    .i_clk              (i_ptp_clk),
    .i_tx_clk           (i_ptp_tx_clk),
    .i_rx_clk           (i_ptp_rx_clk),
    .i_tx_rst_n         (tx_rst_txx_n),   //(i_tx_rst_n),
    .i_rx_rst_n         (rx_rst_rxx_n), //(i_rx_rst_n2),
    .i_tx_rst_n1        (tx_rst_txx_n),  //(i_tx_rst_n1),
    .i_rx_rst_n1        (rx_rst_rxx_n),  //(i_rx_rst_n1),
    .i_all_ready        (i_all_ready),
    .i_ptp_tx_tod       (i_ptp_tx_tod),
    .i_ptp_rx_tod       (i_ptp_rx_tod),
    .i_ptp_update       (i_ptp_update),
    .o_tx_dw_err        (tx_dw_err),
    .o_tx_dw_err2       (tx_dw_err2),
    .o_rx_dw_err        (rx_dw_err),
    .o_rx_dw_err2       (rx_dw_err2),
    .o_tx_error_valid   (o_tx_error_valid),
    .o_tx_error         (o_tx_error),
    .o_rx_error         (o_rx_error),
    .o_rx_error_valid   (o_rx_error_valid),
    .o_rx_tam           (rx_tam),
    .o_tx_tam           (tx_tam),
    .o_req_tx_tam_load  (req_tx_tam_load),
    .o_req_rx_tam_load  (req_rx_tam_load),
    .o_sclk             (o_sclk),
    .i_tx_sclk_return   (tx_roll_over_sclk_return),  // (i_tx_sclk_return),
    .i_tx_am            (tx_am),
    .i_rx_sclk_return   (rx_roll_over_sclk_return),  // (i_rx_sclk_return),
    .i_rx_am            (rx_am),
    .o_tx_calibrate     (tx_calibrate),
    .o_rx_calibrate     (rx_calibrate));
    

  alt_ehipc3_fm_sl_ptp_ts_converter_sn ptp_ts_converter_u_sn(
    .i_clk              (i_ptp_clk),
    .i_tx_rst_n         (i_tx_rst_n3),
    .i_rx_rst_n         (i_rx_rst_n3),
    .i_ptp_tx_ts_valid  (i_ptp_tx_ts_valid),
    .i_ptp_tx_ts        (i_ptp_tx_ts),
    .i_ptp_tx_fp        (i_ptp_tx_fp),
    .i_rx_valid         (i_rx_valid),
    .i_rx_inframe       (i_rx_inframe),
    .i_ptp_rx_ts        (i_ptp_rx_ts),
    .i_ts_format        (i_ts_format),
    .i_tx_tod           (i_ptp_tx_tod),
    .i_rx_tod           (i_ptp_rx_tod),
    .o_ptp_ets_valid    (o_ptp_ets_valid),
    .o_ptp_ets          (o_ptp_ets),
    .o_ptp_ets_fp       (o_ptp_ets_fp),
    .o_ptp_rx_its       (o_ptp_rx_its),
    .o_ptp_rx_its_valid (o_ptp_rx_its_valid)
  );

alt_ehipc3_fm_sl_ptp_opcode_writer #(
    .ENHANCED_PTP_ACCURACY     (ENHANCED_PTP_ACCURACY),
    .SIM_MODE            (SIM_MODE)
  ) ptp_opcode_writer_u (
    .i_enable_rsfec      (i_enable_rsfec),
    .i_tx_preamble_pass  (i_tx_preamble_pass),
    .i_clk               (i_ptp_clk),
    .i_rst_n             (i_tx_rst_n2),
    .i_rx_lock           (i_rx_lock),
    .i_ptp_fp            (i_ptp_fp),
    .i_ptp_ins_ets       (i_ptp_ins_ets),
    .i_ptp_ins_cf        (i_ptp_ins_cf),
    .i_ptp_zero_csum     (i_ptp_zero_csum),
    .i_ptp_update_eb     (i_ptp_update_eb),
    .i_ptp_ts_format     (i_ptp_ts_format),
    .i_ptp_asym          (i_ptp_asym),
    .i_ptp_ts_offset     (i_ptp_ts_offset),
    .i_ptp_cf_offset     (i_ptp_cf_offset),
    .i_ptp_csum_offset   (i_ptp_csum_offset),
    .i_ptp_eb_offset     (i_ptp_eb_offset),
    .i_ptp_tx_its        (i_ptp_tx_its),
    .i_ptp_ts_req        (i_ptp_ts_req),
    .i_tx_valid          (i_tx_valid),
    .i_tx_sop            (i_tx_sop),
    .i_tx_eop            (i_tx_eop),
    .i_tx_inframe        (i_tx_inframe),
    .i_tx_skip_crc       (i_tx_skip_crc),
    .i_req_tx_tam_load   (req_tx_tam_load),
    .i_req_rx_tam_load   (req_rx_tam_load),
    .i_req_tx_ui_load    (i_req_tx_ui_load),
    .i_req_rx_ui_load    (i_req_rx_ui_load),
    .i_tx_tam            (tx_tam),
    .i_tx_tam_adj        (tx_tam_adj),
    .i_rx_tam_adj        (rx_tam_adj),
    .i_rx_tam            (rx_tam),
    .i_tx_ui             (i_tx_ui),
    .i_rx_ui             (i_rx_ui),
    .o_ptp_tx_fp         (o_ptp_tx_fp),
    .o_ptp_ins_type      (o_ptp_ins_type),
    .o_ptp_byte_offset   (o_ptp_byte_offset),
    .o_tx_ptp_ready      (tx_ptp_ready),
    .o_rx_ptp_ready      (rx_ptp_ready),
    .o_ptp_ts            (o_ptp_ts),
    .o_tx_tam_valid      (tx_tam_valid),
    .o_rx_tam_valid      (rx_tam_valid),
    .i_ptp_rx_aib_dsk_done(i_ptp_rx_aib_dsk_done),
    .o_ptp_rx_aib_dsk_done(o_ptp_rx_aib_dsk_done)
  );

  alt_ehipc3_fm_sl_ptp_tam_capture_count ptp_tam_capture_count_u(    
    .i_tx_clk           (i_ptp_clk),
    .i_tx_rst_n         (i_tx_rst_n),
    .i_tx_tam           (tx_tam[47:0]),
    .i_tx_tam_load      (req_tx_tam_load),
    .o_tx_tam_capture   (o_tx_tam_capture),
    .o_tx_tam_count     (o_tx_tam_count),
    .o_tx_tam_cap_valid (o_tx_tam_cap_valid),
    .i_rx_clk           (i_ptp_clk),
    .i_rx_rst_n         (i_rx_rst_n),
    .i_rx_tam           (rx_tam[47:0]),
    .i_rx_tam_load      (req_rx_tam_load), 
    .o_rx_tam_capture   (o_rx_tam_capture),
    .o_rx_tam_count     (o_rx_tam_count),
    .o_rx_tam_cap_valid (o_rx_tam_cap_valid)
  );

  alt_ehipc3_fm_sl_ptp_sub_nano #(
             .nTXRX(0)
	)    tx_ptp_sub_nano  (
    .i_ptp_clk          (i_ptp_tx_clk),
    .i_ptp_sn_sclk      (i_ptp_sn_sclk),
    .i_reset_n          (tx_rst_txx_n),
    .i_ptp_ready        (tx_ptp_ready),
    .i_am               (i_tx_am),
    .i_tam_valid        (tx_tam_valid),
    .i_async_triger     (i_tx_sclk_return),
    .i_apulse_period    (i_tx_apulse_period),
    .i_apulse_skip      (i_tx_apulse_skip),
    .i_apulse_roll_dly  (i_tx_apulse_roll_dly),
    .i_apulse_cnt_pow2  (i_tx_apulse_cnt_pow2),
    .i_apulse_cnt       (i_tx_apulse_cnt),
    .i_sn_config        (i_tx_sn_config),
    .i_sn_pll_sclk_period   (i_sn_pll_sclk_period),
    .o_am               (tx_am),
    .o_sn_en            (tx_sn_en),
    .o_sn_ready         (tx_sn_ready),
    .o_err              (tx_sn_err), 
    .o_err2             (tx_sn_err2), 
    .o_apulse_rt_period (o_tx_apulse_rt_period),
    .o_apulse_sn_period (o_tx_apulse_sn_period),
    .o_apulse_period_err (o_tx_apulse_err_period),
    .o_latency_err2     (o_tx_apulse_err2_latency),
    .o_measure_done     (tx_measure_done),
    .o_roll_over_trig   (tx_roll_over_trig),
    .o_tam_adjust_avg   (tx_tam_adjust_avg)
  );
  
  alt_ehipc3_fm_sl_ptp_sub_nano #(
             .nTXRX(1)
	)   rx_ptp_sub_nano  (
    .i_ptp_clk          (i_ptp_rx_clk),
    .i_ptp_sn_sclk      (i_ptp_sn_sclk),
    .i_reset_n          (rx_rst_rxx_n),
    .i_ptp_ready        (rx_ptp_ready),
    .i_am               (i_rx_am),
    .i_tam_valid        (rx_tam_valid),
    .i_async_triger     (i_rx_sclk_return),
    .i_apulse_period    (i_rx_apulse_period),
    .i_apulse_skip      (i_rx_apulse_skip),
    .i_apulse_roll_dly  (i_rx_apulse_roll_dly),
    .i_apulse_cnt_pow2  (i_rx_apulse_cnt_pow2),
    .i_apulse_cnt       (i_rx_apulse_cnt),
    .i_sn_config        (i_rx_sn_config),
    .i_sn_pll_sclk_period   (i_sn_pll_sclk_period),
    .o_am               (rx_am),
    .o_sn_en            (rx_sn_en),
    .o_sn_ready         (rx_sn_ready),
    .o_err              (rx_sn_err), 
    .o_err2             (rx_sn_err2), 
    .o_apulse_rt_period (o_rx_apulse_rt_period),
    .o_apulse_sn_period (o_rx_apulse_sn_period),
    .o_apulse_period_err (o_rx_apulse_err_period),
    .o_latency_err2     (o_rx_apulse_err2_latency),
    .o_measure_done     (rx_measure_done),
    .o_roll_over_trig   (rx_roll_over_trig),
    .o_tam_adjust_avg   (rx_tam_adjust_avg)
  );
  
  assign rx_sn_dn = (rx_measure_done && rx_sn_en);

  assign tx_roll_over_sclk_return = (tx_calibrate) ? i_tx_sclk_return : (tx_measure_done && tx_sn_en) ? tx_roll_over_trig : i_tx_sclk_return;

  assign rx_roll_over_sclk_return = (rx_calibrate) ? i_rx_sclk_return : (rx_measure_done && rx_sn_en) ? rx_roll_over_trig : i_rx_sclk_return;

  assign o_txrx_dt_status = {rx_dt_status, tx_dt_status};

  always@(posedge i_ptp_tx_clk or negedge tx_rst_txx_n) 
  begin	
    if(!tx_rst_txx_n)
      tx_dt_status <= {16{1'b0}};
    else
      tx_dt_status <= {7'h00, tx_sn_en, tx_dw_err2, tx_dw_err, tx_sn_err2, tx_sn_err, tx_sn_ready, tx_measure_done, tx_ptp_ready_x, ~tx_calibrate};
  end

  always@(posedge i_ptp_rx_clk or negedge rx_rst_rxx_n) 
  begin	
    if(!rx_rst_rxx_n)
      rx_dt_status <= {16{1'b0}};
    else
      rx_dt_status <= {7'h00, rx_sn_en, rx_dw_err2, rx_dw_err, rx_sn_err2, rx_sn_err, rx_sn_ready, rx_measure_done, rx_ptp_ready_x, ~rx_calibrate};
  end

  always@(posedge i_ptp_rx_clk or negedge rx_rst_rxx_n) 
  begin	
    if(!rx_rst_rxx_n)
      o_rx_dt_reg <= {32{1'b0}};
    else
      o_rx_dt_reg <= rx_tam_adjust_avg;
  end

  always@(posedge i_ptp_tx_clk or negedge tx_rst_txx_n) 
  begin	
    if(!tx_rst_txx_n)
      o_tx_dt_reg <= {32{1'b0}};
    else
      o_tx_dt_reg <= tx_tam_adjust_avg;
  end


     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) tx_error_2ds (
        .clk                (i_ptp_clk),
        .reset              (1'b0),
        .d                  (o_tx_error),
        .q                  (tx_error_p)
    );

     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) rx_error_2ds (
        .clk                (i_ptp_clk),
        .reset              (1'b0),
        .d                  (o_rx_error),
        .q                  (rx_error_p)
    );

     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) tx_tam_adjust_avg_2ds (
        .clk                (i_ptp_clk),
        .reset              (1'b0),
        .d                  (tx_tam_adjust_avg),
        .q                  (tx_tam_adjust_avg_p)
    );


     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) rx_tam_adjust_avg_2ds (
        .clk                (i_ptp_clk),
        .reset              (1'b0),
        .d                  (rx_tam_adjust_avg),
        .q                  (rx_tam_adjust_avg_p)
    );


     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) tx_measure_done_2ds (
        .clk                (i_ptp_clk),
        .reset              (1'b0),
        .d                  (tx_measure_done),
        .q                  (tx_measure_done_p)
    );


     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) rx_measure_done_2ds (
        .clk                (i_ptp_clk),
        .reset              (1'b0),
        .d                  (rx_measure_done),
        .q                  (rx_measure_done_p)
    );


    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) tx_ptp_ready_2ds (
        .clk                (i_ptp_tx_clk),
        .reset              (1'b0),
        .d                  (tx_ptp_ready),
        .q                  (tx_ptp_ready_x)
    );

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) rx_ptp_ready_2ds (
        .clk                (i_ptp_rx_clk),
        .reset              (1'b0),
        .d                  (rx_ptp_ready),
        .q                  (rx_ptp_ready_x)
    );


    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) tx_calibrate_2ds (
        .clk                (i_ptp_clk),
        .reset              (1'b0),
        .d                  (tx_calibrate),
        .q                  (tx_calibrate_p)
    );

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) rx_calibrate_2ds (
        .clk                (i_ptp_clk),
        .reset              (1'b0),
        .d                  (rx_calibrate),
        .q                  (rx_calibrate_p)
    );  

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) tx_sn_ready_2ds (
        .clk                (i_ptp_clk),
        .reset              (1'b0),
        .d                  (tx_sn_ready),
        .q                  (tx_sn_ready_p)
    );


    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) rx_sn_ready_2ds (
        .clk                (i_ptp_clk),
        .reset              (1'b0),
        .d                  (rx_sn_ready),
        .q                  (rx_sn_ready_p)
    );


  assign o_tx_ptp_ready = tx_ptp_ready && tx_sn_ready_p;
  assign o_rx_ptp_ready = rx_ptp_ready && rx_sn_ready_p;


    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) tx_rst_txx_n_2ds (
        .clk                (i_ptp_tx_clk),
        .reset              (1'b0),
        .d                  (i_tx_rst_n1),
        .q                  (tx_rst_txx_n)
    );


    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) rx_rst_rxx_n_2ds (
        .clk                (i_ptp_rx_clk),
        .reset              (1'b0),
        .d                  (i_rx_rst_n1),
        .q                  (rx_rst_rxx_n)
    );


endmodule : alt_ehipc3_fm_sl_ptp_sn

//============================================================================//
//                           E N D   O F   F I L E                            //
//============================================================================//
