// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1 ps / 1 ps

module alt_ehipc3_fm_sl_avmm_if (

    input   wire             i_reconfig_clk,
    input   wire             i_avmm_resetn,

`ifdef AVMM_MASTER_BFM
    input   wire   [31:0]    avmm_mst_writedata,
    input   wire             avmm_mst_read,
    input   wire   [31:0]    avmm_mst_address,
    input   wire             avmm_mst_write,
    output  wire   [31:0]    avmm_mst_readdata,
    output  wire             avmm_mst_waitrequest,
    output  wire             avmm_mst_readdata_valid,
`endif

    output  wire [18:0]      o_xcvr_reconfig_addr,           
    output  wire             o_xcvr_reconfig_read,           
    output  wire             o_xcvr_reconfig_write,          
    output  wire [31:0]      o_xcvr_reconfig_writedata,      
    input   wire [31:0]      i_xcvr_reconfig_readdata,       
    input   wire             i_xcvr_reconfig_readdata_valid, 
    input   wire             i_xcvr_reconfig_waitrequest,    
       
    output  wire [18:0]      o_eth_reconfig_addr,
    output  wire             o_eth_reconfig_read,
    output  wire             o_eth_reconfig_write,
    output  wire [31:0]      o_eth_reconfig_writedata,
    input   wire [31:0]      i_eth_reconfig_readdata,
    input   wire             i_eth_reconfig_readdata_valid,
    input   wire             i_eth_reconfig_waitrequest,
    
    output  wire             pkt_avl_mm_read,
    output  wire             pkt_avl_mm_write,
    output  wire [31:0]      pkt_avl_mm_writedata,
    output  wire [11:0]      pkt_avl_mm_baddress,
    input   wire [31:0]      pkt_avl_mm_readdata,
    input   wire             pkt_avl_mm_waitrequest,
 
    output  wire [10:0]      o_rsfec_reconfig_addr,
    output  wire             o_rsfec_reconfig_read,
    output  wire             o_rsfec_reconfig_write,
    input   wire [31:0]      i_rsfec_reconfig_readdata,
    output  wire [31:0]      o_rsfec_reconfig_writedata,
    input   wire             i_rsfec_reconfig_waitrequest,
    
    input   wire [31:0]      pio_stat0,
    output  wire [31:0]      pio_ctrl0
    );

    wire   [3:0]             slave_sel;
    wire   [31:0]            avmm_slv_address;
    wire   [3:0]             avmm_slv_read;
    wire   [3:0]             avmm_slv_write;
    wire   [31:0]            avmm_slv_writedata;
    wire   [127:0]           avmm_slv_readdata;
    wire   [3:0]             avmm_slv_waitrequest;
    wire   [3:0]             avmm_slv_readdata_valid;

`ifndef AVMM_MASTER_BFM

    wire   [31:0]            avmm_mst_writedata;
    wire                     avmm_mst_read;
    wire   [31:0]            avmm_mst_address;
    wire                     avmm_mst_write;
    wire   [31:0]            avmm_mst_readdata;
    wire                     avmm_mst_waitrequest;
    wire                     avmm_mst_readdata_valid;

    alt_ehipc3_fm_jtag_avalon jtag_master
    (/*AUTOINST*/
       // Outputs
       .master_address            (avmm_mst_address[31:0]),
       .master_read               (avmm_mst_read),
       .master_write              (avmm_mst_write),
       .master_writedata          (avmm_mst_writedata[31:0]),
       .master_byteenable         (),
       .master_reset_reset        (),
       // Inputs
       .clk_clk                   (i_reconfig_clk),
       .clk_reset_reset           (~i_avmm_resetn),
       .master_readdata           (avmm_mst_readdata[31:0]),
       .master_waitrequest        (avmm_mst_waitrequest),
       .master_readdatavalid      (avmm_mst_readdata_valid)
       ); 
       
`endif


//////////////////////////////////////   DECODING   ////////////////////////////////////////////
    assign o_eth_reconfig_addr        = {2'b00,avmm_slv_address[18:2]};
    assign o_eth_reconfig_write       = avmm_slv_write[0]; 
    assign o_eth_reconfig_read        = avmm_slv_read[0]; 
    assign o_eth_reconfig_writedata   = avmm_slv_writedata; 
    assign avmm_slv_readdata[31:0]    = i_eth_reconfig_readdata; 
    assign avmm_slv_waitrequest[0]    = i_eth_reconfig_waitrequest; 
    assign avmm_slv_readdata_valid[0] = i_eth_reconfig_readdata_valid;
    
    assign o_xcvr_reconfig_addr       = {2'b00,avmm_slv_address[18:2]}; 
    assign o_xcvr_reconfig_write      = avmm_slv_write[1]; 
    assign o_xcvr_reconfig_read       = avmm_slv_read[1]; 
    assign o_xcvr_reconfig_writedata  = avmm_slv_writedata;
    assign avmm_slv_readdata[63:32]   = i_xcvr_reconfig_readdata;
    assign avmm_slv_waitrequest[1]    = i_xcvr_reconfig_waitrequest;
    assign avmm_slv_readdata_valid[1] = i_xcvr_reconfig_readdata_valid;
       
    wire pkt_avl_mm_valid;  
    assign pkt_avl_mm_valid           = pkt_avl_mm_read & ~pkt_avl_mm_waitrequest;
    assign pkt_avl_mm_baddress        = avmm_slv_address[11:0]; 
    assign pkt_avl_mm_writedata       = avmm_slv_writedata;
    assign pkt_avl_mm_write           = avmm_slv_write[2];
    assign pkt_avl_mm_read            = avmm_slv_read[2]; 
    assign avmm_slv_waitrequest[2]    = pkt_avl_mm_waitrequest;    
    assign avmm_slv_readdata[95:64]   = pkt_avl_mm_readdata; 
    assign avmm_slv_readdata_valid[2] = pkt_avl_mm_valid;


    assign o_rsfec_reconfig_addr       = avmm_slv_address[12:2];
    assign o_rsfec_reconfig_read       = avmm_slv_read[3];
    assign o_rsfec_reconfig_write      = avmm_slv_write[3];
    assign o_rsfec_reconfig_writedata  = avmm_slv_writedata;
    assign avmm_slv_readdata[127:96]   = i_rsfec_reconfig_readdata;
    assign avmm_slv_waitrequest[3]     = i_rsfec_reconfig_waitrequest;
    assign avmm_slv_readdata_valid[3]  = o_rsfec_reconfig_read & ~i_rsfec_reconfig_waitrequest;
       
    alt_ehipc3_fm_sl_avmm_interconnect #(
      .NUM_SLAVE(4)
    ) alt_ehipc3_fm_sl_avmm_interconnect
    (/*AUTOINST*/
       // Outputs
       .avmm_mst_readdata         (avmm_mst_readdata[31:0]),
       .avmm_mst_readdata_valid   (avmm_mst_readdata_valid),
       .avmm_mst_waitrequest      (avmm_mst_waitrequest),
       .avmm_slv_address          (avmm_slv_address[31:0]),
       .avmm_slv_read             (avmm_slv_read[3:0]),
       .avmm_slv_write            (avmm_slv_write[3:0]),
       .avmm_slv_writedata        (avmm_slv_writedata[31:0]),
       .slave_sel                 (slave_sel[3:0]),
       // Inputs
       .clk                       (i_reconfig_clk),
       .rst_n                     (i_avmm_resetn),
       .avmm_mst_address          (avmm_mst_address[31:0]),
       .avmm_mst_read             (avmm_mst_read),
       .avmm_mst_write            (avmm_mst_write),
       .avmm_mst_writedata        (avmm_mst_writedata[31:0]),
       .avmm_slv_readdata         (avmm_slv_readdata[127:0]),
       .avmm_slv_readdata_valid   (avmm_slv_readdata_valid[3:0]),
       .avmm_slv_waitrequest      (avmm_slv_waitrequest[3:0])
       );

     
endmodule
