// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//==============================================================================
//------------------------------------------------------------------------------
//
// Filename         : ptp_avmm_csr_bank.sv
// Author           : Tholu Kiran Kumar <kktholu@intel.com>
// Created On       : Wed Jun 14/06/2017 03:25:24 PM MYT
//
//------------------------------------------------------------------------------
//
// $File: $
// $Revision: $
// $Date: $
// $Author: $
//
//==============================================================================

//------------------------------------------------------------------------------
//
// Description :-
//   
//
//------------------------------------------------------------------------------

`timescale 1ns / 1ns
module alt_ehipc3_fm_sl_ptp_avmm_csr_bank_sn(
  input  logic        i_avmm_clk,
  input  logic        i_avmm_rst_n,
  output logic [31:0] o_avmm_readdata,               //                  csr.readdata
  input  logic        i_avmm_write,                  //                     .write
  input  logic        i_avmm_read,                   //                     .read
  input  logic [31:0] i_avmm_writedata,              //                     .writedata
  output logic        o_avmm_waitrequest,            //                     .waitrequest
  input  logic [5:0]  i_avmm_address,

  input  logic [47:0] i_tx_tam_capture,
  input  logic [15:0] i_tx_tam_count,
  input  logic        i_tx_tam_cap_valid,
  input  logic [47:0] i_rx_tam_capture,
  input  logic [15:0] i_rx_tam_count,
  input  logic        i_rx_tam_cap_valid,
  
  // 1ns
  input logic [31:0]  i_txrx_dt_status,
  input logic [31:0]  i_rx_dt_reg,
  input logic [31:0]  i_tx_dt_reg, 
 
  input logic [31:0]  i_tx_apulse_rt_period,  
  input logic [31:0]  i_tx_apulse_sn_period,  
  input logic [31:0]  i_tx_apulse_err_period,  
  input logic [31:0]  i_tx_apulse_err2_latency,  
  input logic [31:0]  i_rx_apulse_rt_period,  
  input logic [31:0]  i_rx_apulse_sn_period, 
  input logic [31:0]  i_rx_apulse_err_period, 
  input logic [31:0]  i_rx_apulse_err2_latency, 
  
  output logic [31:0] o_tx_apulse_period,
  output logic [31:0] o_tx_apulse_skip,
  output logic [31:0] o_tx_apulse_roll_dly,
  output logic [31:0] o_tx_apulse_cnt_pow2,
  output logic [31:0] o_tx_apulse_cnt,
  output logic [31:0] o_tx_sn_config,

  output logic [31:0] o_rx_apulse_period,
  output logic [31:0] o_rx_apulse_skip,
  output logic [31:0] o_rx_apulse_roll_dly,
  output logic [31:0] o_rx_apulse_cnt_pow2,
  output logic [31:0] o_rx_apulse_cnt,
  output logic [31:0] o_rx_sn_config,

  output logic [31:0] o_sn_pll_sclk_period,
  
  output logic        o_calibrate,
  input  logic        i_tx_calibrate_done,
  input  logic        i_rx_calibrate_done,
  input  logic        i_tx_error_valid,
  input  logic [31:0] i_tx_error,
  input  logic        i_rx_error_valid,
  input  logic [31:0] i_rx_error,
  output logic        o_ts_format,
  output logic        o_req_tx_ui_load,
  output logic        o_req_rx_ui_load,
  output logic [31:0] o_tx_ui,
  output logic [31:0] o_rx_ui,
  output logic [31:0] o_tpiso,
  output logic [31:0] o_rsipo,
  output logic [31:0] o_rxalign_offset
  
);

parameter SIM_MODE          = 0;
parameter ehip_rate_sl      = "rate_25gx1";
parameter TX_UI_ADDR        = 6'h10;
parameter RX_UI_ADDR        = 6'h11;
parameter CAL_ADDR          = 6'h12;
parameter TX_ERR_ADDR       = 6'h13;
parameter RX_ERR_ADDR       = 6'h14;
parameter TS_FORMAT_ADDR    = 6'h15;
parameter TPISO_ADDR        = 6'h16;
parameter RSIPO_ADDR        = 6'h17;
parameter RXALIGH_OFST_ADDR = 6'h18;
parameter TAM_SNAPSHOT_ADDR = 6'h19;
parameter TX_TAM_L_ADDR     = 6'h1A;
parameter TX_TAM_H_ADDR     = 6'h1B;
parameter TX_TAM_COUNT_ADDR = 6'h1C;
parameter RX_TAM_L_ADDR     = 6'h1D;
parameter RX_TAM_H_ADDR     = 6'h1E;
parameter RX_TAM_COUNT_ADDR = 6'h1F;

parameter TXRX_DT_STATUS    = 6'h20; 
parameter RX_DT_REG         = 6'h21; 
parameter TX_DT_REG         = 6'h22; 

parameter TX_APULSE_PERIOD_ADDR       = 6'h23; 
parameter TX_APULSE_SKIP_ADDR         = 6'h24; 
parameter TX_APULSE_ROLL_DLY_ADDR     = 6'h25; 
parameter TX_APULSE_CNT_POW2_ADDR     = 6'h26; 
parameter TX_APULSE_CNT_ADDR          = 6'h27; 
parameter TX_APULSE_PERIOD_RT_ADDR    = 6'h28; 
parameter TX_APULSE_PERIOD_SN_ADDR    = 6'h29; 
parameter TX_APULSE_PERIOD_ERR_ADDR   = 6'h2A; 
parameter TX_APULSE_LATENCY_ERR2_ADDR = 6'h2B; 
parameter TX_SN_CONFIG_ADDR           = 6'h2C; 

parameter RX_APULSE_PERIOD_ADDR       = 6'h2D; 
parameter RX_APULSE_SKIP_ADDR         = 6'h2E; 
parameter RX_APULSE_ROLL_DLY_ADDR     = 6'h2F; 
parameter RX_APULSE_CNT_POW2_ADDR     = 6'h30; 
parameter RX_APULSE_CNT_ADDR          = 6'h31; 
parameter RX_APULSE_PERIOD_RT_ADDR    = 6'h32; 
parameter RX_APULSE_PERIOD_SN_ADDR    = 6'h33;
parameter RX_APULSE_PERIOD_ERR_ADDR   = 6'h34;
parameter RX_APULSE_LATENCY_ERR2_ADDR = 6'h35;
parameter RX_SN_CONFIG_ADDR           = 6'h36; 


parameter SN_PLL_SCLK_PERIOD_ADDR     = 6'h37; 


localparam UI_RESET_VALUE = (ehip_rate_sl == "rate_25gx1_fec") ? 32'h00_09_EE_01 :
                            (ehip_rate_sl == "rate_25gx1")     ? 32'h00_09_EE_01 :
                            (ehip_rate_sl == "rate_10gx1_25lr")? 32'h00_18_D3_02 :
                            (ehip_rate_sl == "rate_10gx1")     ? 32'h00_18_D3_02 :
                                                                 32'h00_18_D3_02 ;

localparam TX_ASYNC_PULSE_PERIOD =  (SIM_MODE == 1) ? (ehip_rate_sl == "rate_25gx1_fec") ? 32'd80    :
                                                      (ehip_rate_sl == "rate_25gx1")     ? 32'd5120  :
                                                      (ehip_rate_sl == "rate_10gx1_25lr")? 32'd5120  : 
                                                      (ehip_rate_sl == "rate_10gx1")     ? 32'd5120  :  32'd5280
                                                    : (ehip_rate_sl == "rate_25gx1_fec") ? 32'd80    :
                                                      (ehip_rate_sl == "rate_25gx1")     ? 32'd81920 :
                                                      (ehip_rate_sl == "rate_10gx1_25lr")? 32'd81920 : 
                                                      (ehip_rate_sl == "rate_10gx1")     ? 32'd81920 :  32'd84480;


localparam TX_APULSE_SKIP        =  (SIM_MODE == 1) ? (ehip_rate_sl == "rate_25gx1_fec") ? {16'd1, 16'd3}      :
                                                      (ehip_rate_sl == "rate_25gx1")     ? {16'd1, 16'd3}      : {16'd1, 16'd3}
                                                    : (ehip_rate_sl == "rate_25gx1_fec") ? {16'd1, 16'd7}      :
                                                      (ehip_rate_sl == "rate_25gx1")     ? {16'd1, 16'd7}      : {16'd1, 16'd7};

localparam TX_APULSE_ROLL_DLY    =  (SIM_MODE == 1) ? (ehip_rate_sl == "rate_25gx1_fec") ? {8'd15, 8'd4, 16'd16} :
                                                      (ehip_rate_sl == "rate_25gx1")     ? {8'd15, 8'd4, 16'd8}  :
                                                      (ehip_rate_sl == "rate_10gx1_25lr")? {8'd15, 8'd4, 16'd16} : 
                                                      (ehip_rate_sl == "rate_10gx1")     ? {8'd15, 8'd4, 16'd16} : {8'd15, 8'd4, 16'd8}
                                                    : (ehip_rate_sl == "rate_25gx1_fec") ? {8'd15, 8'd4, 16'd16} :
                                                      (ehip_rate_sl == "rate_25gx1")     ? {8'd15, 8'd4, 16'd8}  :
                                                      (ehip_rate_sl == "rate_10gx1_25lr")? {8'd15, 8'd4, 16'd16} : 
                                                      (ehip_rate_sl == "rate_10gx1")     ? {8'd15, 8'd4, 16'd16} : {8'd15, 8'd4, 16'd8};

localparam TX_APULSE_CNT         =  (SIM_MODE == 1) ? (ehip_rate_sl == "rate_25gx1_fec") ? 32'd32     :
                                                      (ehip_rate_sl == "rate_25gx1")     ? 32'd32     : 32'd32
                                                    : (ehip_rate_sl == "rate_25gx1_fec") ? 32'd1024   :
                                                      (ehip_rate_sl == "rate_25gx1")     ? 32'd1024   : 32'd1024;

localparam TX_APULSE_CNT_POW2    =  (SIM_MODE == 1) ? (ehip_rate_sl == "rate_25gx1_fec") ? 32'd5      :
                                                      (ehip_rate_sl == "rate_25gx1")     ? 32'd5      : 32'd5
                                                    : (ehip_rate_sl == "rate_25gx1_fec") ? 32'd10     :
                                                      (ehip_rate_sl == "rate_25gx1")     ? 32'd10     : 32'd10;

localparam TX_SN_CONFIG          =  (SIM_MODE == 1) ? (ehip_rate_sl == "rate_25gx1_fec") ? {16'd1, 16'd1} :
                                                      (ehip_rate_sl == "rate_25gx1")     ? {16'd0, 16'd1} : 
                                                      (ehip_rate_sl == "rate_10gx1_25lr")? {16'd2, 16'd1} : 
                                                      (ehip_rate_sl == "rate_10gx1")     ? {16'd2, 16'd1} : 32'd0
                                                    : (ehip_rate_sl == "rate_25gx1_fec") ? {16'd1, 16'd1} :
                                                      (ehip_rate_sl == "rate_25gx1")     ? {16'd0, 16'd1} : 
                                                      (ehip_rate_sl == "rate_10gx1_25lr")? {16'd2, 16'd1} : 
                                                      (ehip_rate_sl == "rate_10gx1")     ? {16'd2, 16'd1} : 32'd0;


localparam RX_ASYNC_PULSE_PERIOD =  (SIM_MODE == 1) ? (ehip_rate_sl == "rate_25gx1_fec") ? 32'd80    :
                                                      (ehip_rate_sl == "rate_25gx1")     ? 32'd96    :
                                                      (ehip_rate_sl == "rate_10gx1_25lr")? 32'd96    : 
                                                      (ehip_rate_sl == "rate_10gx1")     ? 32'd96    : 32'd99
                                                    : (ehip_rate_sl == "rate_25gx1_fec") ? 32'd80    :
                                                      (ehip_rate_sl == "rate_25gx1")     ? 32'd96    :
                                                      (ehip_rate_sl == "rate_10gx1_25lr")? 32'd96    : 
                                                      (ehip_rate_sl == "rate_10gx1")     ? 32'd96    : 32'd99;
                                                      
localparam RX_APULSE_SKIP        =  (SIM_MODE == 1) ? (ehip_rate_sl == "rate_25gx1_fec") ? {16'd20, 16'd10}      :
                                                      (ehip_rate_sl == "rate_25gx1")     ? {16'd20, 16'd10}      : {16'd20, 16'd10}
                                                    : (ehip_rate_sl == "rate_25gx1_fec") ? {16'd20, 16'd5000}  :
                                                      (ehip_rate_sl == "rate_25gx1")     ? {16'd20, 16'd5000}  : {16'd20, 16'd5000};

localparam RX_APULSE_ROLL_DLY    =  (SIM_MODE == 1) ? (ehip_rate_sl == "rate_25gx1_fec") ? {8'd15, 8'd4, 16'd16} :
                                                      (ehip_rate_sl == "rate_25gx1")     ? {8'd15, 8'd4, 16'd8}  :
                                                      (ehip_rate_sl == "rate_10gx1_25lr")? {8'd15, 8'd4, 16'd8}  : 
                                                      (ehip_rate_sl == "rate_10gx1")     ? {8'd15, 8'd4, 16'd8}  : {8'd15, 8'd4, 16'd8}
                                                    : (ehip_rate_sl == "rate_25gx1_fec") ? {8'd15, 8'd4, 16'd16} :
                                                      (ehip_rate_sl == "rate_25gx1")     ? {8'd15, 8'd4, 16'd8}  :
                                                      (ehip_rate_sl == "rate_10gx1_25lr")? {8'd15, 8'd4, 16'd8}  : 
                                                      (ehip_rate_sl == "rate_10gx1")     ? {8'd15, 8'd4, 16'd8}  : {8'd15, 8'd4, 16'd8};
                                                      
localparam RX_APULSE_CNT         =  (SIM_MODE == 1) ? (ehip_rate_sl == "rate_25gx1_fec") ? 32'd32     :
                                                      (ehip_rate_sl == "rate_25gx1")     ? 32'd32     : 32'd32
                                                    : (ehip_rate_sl == "rate_25gx1_fec") ? 32'd1024   :
                                                      (ehip_rate_sl == "rate_25gx1")     ? 32'd1024   : 32'd1024;

localparam RX_APULSE_CNT_POW2    =  (SIM_MODE == 1) ? (ehip_rate_sl == "rate_25gx1_fec") ? 32'd5      :
                                                      (ehip_rate_sl == "rate_25gx1")     ? 32'd5      : 32'd5
                                                    : (ehip_rate_sl == "rate_25gx1_fec") ? 32'd10     :
                                                      (ehip_rate_sl == "rate_25gx1")     ? 32'd10     : 32'd10;

localparam RX_SN_CONFIG          =  (SIM_MODE == 1) ? (ehip_rate_sl == "rate_25gx1_fec") ? {16'd1, 16'd1} :
                                                      (ehip_rate_sl == "rate_25gx1")     ? {16'd0, 16'd1} : 
                                                      (ehip_rate_sl == "rate_10gx1_25lr")? {16'd2, 16'd1} : 
                                                      (ehip_rate_sl == "rate_10gx1")     ? {16'd2, 16'd1} : 32'd0
                                                    : (ehip_rate_sl == "rate_25gx1_fec") ? {16'd1, 16'd1} :
                                                      (ehip_rate_sl == "rate_25gx1")     ? {16'd0, 16'd1} : 
                                                      (ehip_rate_sl == "rate_10gx1_25lr")? {16'd2, 16'd1} : 
                                                      (ehip_rate_sl == "rate_10gx1")     ? {16'd2, 16'd1} : 32'd0;
                                                      
localparam SN_PLL_SCLK_PERIOD    =  20'h8C000;  //8.75ns
//localparam SN_PLL_SCLK_PERIOD    =  20'h40000;  //4ns

localparam SYNC_FLOPS            = 4'd3;

localparam SN_DBG                =  0;  



logic [31:0] r_tx_apulse_period;
logic [31:0] r_tx_apulse_skip;
logic [31:0] r_tx_apulse_roll_dly;
logic [31:0] r_tx_apulse_cnt_pow2;
logic [31:0] r_tx_apulse_cnt;
logic [31:0] r_tx_sn_config;

logic [31:0] r_rx_apulse_period;
logic [31:0] r_rx_apulse_skip;
logic [31:0] r_rx_apulse_roll_dly;
logic [31:0] r_rx_apulse_cnt_pow2;
logic [31:0] r_rx_apulse_cnt;
logic [31:0] r_rx_sn_config;

logic [31:0] r_sn_pll_sclk_period;

logic        tx_calibrate_done;
logic        rx_calibrate_done;
logic        tx_calibrate_done_r;
logic        rx_calibrate_done_r;
logic [31:0] r_calibrate;
logic [31:0] r_tx_error;
logic [31:0] r_rx_error;
logic [31:0] r_ts_format;
logic [31:0] r_tx_ui;
logic [31:0] r_rx_ui;
logic [31:0] r_tpiso;
logic [31:0] r_rsipo;
logic [31:0] r_rxalign_offset;
logic [5:0]  r_avmm_address;
logic        r_avmm_read;
logic        r_avmm_waitrequest;
logic        i_avmm_write_r;
logic        i_avmm_read_r;
    
logic        r_tam_snapshot;
logic [31:0] r_tx_tam_l;
logic [15:0] r_tx_tam_h;
logic [15:0] r_tx_tam_count;
logic [31:0] r_rx_tam_l;
logic [15:0] r_rx_tam_h;
logic [15:0] r_rx_tam_count; 


logic [31:0]  txrx_dt_status_2d;
logic [31:0]  rx_dt_reg_2d;
logic [31:0]  tx_dt_reg_2d; 
 
logic [31:0]  tx_apulse_rt_period_2d;  
logic [31:0]  tx_apulse_sn_period_2d;  
logic [31:0]  tx_apulse_err_period_2d;  
logic [31:0]  tx_apulse_err2_latency_2d;  
logic [31:0]  rx_apulse_rt_period_2d;  
logic [31:0]  rx_apulse_sn_period_2d; 
logic [31:0]  rx_apulse_err_period_2d; 
logic [31:0]  rx_apulse_err2_latency_2d; 
 
    
alt_xcvr_resync_std #(
    .SYNC_CHAIN_LENGTH  (3),
    .WIDTH              (1),
    .INIT_VALUE         (0)
) tx_calibrate_done_sync (
    .clk                (i_avmm_clk),
    .reset              (!i_avmm_rst_n),
    .d                  (i_tx_calibrate_done),
    .q                  (tx_calibrate_done)
);

alt_xcvr_resync_std #(
    .SYNC_CHAIN_LENGTH  (3),
    .WIDTH              (1),
    .INIT_VALUE         (0)
) rx_calibrate_done_sync (
    .clk                (i_avmm_clk),
    .reset              (!i_avmm_rst_n),
    .d                  (i_rx_calibrate_done),
    .q                  (rx_calibrate_done)
);

always @ (posedge i_avmm_clk or negedge i_avmm_rst_n)
begin
  if(!i_avmm_rst_n)
  begin
    i_avmm_write_r <= 0;
    i_avmm_read_r  <= 0;
  end
  else
  begin
    i_avmm_write_r <= i_avmm_write;
    i_avmm_read_r  <= i_avmm_read;
  end
end

assign o_avmm_waitrequest = (i_avmm_write_r || (i_avmm_read_r && r_avmm_read && r_avmm_waitrequest && (i_avmm_address == r_avmm_address))) ? 1'b0 : 1'b1;
assign o_calibrate = r_calibrate[0];
assign o_ts_format = r_ts_format[0];
assign o_tx_ui = r_tx_ui;
assign o_rx_ui = r_rx_ui;
assign o_tpiso = r_tpiso;
assign o_rsipo = r_rsipo;
assign o_rxalign_offset = r_rxalign_offset;


assign o_tx_apulse_period   = (SN_DBG == 1) ? r_tx_apulse_period : TX_ASYNC_PULSE_PERIOD;
assign o_tx_apulse_skip     = (SN_DBG == 1) ? r_tx_apulse_skip : TX_APULSE_SKIP;
assign o_tx_apulse_roll_dly = (SN_DBG == 1) ? r_tx_apulse_roll_dly : TX_APULSE_ROLL_DLY;
assign o_tx_apulse_cnt_pow2 = (SN_DBG == 1) ? r_tx_apulse_cnt_pow2 : TX_APULSE_CNT_POW2;
assign o_tx_apulse_cnt      = (SN_DBG == 1) ? r_tx_apulse_cnt : TX_APULSE_CNT;
assign o_tx_sn_config       = (SN_DBG == 1) ? r_tx_sn_config : TX_SN_CONFIG;

assign o_rx_apulse_period   = (SN_DBG == 1) ? r_rx_apulse_period : RX_ASYNC_PULSE_PERIOD;
assign o_rx_apulse_skip     = (SN_DBG == 1) ? r_rx_apulse_skip : RX_APULSE_SKIP;
assign o_rx_apulse_roll_dly = (SN_DBG == 1) ? r_rx_apulse_roll_dly : RX_APULSE_ROLL_DLY;
assign o_rx_apulse_cnt_pow2 = (SN_DBG == 1) ? r_rx_apulse_cnt_pow2 : RX_APULSE_CNT_POW2;
assign o_rx_apulse_cnt      = (SN_DBG == 1) ? r_rx_apulse_cnt : RX_APULSE_CNT;
assign o_rx_sn_config       = (SN_DBG == 1) ? r_rx_sn_config : RX_SN_CONFIG;

assign o_sn_pll_sclk_period = (SN_DBG == 1) ? r_sn_pll_sclk_period : SN_PLL_SCLK_PERIOD;

always @ (posedge i_avmm_clk or negedge i_avmm_rst_n)
begin
  if(!i_avmm_rst_n)
  begin
    r_calibrate <= 0;
    //r_tx_error <= 0;
    //r_rx_error <= 0;
    r_ts_format <= 0;
    r_tx_ui <= UI_RESET_VALUE;
    r_rx_ui <= UI_RESET_VALUE;
    r_tpiso <= 0;
    r_rsipo <= 0;
    r_rxalign_offset <= 0;
    r_tam_snapshot <= 1'b0;
    
    r_tx_apulse_period   <= TX_ASYNC_PULSE_PERIOD;
    r_tx_apulse_skip     <= TX_APULSE_SKIP;
    r_tx_apulse_roll_dly <= TX_APULSE_ROLL_DLY;
    r_tx_apulse_cnt_pow2 <= TX_APULSE_CNT_POW2;
    r_tx_apulse_cnt      <= TX_APULSE_CNT;
    r_tx_sn_config       <= TX_SN_CONFIG;

    r_rx_apulse_period   <= RX_ASYNC_PULSE_PERIOD;
    r_rx_apulse_skip     <= RX_APULSE_SKIP;
    r_rx_apulse_roll_dly <= RX_APULSE_ROLL_DLY;
    r_rx_apulse_cnt_pow2 <= RX_APULSE_CNT_POW2;
    r_rx_apulse_cnt      <= RX_APULSE_CNT;
    r_rx_sn_config       <= RX_SN_CONFIG;

    r_sn_pll_sclk_period <= SN_PLL_SCLK_PERIOD;

  end
  else
  begin
    if (i_avmm_write)
    begin
      case (i_avmm_address) 
        TX_UI_ADDR          : r_tx_ui          <= i_avmm_writedata;
        RX_UI_ADDR          : r_rx_ui          <= i_avmm_writedata;
        CAL_ADDR            : r_calibrate      <= i_avmm_writedata;
        //TX_ERR_ADDR         : r_tx_error       <= i_avmm_writedata;
        //RX_ERR_ADDR         : r_rx_error       <= i_avmm_writedata;
        TPISO_ADDR          : r_tpiso          <= i_avmm_writedata;
        TS_FORMAT_ADDR      : r_ts_format      <= i_avmm_writedata;
        RSIPO_ADDR          : r_rsipo          <= i_avmm_writedata;
        RXALIGH_OFST_ADDR   : r_rxalign_offset <= i_avmm_writedata;
        TAM_SNAPSHOT_ADDR   : r_tam_snapshot   <= i_avmm_writedata[0];

        TX_APULSE_PERIOD_ADDR    : r_tx_apulse_period     <= i_avmm_writedata;
        TX_APULSE_SKIP_ADDR      : r_tx_apulse_skip       <= i_avmm_writedata;
        TX_APULSE_ROLL_DLY_ADDR  : r_tx_apulse_roll_dly   <= i_avmm_writedata;
        TX_APULSE_CNT_POW2_ADDR  : begin
                                     r_tx_apulse_cnt_pow2   <= i_avmm_writedata;
                                     r_tx_apulse_cnt        <= 1'b1 << i_avmm_writedata;
                                   end
 //       TX_APULSE_CNT_ADDR       : r_tx_apulse_cnt        <= i_avmm_writedata;
        TX_SN_CONFIG_ADDR        : r_tx_sn_config   <= {r_tx_sn_config[31:16], i_avmm_writedata[15:0]};

        RX_APULSE_PERIOD_ADDR    : r_rx_apulse_period     <= i_avmm_writedata;
        RX_APULSE_SKIP_ADDR      : r_rx_apulse_skip       <= i_avmm_writedata;
        RX_APULSE_ROLL_DLY_ADDR  : r_rx_apulse_roll_dly   <= i_avmm_writedata;
        RX_APULSE_CNT_POW2_ADDR  : begin
                                     r_rx_apulse_cnt_pow2   <= i_avmm_writedata;
                                     r_rx_apulse_cnt        <= 1'b1 << i_avmm_writedata;
                                   end
//        RX_APULSE_CNT_ADDR       : r_rx_apulse_cnt        <= i_avmm_writedata;
        RX_SN_CONFIG_ADDR        : r_rx_sn_config   <= {r_rx_sn_config[31:16], i_avmm_writedata[15:0]};

        SN_PLL_SCLK_PERIOD_ADDR  : r_sn_pll_sclk_period   <= i_avmm_writedata;

      endcase
    end
  end
end

always @ (posedge i_avmm_clk or negedge i_avmm_rst_n)
begin
  if(!i_avmm_rst_n)
  begin
    r_rx_error <= 0;
  end
  else
  begin
    if (i_rx_error_valid)
    begin
      r_rx_error <= i_rx_error;
    end
  end
end

always @ (posedge i_avmm_clk or negedge i_avmm_rst_n)
begin
  if(!i_avmm_rst_n)
  begin
    r_tx_error <= 0;
  end
  else
  begin
    if (i_tx_error_valid)
    begin
      r_tx_error <= i_tx_error;
    end
  end
end

always @ (posedge i_avmm_clk or negedge i_avmm_rst_n)
begin
  if(!i_avmm_rst_n)
  begin
    tx_calibrate_done_r <= 0;
    rx_calibrate_done_r <= 0;
    o_req_tx_ui_load <= 0;
    o_req_rx_ui_load <= 0;
  end
  else
  begin
    tx_calibrate_done_r <= tx_calibrate_done;
    rx_calibrate_done_r <= rx_calibrate_done;
    o_req_tx_ui_load <= (i_avmm_write && (i_avmm_address == TX_UI_ADDR)) || (!tx_calibrate_done_r && tx_calibrate_done);
    o_req_rx_ui_load <= (i_avmm_write && (i_avmm_address == RX_UI_ADDR)) || (!rx_calibrate_done_r && rx_calibrate_done);
  end
end

always @ (posedge i_avmm_clk or negedge i_avmm_rst_n)
begin
  if (!i_avmm_rst_n)
  begin
    r_tx_tam_l     <= 32'h0;
    r_tx_tam_h     <= 16'h0;
    r_tx_tam_count <= 16'h0;
    r_rx_tam_l     <= 32'h0;
    r_rx_tam_h     <= 16'h0;
    r_rx_tam_count <= 16'h0;     
  end
  else
  begin
    r_tx_tam_l     <= (~r_tam_snapshot & i_tx_tam_cap_valid) ? i_tx_tam_capture[31:0]  : r_tx_tam_l;
    r_tx_tam_h     <= (~r_tam_snapshot & i_tx_tam_cap_valid) ? i_tx_tam_capture[47:32] : r_tx_tam_h;
    r_tx_tam_count <= (~r_tam_snapshot & i_tx_tam_cap_valid) ? i_tx_tam_count          : r_tx_tam_count;
    r_rx_tam_l     <= (~r_tam_snapshot & i_rx_tam_cap_valid) ? i_rx_tam_capture[31:0]  : r_rx_tam_l;
    r_rx_tam_h     <= (~r_tam_snapshot & i_rx_tam_cap_valid) ? i_rx_tam_capture[47:32] : r_rx_tam_h;
    r_rx_tam_count <= (~r_tam_snapshot & i_rx_tam_cap_valid) ? i_rx_tam_count          : r_rx_tam_count;   
  end
end

always @ (posedge i_avmm_clk or negedge i_avmm_rst_n)
begin
  if (!i_avmm_rst_n)
  begin
    r_avmm_address <= 0;
    r_avmm_read <= 0;
    r_avmm_waitrequest <= 0;
    o_avmm_readdata <= 0;
  end
  else
  begin
    r_avmm_address <= i_avmm_address;
    r_avmm_read <= i_avmm_read_r;
    r_avmm_waitrequest <= o_avmm_waitrequest;
    if (i_avmm_read_r)
    begin
      case (i_avmm_address)
        TX_UI_ADDR          : o_avmm_readdata <= r_tx_ui;          
        RX_UI_ADDR          : o_avmm_readdata <= r_rx_ui;          
        CAL_ADDR            : o_avmm_readdata <= r_calibrate;
        TX_ERR_ADDR         : o_avmm_readdata <= r_tx_error;       
        RX_ERR_ADDR         : o_avmm_readdata <= r_rx_error;       
        TS_FORMAT_ADDR      : o_avmm_readdata <= r_ts_format;      
        TPISO_ADDR          : o_avmm_readdata <= r_tpiso;          
        RSIPO_ADDR          : o_avmm_readdata <= r_rsipo;          
        RXALIGH_OFST_ADDR   : o_avmm_readdata <= r_rxalign_offset; 
        TAM_SNAPSHOT_ADDR   : o_avmm_readdata <= {31'h0,r_tam_snapshot};
        TX_TAM_L_ADDR       : o_avmm_readdata <= r_tx_tam_l;     
        TX_TAM_H_ADDR       : o_avmm_readdata <= {16'h0,r_tx_tam_h};     
        TX_TAM_COUNT_ADDR   : o_avmm_readdata <= {16'h0,r_tx_tam_count}; 
        RX_TAM_L_ADDR       : o_avmm_readdata <= r_rx_tam_l;     
        RX_TAM_H_ADDR       : o_avmm_readdata <= {16'h0,r_rx_tam_h};     
        RX_TAM_COUNT_ADDR   : o_avmm_readdata <= {16'h0,r_rx_tam_count}; 

        TXRX_DT_STATUS      : o_avmm_readdata <= txrx_dt_status_2d;  
        RX_DT_REG           : o_avmm_readdata <= rx_dt_reg_2d;  
        TX_DT_REG           : o_avmm_readdata <= tx_dt_reg_2d;  

        TX_APULSE_PERIOD_ADDR       : o_avmm_readdata <= (SN_DBG == 1) ? r_tx_apulse_period : TX_ASYNC_PULSE_PERIOD;
        TX_APULSE_SKIP_ADDR         : o_avmm_readdata <= (SN_DBG == 1) ? r_tx_apulse_skip : TX_APULSE_SKIP;
        TX_APULSE_ROLL_DLY_ADDR     : o_avmm_readdata <= (SN_DBG == 1) ? r_tx_apulse_roll_dly : TX_APULSE_ROLL_DLY;
        TX_APULSE_CNT_POW2_ADDR     : o_avmm_readdata <= (SN_DBG == 1) ? r_tx_apulse_cnt_pow2 : TX_APULSE_CNT_POW2;
        TX_APULSE_CNT_ADDR          : o_avmm_readdata <= (SN_DBG == 1) ? r_tx_apulse_cnt : TX_APULSE_CNT;
        TX_APULSE_PERIOD_RT_ADDR    : o_avmm_readdata <= tx_apulse_rt_period_2d;
        TX_APULSE_PERIOD_SN_ADDR    : o_avmm_readdata <= tx_apulse_sn_period_2d;
        TX_APULSE_PERIOD_ERR_ADDR   : o_avmm_readdata <= tx_apulse_err_period_2d;
        TX_APULSE_LATENCY_ERR2_ADDR : o_avmm_readdata <= tx_apulse_err2_latency_2d;
        TX_SN_CONFIG_ADDR           : o_avmm_readdata <= (SN_DBG == 1) ? r_tx_sn_config : TX_SN_CONFIG;

        RX_APULSE_PERIOD_ADDR       : o_avmm_readdata <= (SN_DBG == 1) ? r_rx_apulse_period : RX_ASYNC_PULSE_PERIOD;
        RX_APULSE_SKIP_ADDR         : o_avmm_readdata <= (SN_DBG == 1) ? r_rx_apulse_skip : RX_APULSE_SKIP;
        RX_APULSE_ROLL_DLY_ADDR     : o_avmm_readdata <= (SN_DBG == 1) ? r_rx_apulse_roll_dly : RX_APULSE_ROLL_DLY;
        RX_APULSE_CNT_POW2_ADDR     : o_avmm_readdata <= (SN_DBG == 1) ? r_rx_apulse_cnt_pow2 : RX_APULSE_CNT_POW2;
        RX_APULSE_CNT_ADDR          : o_avmm_readdata <= (SN_DBG == 1) ? r_rx_apulse_cnt : RX_APULSE_CNT;
        RX_APULSE_PERIOD_RT_ADDR    : o_avmm_readdata <= rx_apulse_rt_period_2d;
        RX_APULSE_PERIOD_SN_ADDR    : o_avmm_readdata <= rx_apulse_sn_period_2d;
        RX_APULSE_PERIOD_ERR_ADDR   : o_avmm_readdata <= rx_apulse_err_period_2d;
        RX_APULSE_LATENCY_ERR2_ADDR : o_avmm_readdata <= rx_apulse_err2_latency_2d;
        RX_SN_CONFIG_ADDR           : o_avmm_readdata <= (SN_DBG == 1) ? r_rx_sn_config : RX_SN_CONFIG;

        SN_PLL_SCLK_PERIOD_ADDR     : o_avmm_readdata <= (SN_DBG == 1) ? r_sn_pll_sclk_period : SN_PLL_SCLK_PERIOD;

      endcase
    end
  end
end



     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) txrx_dt_status_2ds (
        .clk                (i_avmm_clk),
        .reset              (1'b0),
        .d                  (i_txrx_dt_status),
        .q                  (txrx_dt_status_2d)
    );

     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) rx_dt_reg_2ds (
        .clk                (i_avmm_clk),
        .reset              (1'b0),
        .d                  (i_rx_dt_reg),
        .q                  (rx_dt_reg_2d)
    );


     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) tx_dt_reg_2ds (
        .clk                (i_avmm_clk),
        .reset              (1'b0),
        .d                  (i_tx_dt_reg),
        .q                  (tx_dt_reg_2d)
    );



     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) tx_apulse_rt_period_2ds (
        .clk                (i_avmm_clk),
        .reset              (1'b0),
        .d                  (i_tx_apulse_rt_period),
        .q                  (tx_apulse_rt_period_2d)
    );

     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) tx_apulse_sn_period_2ds (
        .clk                (i_avmm_clk),
        .reset              (1'b0),
        .d                  (i_tx_apulse_sn_period),
        .q                  (tx_apulse_sn_period_2d)
    );

     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) tx_apulse_err_period_2ds (
        .clk                (i_avmm_clk),
        .reset              (1'b0),
        .d                  (i_tx_apulse_err_period),
        .q                  (tx_apulse_err_period_2d)
    );

     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) tx_apulse_err2_latency_2ds (
        .clk                (i_avmm_clk),
        .reset              (1'b0),
        .d                  (i_tx_apulse_err2_latency),
        .q                  (tx_apulse_err2_latency_2d)
    );

     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) rx_apulse_rt_period_2ds (
        .clk                (i_avmm_clk),
        .reset              (1'b0),
        .d                  (i_rx_apulse_rt_period),
        .q                  (rx_apulse_rt_period_2d)
    );

     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) rx_apulse_sn_period_2ds (
        .clk                (i_avmm_clk),
        .reset              (1'b0),
        .d                  (i_rx_apulse_sn_period),
        .q                  (rx_apulse_sn_period_2d)
    );

     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) rx_apulse_err_period_2ds (
        .clk                (i_avmm_clk),
        .reset              (1'b0),
        .d                  (i_rx_apulse_err_period),
        .q                  (rx_apulse_err_period_2d)
    );

     alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (SYNC_FLOPS),
        .WIDTH              (32),
        .INIT_VALUE         (0)
    ) rx_apulse_err2_latency_2ds (
        .clk                (i_avmm_clk),
        .reset              (1'b0),
        .d                  (i_rx_apulse_err2_latency),
        .q                  (rx_apulse_err2_latency_2d)
    );




  endmodule : alt_ehipc3_fm_sl_ptp_avmm_csr_bank_sn

//============================================================================//
//                           E N D   O F   F I L E                            //
//============================================================================//
