// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2018 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ns/1ns

// Ack state enumeration
`define  RESP_WAIT_FOR_REQ_ASSERT   2'b00
`define  RESP_WAIT_FOR_ACK_ASSERT   2'b01
`define  RESP_WAIT_FOR_REQ_DEASSERT 2'b10


module dp_hs_resp
(
   input                async_reset_n,
   input                sync_reset_n,

   input       [1:0]    sync_ctrl,
   
   input                si_clk,
   output reg           si_req_n /* synthesis keep */,
   input                si_ack_n /* synthesis keep */,

   input                ai_req_n /* synthesis keep */,
   output reg           ai_ack_n /* synthesis keep */
);

   reg                  next_si_req_n;
   reg                  next_ai_ack_n;
   reg         [1:0]    ack_state, next_ack_state;
   wire                 sync_ai_req_n;

   
   // Instantiate the request synchronizer	
   altera_std_synchronizer_nocut #
   (
     .depth(2),
     .rst_value(1)
   ) 
   req_sync 
     (
     .clk(si_clk), 
     .reset_n(sync_reset_n), 
     .din(ai_req_n), 
     .dout(sync_ai_req_n)
   );


   //
   // Req state machine - This logic converts an 
   // incoming request from the other clock domain
   // into a single cycle event.
   //
   
   // Ack state machine storage
   always @(posedge si_clk or negedge async_reset_n)
   begin

      if (async_reset_n == 1'b0) begin

         ack_state   <= `RESP_WAIT_FOR_REQ_ASSERT;
         si_req_n    <= 1'b1;
         ai_ack_n    <= 1'b1;

      end else begin

         if (sync_reset_n == 1'b0) begin

            ack_state   <= `RESP_WAIT_FOR_REQ_ASSERT;
            si_req_n    <= 1'b1;
            ai_ack_n    <= 1'b1;

         end else begin

            ack_state   <= next_ack_state;
            si_req_n    <= next_si_req_n;
            ai_ack_n    <= next_ai_ack_n;

         end

      end

   end


   // Ack state machine decoder
   always @(ack_state or sync_ai_req_n or si_ack_n)
   begin

      case (ack_state)

         `RESP_WAIT_FOR_REQ_ASSERT: begin

            if (sync_ai_req_n == 1'b0) begin
                    
               next_ack_state    = `RESP_WAIT_FOR_ACK_ASSERT;
               next_si_req_n     = 1'b0;
               next_ai_ack_n     = 1'b1;

            end else begin
                    
               next_ack_state    = `RESP_WAIT_FOR_REQ_ASSERT;
               next_si_req_n     = 1'b1;
               next_ai_ack_n     = 1'b1;

            end

         end
         
            
         `RESP_WAIT_FOR_ACK_ASSERT: begin
                 
            if (si_ack_n == 1'b0) begin
                    
               next_ack_state    = `RESP_WAIT_FOR_REQ_DEASSERT;
               next_si_req_n     = 1'b1;
               next_ai_ack_n     = 1'b0;

            end else begin
                    
               next_ack_state    = `RESP_WAIT_FOR_ACK_ASSERT;
               next_si_req_n     = 1'b0;
               next_ai_ack_n     = 1'b1;

            end
            
         end

         
         `RESP_WAIT_FOR_REQ_DEASSERT: begin
                 
            if (sync_ai_req_n == 1'b1) begin
                    
               next_ack_state    = `RESP_WAIT_FOR_REQ_ASSERT;
               next_si_req_n     = 1'b1;
               next_ai_ack_n     = 1'b0;

            end else begin
                    
               next_ack_state    = `RESP_WAIT_FOR_REQ_DEASSERT;
               next_si_req_n     = 1'b1;
               next_ai_ack_n     = 1'b0;

            end
            
         end

         
         default: begin

            next_ack_state    = `RESP_WAIT_FOR_REQ_ASSERT;
            next_si_req_n     = 1'b1;
            next_ai_ack_n     = 1'b1;
               
         end
         
      endcase

   end

   
endmodule
