// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ns/1ns


module seriallite4_dp_sync #
(
   parameter DP_WIDTH = 1,
   parameter DP_RESET = {(DP_WIDTH){1'b0}}
)
(
   input                         async_reset_n,
   input                         sync_reset_n,
   input                         clk,
   input       [1:0]             sync_ctrl,
   input       [DP_WIDTH-1:0]    d,
   
   output reg  [DP_WIDTH-1:0]    o

)/* synthesis syn_noprune=1 */;

   wire        [DP_WIDTH-1:0]    dp_sync_stage_1_dout;
   wire        [DP_WIDTH-1:0]    dp_sync_stage_2_dout;

   
   // Synchronizer registers - long, unique names are used here used to
   // simplify the creation of constraints to ignore timing checks up to
   // the stage 1 register, and find these flops when special metastability
   // hardened flops are to be used.

   // The first stage synchronization register
   seriallite4_dp_sync_regstage #
   (
      .DP_WIDTH(DP_WIDTH),
      .DP_INIT(DP_RESET)
   )
   dp_sync_stage_1
   (
      .clk(clk),
      .async_reset_n(async_reset_n),
      .sync_reset_n(sync_reset_n),
      .clke_n(1'b0),
      .d(d),
      .o(dp_sync_stage_1_dout)
   );


   // The second stage synchronization register
   seriallite4_dp_sync_regstage #
   (
      .DP_WIDTH(DP_WIDTH),
      .DP_INIT(DP_RESET)
   )
   dp_sync_stage_2
   (
      .clk(clk),
      .async_reset_n(async_reset_n),
      .sync_reset_n(sync_reset_n),
      .clke_n(1'b0),
      .d(dp_sync_stage_1_dout),
      .o(dp_sync_stage_2_dout)
   );


   // The bypass mux
   always @* begin
           
      case (sync_ctrl)
              
         2'b00:   o = d;                              // No synchronization 
         2'b01:   o = dp_sync_stage_1_dout;   // One stage of synchronization
         2'b10:   o = dp_sync_stage_2_dout;   // Two stages of synchronization
         default: o = d;                              // No synchronization

      endcase

   end
   
endmodule


module seriallite4_dp_sync_regstage #
(
   parameter                     DP_WIDTH = 32,
   parameter                     DP_INIT  = 32'd0
)
(
   input                         async_reset_n,
   input                         sync_reset_n,
   input                         clk,
   input                         clke_n,
   input       [DP_WIDTH-1:0]    d,
   
   output reg  [DP_WIDTH-1:0]    o /* synthesis syn_keep=1 */
)/* synthesis syn_noprune=1 */;


   always @(posedge clk or negedge async_reset_n) begin

      if (async_reset_n == 1'b0)

         o <= DP_INIT;
         
      else if (sync_reset_n == 0)

         o <= DP_INIT;

      else if (clke_n == 0)

         o <= d;

   end
   
endmodule
