// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.

//`define ENABLE_QSFPDDA
`define ENABLE_QSFPDDB
`define USE_CH_4567 // ch 0/1/2/3 is used if not defied

`timescale 1 ps / 1 ps

module seriallite_iv_streaming_demo #
(
parameter                  lanes                      = 4,
parameter   [63:0]         total_samples_to_transfer  = 64'hFFFFFFFFFFFFFFFF,
parameter		          STREAM_MODE		 = 1,
parameter		        	MIN_INTER_BURST_GAP	 = 2,
parameter                  DEVICE_FAMILY         = "Agilex"
)
(

// Tx/Rx activity indicators
output                     tx_activity_n, //VIRTUAL_PIN
output                     rx_activity_n, //VIRTUAL_PIN

// reserved
output		 [31:0] 			reserved_bits,//VIRTUAL_PIN


      ///////// CLOCK /////////
      input              CLK_100_B2A,
      input              CLK_50_B2C,
      input              CLK_50_B3A,
      input              CLK_50_B3C,
      input              UFL_CLKIN_p,

      ///////// Buttons /////////
      input              CPU_RESET_n,
      input    [ 1: 0]   BUTTON, //BUTTON is Low-Active

      ///////// Swtiches /////////
      input    [ 1: 0]   SW,

      ///////// LED /////////
      output   [ 3: 0]   LED_BRACKET, //LED is Low-Active
      output   [ 3: 0]   LED, //LED is Low-Active

`ifdef ENABLE_QSFPRSV_REFCLK
      ///////// QSFPRSV_REFCLK /////////
      input              QSFPDDRSV_REFCLK_p,
`endif /*ENABLE_QSFPRSV_REFCLK*/

`ifdef ENABLE_DDR4A
      ///////// DDR4A /////////
      input              DDR4A_REFCLK_p,
      output   [16: 0]   DDR4A_A,
      output   [ 1: 0]   DDR4A_BA,
      output   [ 1: 0]   DDR4A_BG,
      output   [RANK: 0] DDR4A_CK,
      output   [RANK: 0] DDR4A_CK_n,
      output   [RANK: 0] DDR4A_CKE,
      inout    [ 8: 0]   DDR4A_DQS,
      inout    [ 8: 0]   DDR4A_DQS_n,
      inout    [71: 0]   DDR4A_DQ,
      inout    [ 8: 0]   DDR4A_DBI_n,
      output   [RANK: 0] DDR4A_CS_n,
      output             DDR4A_RESET_n,
      output   [RANK: 0] DDR4A_ODT,
      output             DDR4A_PAR,
      input              DDR4A_ALERT_n,
      output             DDR4A_ACT_n,
      input              DDR4A_EVENT_n,
      inout              DDR4A_SCL,
      inout              DDR4A_SDA,
      input              DDR4A_RZQ,
`endif /*ENABLE_DDR4A*/

`ifdef ENABLE_DDR4B
      ///////// DDR4B /////////
      input              DDR4B_REFCLK_p,
      output   [16: 0]   DDR4B_A,
      output   [ 1: 0]   DDR4B_BA,
      output   [ 1: 0]   DDR4B_BG,
      output   [RANK: 0] DDR4B_CK,
      output   [RANK: 0] DDR4B_CK_n,
      output   [RANK: 0] DDR4B_CKE,
      inout    [ 8: 0]   DDR4B_DQS,
      inout    [ 8: 0]   DDR4B_DQS_n,
      inout    [71: 0]   DDR4B_DQ,
      inout    [ 8: 0]   DDR4B_DBI_n,
      output   [RANK: 0] DDR4B_CS_n,
      output             DDR4B_RESET_n,
      output   [RANK: 0] DDR4B_ODT,
      output             DDR4B_PAR,
      input              DDR4B_ALERT_n,
      output             DDR4B_ACT_n,
      input              DDR4B_EVENT_n,
      inout              DDR4B_SCL,
      inout              DDR4B_SDA,
      input              DDR4B_RZQ,
`endif /*ENABLE_DDR4B*/

`ifdef ENABLE_DDR4C
      ///////// DDR4C /////////
      input              DDR4C_REFCLK_p,
      output   [16: 0]   DDR4C_A,
      output   [ 1: 0]   DDR4C_BA,
      output   [ 1: 0]   DDR4C_BG,
      output   [RANK: 0] DDR4C_CK,
      output   [RANK: 0] DDR4C_CK_n,
      output   [RANK: 0] DDR4C_CKE,
      inout    [ 8: 0]   DDR4C_DQS,
      inout    [ 8: 0]   DDR4C_DQS_n,
      inout    [71: 0]   DDR4C_DQ,
      inout    [ 8: 0]   DDR4C_DBI_n,
      output   [RANK: 0] DDR4C_CS_n,
      output             DDR4C_RESET_n,
      output   [RANK: 0] DDR4C_ODT,
      output             DDR4C_PAR,
      input              DDR4C_ALERT_n,
      output             DDR4C_ACT_n,
      input              DDR4C_EVENT_n,
      inout              DDR4C_SCL,
      inout              DDR4C_SDA,
      input              DDR4C_RZQ,
`endif /*ENABLE_DDR4C*/

`ifdef ENABLE_DDR4D
      ///////// DDR4D /////////
      input              DDR4D_REFCLK_p,
      output   [16: 0]   DDR4D_A,
      output   [ 1: 0]   DDR4D_BA,
      output   [ 1: 0]   DDR4D_BG,
      output   [RANK: 0] DDR4D_CK,
      output   [RANK: 0] DDR4D_CK_n,
      output   [RANK: 0] DDR4D_CKE,
      inout    [ 8: 0]   DDR4D_DQS,
      inout    [ 8: 0]   DDR4D_DQS_n,
      inout    [71: 0]   DDR4D_DQ,
      inout    [ 8: 0]   DDR4D_DBI_n,
      output   [RANK: 0] DDR4D_CS_n,
      output             DDR4D_RESET_n,
      output   [RANK: 0] DDR4D_ODT,
      output             DDR4D_PAR,
      input              DDR4D_ALERT_n,
      output             DDR4D_ACT_n,
      input              DDR4D_EVENT_n,
      inout              DDR4D_SCL,
      inout              DDR4D_SDA,
      input              DDR4D_RZQ,
`endif /*ENABLE_DDR4D*/

      ///////// SI5340A /////////
      inout              SI5340A_I2C_SCL,
      inout              SI5340A_I2C_SDA,
      input              SI5340A_LOL,
      input              SI5340A_LOS_XAXB,
      inout              SI5340A_OE_n,
      inout              SI5340A_RST_n,

      ///////// GPIO /////////
      input    [ 1: 0]   GPIO_CLK,
      inout    [ 3: 0]   GPIO_p,

`ifdef ENABLE_PCIE
      ///////// PCIE /////////
      inout              PCIE_SMBCLK,
      inout              PCIE_SMBDAT,
      output   [15: 0]   PCIE_TX_p,
      output   [15: 0]   PCIE_TX_n,
      input    [15: 0]   PCIE_RX_p,
      input    [15: 0]   PCIE_RX_n,
      output             PCIE_CLKREQ_n,
      input    [ 1: 0]   PCIE_REFCLK_p,
      input              PCIE_PERST_n,
      output             PCIE_WAKE_n,
`endif /*ENABLE_PCIE*/

`ifdef ENABLE_QSFPDDA
      ///////// QSFPDDA /////////
	`ifdef USE_CH_4567
      output   [ 7: 4]   QSFPDDA_TX_p,
      //output   [ 7: 4]   QSFPDDA_TX_n,
      input    [ 7: 4]   QSFPDDA_RX_p,
      //input    [ 7: 4]   QSFPDDA_RX_n,
	`else
      output   [ 3: 0]   QSFPDDA_TX_p,
      //output   [ 3: 0]   QSFPDDA_TX_n,
      input    [ 3: 0]   QSFPDDA_RX_p,
      //input    [ 3: 0]   QSFPDDA_RX_n,
	`endif
      input              QSFPDDA_REFCLK_p,
      output             QSFPDDA_INITMODE,
      input              QSFPDDA_INTERRUPT_n,
      input              QSFPDDA_MOD_PRS_n,
      output             QSFPDDA_MOD_SEL_n,
      output             QSFPDDA_RST_n,
      inout              QSFPDDA_SCL,
      inout              QSFPDDA_SDA,
`endif /*ENABLE_QSFPDDA*/

`ifdef ENABLE_QSFPDDB
      ///////// QSFPDDB /////////
	`ifdef USE_CH_4567
      output   [ 7: 4]   QSFPDDB_TX_p,
      //output   [ 7: 4]   QSFPDDB_TX_n,
      input    [ 7: 4]   QSFPDDB_RX_p,
      //input    [ 7: 4]   QSFPDDB_RX_n,
	`else
      output   [ 3: 0]   QSFPDDB_TX_p,
      //output   [ 3: 0]   QSFPDDB_TX_n,
      input    [ 3: 0]   QSFPDDB_RX_p,
      //input    [ 3: 0]   QSFPDDB_RX_n,
	`endif
      input              QSFPDDB_REFCLK_p,
      output             QSFPDDB_INITMODE,
      input              QSFPDDB_INTERRUPT_n,
      input              QSFPDDB_MOD_PRS_n,
      output             QSFPDDB_MOD_SEL_n,
      output             QSFPDDB_RST_n,
      inout              QSFPDDB_SCL,
      inout              QSFPDDB_SDA,
`endif /*ENABLE_QSFPDDB*/

      ///////// AG /////////
      output             AG_UART_TX,
      input              AG_UART_RX,
      input              AG_UART_CTS,
      output             AG_UART_RTS,

      ///////// EXP /////////
      input              EXP_EN,

      ///////// INFO /////////
      output             INFO_SPI_SCLK,
      input              INFO_SPI_MISO,
      output             INFO_SPI_MOSI,
      output             INFO_SPI_CS_n,

      ///////// QSPI /////////
      output             QSPI_CLK,
      inout    [ 3: 0]   QSPI_DATA,
      output             QSPI_NCSO 



);


wire 		[3:0]       tx;
wire 		[3:0]       rx;
wire 						mgmt_clk;
wire 						pll_ref_clk;
wire 						iopll_ref_clk;

assign mgmt_clk = CLK_100_B2A;
assign mgmt_reset_n =  CPU_RESET_n;
assign iopll_ref_clk = CLK_50_B2C;
assign LED[2] = src_link_up_n;
assign LED[3] = snk_link_up_n;
assign LED[0] = src_core_reset_n;
assign LED[1] = snk_core_reset_n;


`ifdef ENABLE_QSFPDDA
	assign QSFPDDA_INITMODE = SW[0]; // 1(SW UP): low power, 0(SW DOWN): not low power
	assign QSFPDDA_MOD_SEL_n = SW[1];// 0: enable i2c
	assign pll_ref_clk = QSFPDDA_REFCLK_p;
	assign QSFPDDA_RST_n = CPU_RESET_n;
	assign QSFPDDA_TX_p = {tx[3],tx[1],tx[2],tx[0]};
	assign {rx[3],rx[1],rx[2],rx[0]} = QSFPDDA_RX_p;
`endif // ENABLE_QSFP28A  


`ifdef ENABLE_QSFPDDB
	assign QSFPDDB_INITMODE = SW[0]; // 1(SW UP): low power, 0(SW DOWN): not low power
	assign QSFPDDB_MOD_SEL_n = SW[1];// 0: enable i2c
	assign pll_ref_clk = QSFPDDB_REFCLK_p;
	assign QSFPDDB_RST_n = CPU_RESET_n;
	assign QSFPDDB_TX_p = {tx[3],tx[1],tx[2],tx[0]};
	assign {rx[3],rx[1],rx[2],rx[0]} = QSFPDDB_RX_p;
`endif // ENABLE_QSFP28B  


wire                       src_core_reset; //
wire                       src_user_clock; //
wire                       src_link_up; //
wire                       src_ready; //
wire   [(lanes*64)-1:0]    src_data; //

wire   [4:0]               src_error; //

wire   [7:0]               tx_channel; //
wire                       tx_startofpacket; //
wire                       tx_endofpacket; //
wire   [3:0]               tg_tx_num_valid_bytes_eob;  //
wire                       tg_tx_is_usr_cmd; //
wire   [3:0]               tg_tx_num_valid_bytes_eob_in; //
wire                       tg_tx_is_usr_cmd_in; //
wire   [4:0]               src_empty; //
wire                       src_valid; //
wire 				  [((lanes)*64)+32-1:0] tx_avs_data;
wire 					   tx_avs_valid;

wire                       snk_core_reset; //
wire                       snk_user_clock; //
wire                       snk_link_up; //
wire   [(lanes*64)-1:0]    snk_data; //
wire   [(lanes*2)+2:0]     snk_error; //

wire   [7:0]               rx_channel; //
wire                       rx_startofpacket; //
wire                       rx_endofpacket; //
wire   [3:0]               tc_rx_num_valid_bytes_eob; //
wire                       tc_rx_is_usr_cmd; //
wire   [3:0]               tc_rx_num_valid_bytes_eob_in; //
wire                       tc_rx_is_usr_cmd_in; //
wire   [4:0]               snk_empty; //

wire                       snk_valid;

//FIFO Signals
wire                       src_valid_in;
wire                       src_startofpacket_in;
wire                       src_endofpacket_in;
wire   [7:0]               src_channel_in;
wire   [4:0]               src_empty_in;

reg                        ecc_fifo_startofpacket_d;
reg                        ecc_fifo_endofpacket_d;
reg    [7:0]               ecc_fifo_channel_d;
reg    [4:0]               ecc_fifo_empty_d;
reg   [3:0]                ecc_fifo_num_valid_bytes_eob_d;  //
reg                        ecc_fifo_is_usr_cmd_d; //
reg 							rx_adaptation_fifo_full;
wire    [5:0]                   rdusedw;
wire        		  tx_fifo_empty;
wire 				  [lanes-1:0] tx_adaptation_fifo_full;
wire 				  [lanes-1:0] adaptation_fifo_pfull;
wire 				  [lanes-1:0] tx_data_valid;
wire 				  [lanes-1:0] tx_ecc_fatal, tx_ecc_cor;
wire 				  [((lanes)*64)+32-1:0] tx_parallel_data;

reg 			    ecc_fifo_valid_input_d;
wire                        snk_valid_in;

reg                         ecc_fifo_valid_d;
reg   [(lanes*64)-1:0]      ecc_fifo_data_d;
reg   [(lanes*64)+32-1:0]   snk_data_fifo_in; //
wire  [(lanes*64)+32-1:0]  snk_data_fifo_out; //
reg   [(lanes*64)+32-1:0]   snk_data_fifo_out_d; /* synthesis preserve_syn_only */
wire  [(lanes*64)-1:0]     snk_data_in;
wire                       snk_startofpacket_in;
wire                       snk_endofpacket_in;
wire   [7:0]               snk_channel_in;
wire   [4:0]               snk_empty_in;
//EHIP related signals
wire   [(lanes)*8-1:0]    xcvr_reconfig_readdata;
wire   [(lanes)*8-1:0]    xcvr_reconfig_writedata;
wire   [(lanes)*19-1:0]    xcvr_reconfig_address;
wire   [lanes-1:0]        xcvr_reconfig_read;
wire   [lanes-1:0]        xcvr_reconfig_write;
wire   [lanes-1:0]        xcvr_reconfig_waitrequest;
wire        			  source_mgmt_readdatavalid;



wire         			  rsfec_reconfig_read;
wire        	 		  rsfec_reconfig_write;
wire [10:0]  			  rsfec_reconfig_address;
wire [7:0]   			  rsfec_reconfig_readdata;
wire [0:0]   			  rsfec_reconfig_waitrequest;
wire [7:0]   			  rsfec_reconfig_writedata;

wire   [29:0]   		  reconfig_addr;
wire         			  status_write;
wire         			  status_read;
wire   [31:0]   		  status_readdata;
wire   [31:0]  			  status_writedata;
wire         			  status_waitrequest;
wire         			  status_readdata_valid;
wire   [lanes-1:0] 		  xcvr_reconfig_readdata_valid;

wire   [lanes-1:0]         phy_ehip_ready;
wire   [lanes-1:0]         phy_tx_lanes_stable;
wire   [lanes-1:0]         phy_rx_pcs_ready;
wire   [lanes-1:0]         phy_rx_block_lock;
wire   [lanes-1:0]         phy_rx_hi_ber;

wire                       tg_enable;
wire					   tc_enable;
wire                       tg_test_mode;
wire                       tg_enable_stalls;
wire                       crc_error_inject;
wire                       enable_slave_mode, enable_slave_mode_sync;

wire   [26:0]              tg_burst_count;
wire   [63:0]              tg_words_transferred;

wire   [26:0]              tc_burst_count;
wire   [63:0]              tc_words_received;

wire   [lanes-1:0]         tc_lane_swap_error;
wire   [lanes-1:0]         tc_lane_sequence_error;
wire   [lanes-1:0]         tc_lane_alignment_error;

wire   [8:0]               demo_mgmt_address;
wire                       demo_mgmt_write;
wire                       demo_mgmt_read;
wire   [31:0]              demo_mgmt_readdata;
wire   [31:0]              demo_mgmt_writedata;
wire                       demo_mgmt_waitrequest;
wire                       mgmt_clk_reset;
wire					   demo_mgmt_readdata_valid;

wire                       rx_core_clkout; 
reg                        rx_core_clock_reset;
wire			   rx_core_rst_n;
reg			   rx_core_rst_n_d;
reg			   rx_core_rst_n_d1;
wire  [1:0]                rx_core_reset_n_d;
 
wire                       tx_core_clkout; 
reg                        tx_core_clock_reset;
wire 			   tx_core_rst_n;
reg			   tx_core_rst_n_d;
reg			   tx_core_rst_n_d1;
wire  [1:0]                tx_core_reset_n_d;

wire  [(lanes*2)+3-1:0]   sink_error_out;
reg 					  tx_adaptation_fifo_full_p;
reg 					  rx_fifo_full;
reg 					  rx_fifo_full_d;
reg 					  snk_valid_d;
reg   [(lanes*64)-1:0]    snk_data_d;
reg                       valid, valid_d; 
reg 					  rx_startofpacket_d, rx_endofpacket_d, tc_rx_is_usr_cmd_d;
reg	  [4:0]				  snk_empty_d;
reg   [7:0]               rx_channel_d;
reg   [3:0]               tc_rx_num_valid_bytes_eob_d;

//crete type
wire crete_type;
assign crete_type = 1'b1;

wire pcs_mode;
assign pcs_mode = 1'b1;

wire op_mode;
assign op_mode = 1'b1;

wire crc_mode;
assign crc_mode = 1'b0;

wire fec_mode;
assign fec_mode = 1'b1;
// Drive the, active-low, user leds
//
assign src_core_reset_n = ~src_core_reset;
assign snk_core_reset_n = ~snk_core_reset;

// Activity indicators
assign tx_activity_n    = ~src_valid;
assign rx_activity_n    = ~snk_valid;

// Link Status indicators
assign src_link_up_n    = ~src_link_up;
assign snk_link_up_n    = ~snk_link_up;

wire		[8:0] issp_sources;
wire		sync_issp_sources_n4;
wire		sync_issp_sources_n5;
wire		sync_issp_sources_n6;
wire		sync_issp_sources_n7;
wire		sync_issp_sources_n8;
wire		[lanes-1:0] tx_pll_locked;
wire		tx_pll_locked_comb;
wire		pll_locked_sync;
wire		pll_locked_det;
wire		tx_pcs_ready;
wire		rx_pcs_ready;
wire		rx_hi_ber;
wire 		det_tc_burst_count;
wire 		det_tc_words_received;
wire 		[31:0] system_status_pio_in;
wire 		[31:0] tx_lanes_status_pio_in;
wire 		[31:0] txrx_lanes_status_pio_in;
wire 		[31:0] rx_lanes_status_pio_in;
wire		demo_mgmt_crc_inject;
wire 		prbs_en_data;

assign 	prbs_en_data = prbs_enable;  // Data type : 0 = incremental, 1 = prbs 
assign  crc_error_inject = sync_issp_sources_n6 | demo_mgmt_crc_inject;
assign 	det_tc_burst_count = (tc_burst_count != 27'h0) ? 1'b1 : 1'b0;
assign 	det_tc_words_received = (tc_words_received != 64'h0) ? 1'b1 : 1'b0;
assign  system_status_pio_in = {28'b0, tc_enable, tg_enable, snk_link_up, src_link_up}; 
assign  tx_lanes_status_pio_in    = {phy_tx_lanes_stable, tx_pll_locked};
assign  txrx_lanes_status_pio_in    = {phy_rx_block_lock, phy_ehip_ready};
assign  rx_lanes_status_pio_in    = {phy_rx_hi_ber, phy_rx_pcs_ready};
assign	tx_pll_locked_comb = &tx_pll_locked;
assign 	pll_locked_det = pll_locked_sync;
assign 	tx_lanes_stable = &phy_tx_lanes_stable;
assign 	tx_ehip_ready = &phy_ehip_ready;
assign 	rx_block_lock = &phy_rx_block_lock;
assign 	rx_pcs_ready = &phy_rx_pcs_ready;
assign	rx_hi_ber = &phy_rx_hi_ber;
assign 	snk_user_clock = src_user_clock;

wire ninit_done;

reset_release_ip_reset_release_ip reset_release_ip_inst (
.ninit_done (ninit_done)
);

wire 		iopll_locked;

altera_std_synchronizer_nocut #
(
.depth(2),
.rst_value(1)
)
iopll_locked_user_clk_sync
(
.clk(src_user_clock),
 .reset_n(~mgmt_clk_reset),
.din(iopll_locked),
.dout(pll_locked_sync)
);

wire [1:0] src_user_clock_reset;
wire 		tx_user_clock_reset;
wire [1:0] snk_user_clock_reset;
wire 		rx_user_clock_reset;

genvar k;

generate
for (k = 0; k < 2 ; k = k + 1) begin: reset_sync_loop

altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
user_clk_reset_sync
(
.clk(src_user_clock),
.reset_n(pll_locked_det),
.din(1'b0),
.dout(src_user_clock_reset[k])
);

altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
user_clk_snk_reset_sync
(
.clk(snk_user_clock),
.reset_n(pll_locked_det),
.din(1'b0),
.dout(snk_user_clock_reset[k])
);

end
endgenerate

assign tx_user_clock_reset = src_user_clock_reset[1] | src_core_reset;
assign rx_user_clock_reset = snk_user_clock_reset[1] | snk_core_reset;



altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
slave_mode_sync
(
.clk(src_user_clock),
.reset_n(~src_user_clock_reset[1] | ~snk_user_clock_reset[1]),
.din(enable_slave_mode),
.dout(enable_slave_mode_sync)
);

wire  [lanes-1:0] 			  xcvr_cs;
wire  [lanes-1:0] 			  xcvr_read;
wire  [lanes-1:0] 			  xcvr_write;

//Decoding for avmm address map registers to EHIP
// NOTES! Ensure the base address for source_mgmt mm bridge slave in Qsys is set to 0x0
wire  [27:0] 			  status_addr;
wire  					  select_waitrequest;
assign status_addr = reconfig_addr[29:2];
assign select_waitrequest = 
			((status_addr >= 28'h100000) && (status_addr <= 28'h1fffff)) ? xcvr_reconfig_waitrequest [0] :
			((status_addr >= 28'h200000) && (status_addr <= 28'h2fffff)) ? xcvr_reconfig_waitrequest [1] :
			((status_addr >= 28'h300000) && (status_addr <= 28'h3fffff)) ? xcvr_reconfig_waitrequest [2] :
			((status_addr >= 28'h400000) && (status_addr <= 28'h4fffff)) ? xcvr_reconfig_waitrequest [3] :
			((status_addr >= 28'h10000) && (status_addr <= 28'h13fff)) ? rsfec_reconfig_waitrequest :
			1'b0;

assign status_waitrequest = select_waitrequest;

assign status_readdata_valid = (o_rsfec_reconfig_readdata_valid | (|xcvr_reconfig_readdata_valid[lanes-1:0]));

//Reconfig Decoding RSFEC
wire    rsfec_cs    = ((status_addr >= 28'h10000) && (status_addr <= 28'h13fff));
wire    rsfec_read = status_read && rsfec_cs;
wire    rsfec_write = status_write && rsfec_cs;
assign  rsfec_reconfig_address = (rsfec_read|| rsfec_write) ? status_addr[10:0] : 11'b0;
assign  o_rsfec_reconfig_readdata_valid = rsfec_read && !rsfec_reconfig_waitrequest;

genvar i;
generate
for (i = 0; i < lanes ; i = i + 1) begin: reconfig_loop
assign  xcvr_read[i] = status_read && xcvr_cs[i];
assign  xcvr_write[i] = status_write && xcvr_cs[i];
assign  xcvr_reconfig_address[(19*(i+1))-1:(19*i)]   = (xcvr_read[i] || xcvr_write[i]) ? status_addr[18:0] : 19'b0;
assign  xcvr_reconfig_readdata_valid[i] = xcvr_read[i] && !xcvr_reconfig_waitrequest[i];
end
endgenerate

assign  xcvr_cs[0]    = ((status_addr >= 28'h100000) && (status_addr <= 28'h1fffff));
assign  xcvr_cs[1]    = ((status_addr >= 28'h200000) && (status_addr <= 28'h2fffff));
assign  xcvr_cs[2]    = ((status_addr >= 28'h300000) && (status_addr <= 28'h3fffff));
assign  xcvr_cs[3]    = ((status_addr >= 28'h400000) && (status_addr <= 28'h4fffff));
assign xcvr_reconfig_read   = {xcvr_read[3], xcvr_read[2], xcvr_read[1], xcvr_read[0]};
assign xcvr_reconfig_write  = {xcvr_write[3], xcvr_write[2], xcvr_write[1], xcvr_write[0]};

assign rsfec_reconfig_read  = rsfec_read;
assign rsfec_reconfig_write = rsfec_write;
assign rsfec_reconfig_writedata = status_writedata[7:0];
assign xcvr_reconfig_writedata  = {4{status_writedata[7:0]}};
assign status_readdata =
o_rsfec_reconfig_readdata_valid ?  {24'b0, rsfec_reconfig_readdata}  :
xcvr_reconfig_readdata_valid[0] ?  xcvr_reconfig_readdata[(8*1)-1:(8*0)]  :
xcvr_reconfig_readdata_valid[1] ?  xcvr_reconfig_readdata[(8*2)-1:(8*1)]  :
xcvr_reconfig_readdata_valid[2] ?  xcvr_reconfig_readdata[(8*3)-1:(8*2)]  :
xcvr_reconfig_readdata_valid[3] ?  xcvr_reconfig_readdata[(8*4)-1:(8*3)]  :
											 32'hdead_c0de;


assign  demo_mgmt_readdata_valid = demo_mgmt_read && !demo_mgmt_waitrequest;

demo_mgmt #
(
.lanes(lanes)
)
demo_mgmt
(
// Traffic generator control
.tg_enable(tg_enable),
.tg_test_mode(tg_test_mode),
.tg_enable_stalls(tg_enable_stalls),
.crc_err_inject(demo_mgmt_crc_inject),
.enable_slave_mode(enable_slave_mode),
.prbs_enable(prbs_enable),
// Traffic generator status
.tg_burst_count(tg_burst_count),
.tg_words_transferred(tg_words_transferred),

// Source core interface
.source_reset(src_core_reset),
.source_user_clock(src_user_clock),
.source_user_clock_reset(src_user_clock_reset[0]),
.source_error(src_error[3]|src_error[0]),
.src_fatal_errors (&tx_ecc_fatal),
.src_correctable_errors (&tx_ecc_cor),  
// Sink core interface
.sink_reset(snk_core_reset),
.sink_user_clock(snk_user_clock),
.sink_user_clock_reset(snk_user_clock_reset[0]),
.sink_error(sink_error_out),
// Traffic checker status
.tc_burst_count(tc_burst_count),
.tc_words_received(tc_words_received),
.tc_lane_swap_error(tc_lane_swap_error),
.tc_lane_sequence_error(tc_lane_sequence_error),
.tc_lane_alignment_error(tc_lane_alignment_error),
// FIFO Status
.snk_overflow_errors (rx_fifo_full_d),
.src_overflow_errors (tx_adaptation_fifo_full_p),
// Demo management interface
.demo_mgmt_clk(mgmt_clk),
.demo_mgmt_clk_reset(mgmt_clk_reset),
.demo_mgmt_address(demo_mgmt_address),
.demo_mgmt_write(demo_mgmt_write),
.demo_mgmt_read(demo_mgmt_read),
.demo_mgmt_readdata(demo_mgmt_readdata),
.demo_mgmt_writedata(demo_mgmt_writedata),
.demo_mgmt_waitrequest(demo_mgmt_waitrequest),

//Operation mode
.op_mode	(op_mode),

//PCS mode
.pcs_mode	(pcs_mode),

//CRC mode
.crc_mode	(crc_mode),

//FEC mode
.fec_mode	(fec_mode),

//crete type
.crete_type (crete_type)
);

traffic_gen #
(
.LANES(lanes),
.TOTAL_SAMPLES_TO_TRANSFER(total_samples_to_transfer),
.STREAM                    (STREAM_MODE),
.MIN_INTER_BURST_GAP	 (MIN_INTER_BURST_GAP)
)
traffic_gen
(
// Clocks and reset
.user_clock(src_user_clock),    //input
 .user_clock_reset(tx_user_clock_reset),
// Streaming data interface
.data(src_data),

.sync(tx_channel),
.start_of_burst(tx_startofpacket),
.tx_num_valid_bytes_eob    (tg_tx_num_valid_bytes_eob),      
.tx_is_usr_cmd             (tg_tx_is_usr_cmd), 
.end_of_burst(tx_endofpacket),
.empty                     (src_empty),
.valid(src_valid),

.link_up(src_link_up),

// Test control ports
.tg_enable(tg_enable & snk_link_up),
.tg_test_mode(tg_test_mode),
.tg_enable_stalls(tg_enable_stalls),
.fifo_full_stalls   (&adaptation_fifo_pfull),
// Status interface
.tg_burst_count       (tg_burst_count),
.tg_words_transferred (tg_words_transferred),

//prbs data enable
.prbs_en_data(prbs_en_data)
);

always @(posedge src_user_clock) begin
ecc_fifo_valid_input_d         <= (enable_slave_mode_sync) ?  valid_d : src_valid;
ecc_fifo_valid_d               <= (enable_slave_mode_sync) ?  snk_valid_in : src_valid;
ecc_fifo_data_d                <= (enable_slave_mode_sync) ?  snk_data_in  : src_data;
ecc_fifo_startofpacket_d       <= (enable_slave_mode_sync) ?  snk_startofpacket_in : tx_startofpacket;
ecc_fifo_endofpacket_d         <= (enable_slave_mode_sync) ?  snk_endofpacket_in : tx_endofpacket;
ecc_fifo_channel_d             <= (enable_slave_mode_sync) ?  snk_channel_in : tx_channel;
ecc_fifo_empty_d               <= (enable_slave_mode_sync) ?  snk_empty_in : src_empty;
ecc_fifo_num_valid_bytes_eob_d <= (enable_slave_mode_sync) ?  tc_rx_num_valid_bytes_eob_in : tg_tx_num_valid_bytes_eob;
ecc_fifo_is_usr_cmd_d          <= (enable_slave_mode_sync) ?  tc_rx_is_usr_cmd_in : tg_tx_is_usr_cmd;
end
// TX Data and Control Signals FIFO
seriallite4_source_adaptation_ecc #
(
.LANES(lanes),
.DEVICE_FAMILY (DEVICE_FAMILY)
) tx_dcfifo_ecc (
.tx_user_clock (src_user_clock),
 .tx_user_clock_reset (tx_user_clock_reset),
.txcore_clock (tx_core_clkout),
.txcore_clock_reset (tx_core_clock_reset),
.fi_data ({11'b0, ecc_fifo_valid_d, ecc_fifo_startofpacket_d, ecc_fifo_endofpacket_d, ecc_fifo_channel_d, ecc_fifo_empty_d, ecc_fifo_num_valid_bytes_eob_d, ecc_fifo_is_usr_cmd_d, ecc_fifo_data_d}),
.fi_valid (ecc_fifo_valid_input_d),
.tx_fifo_pfull (src_ready),
.tx_parallel_data (tx_parallel_data),
.tx_data_valid(tx_data_valid),

// Error indication
.tx_ecc_cor (tx_ecc_cor),
.tx_ecc_fatal (tx_ecc_fatal),
.adaptation_fifo_pfull (adaptation_fifo_pfull),
.adaptation_fifo_full (tx_adaptation_fifo_full)
);

assign src_valid_in                    = tx_parallel_data[20+(lanes*64)];
assign tx_avs_data[(lanes*64)-1:0]     = tx_parallel_data[(lanes*64)-1:0];

assign tx_avs_data[(lanes*64)+32-1:(lanes*64)]     = 32'b0;
assign src_startofpacket_in            = tx_parallel_data[19+(lanes*64)];
assign src_endofpacket_in              = tx_parallel_data[18+(lanes*64)];
assign src_channel_in                  = tx_parallel_data[17+(lanes*64):10+(lanes*64)];
assign src_empty_in                    = tx_parallel_data[9+(lanes*64):5+(lanes*64)];
assign tg_tx_num_valid_bytes_eob_in    = tx_parallel_data[4+(lanes*64):1+(lanes*64)];
assign tg_tx_is_usr_cmd_in             = tx_parallel_data[0+(lanes*64)];

always @(posedge src_user_clock) begin
if (tx_user_clock_reset) begin
tx_adaptation_fifo_full_p <= 1'd0;
end else begin
tx_adaptation_fifo_full_p <= |tx_adaptation_fifo_full;
end
end

wire tx_ehip_ready_coreclk_sync;
wire rx_pcs_ready_coreclk_sync;
wire src_user_clock_reset_sync;
wire snk_user_clock_reset_sync;
wire issp_rx_core_rst_sync;
wire issp_tx_core_rst_sync;
wire issp_fec_phy_rst_sync;

altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
sync_tx_ehip_ready
(
.clk(tx_core_clkout),
.reset_n(1'b1),
.din(tx_ehip_ready),
.dout(tx_ehip_ready_coreclk_sync)
);

altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
sync_rx_pcs_ready
(
.clk(rx_core_clkout),
.reset_n(1'b1),
.din(rx_pcs_ready),
.dout(rx_pcs_ready_coreclk_sync)
);

altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
sync_issp_tx_core_rst
(
.clk(tx_core_clkout),
.reset_n(1'b1),
.din(issp_sources[2]),
.dout(issp_tx_core_rst_sync)
);

altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
sync_issp_rx_core_rst
(
.clk(rx_core_clkout),
.reset_n(1'b1),
.din(issp_sources[1]),
.dout(issp_rx_core_rst_sync)
);

altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
sync_issp_fec_phy_rst
(
.clk(tx_core_clkout),
.reset_n(1'b1),
.din(issp_sources[3]),
.dout(issp_fec_phy_rst_sync)
);

altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
sync_issp_n4
(
.clk(rx_core_clkout),
.reset_n(1'b1),
.din(issp_sources[4]),
.dout(sync_issp_sources_n4)
);

altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
sync_issp_n5
(
.clk(tx_core_clkout),
.reset_n(1'b1),
.din(issp_sources[5]),
.dout(sync_issp_sources_n5)
);

altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
sync_issp_n6
(
.clk(rx_core_clkout),
.reset_n(1'b1),
.din(issp_sources[6]),
.dout(sync_issp_sources_n6)
);
altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
sync_issp_n7
(
.clk(rx_core_clkout),
.reset_n(1'b1),
.din(issp_sources[7]),
.dout(sync_issp_sources_n7)
);

altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
sync_issp_n8
(
.clk(rx_core_clkout),
.reset_n(1'b1),
.din(issp_sources[8]),
.dout(sync_issp_sources_n8)
);

altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
sync_src_user_clock_reset
(
.clk(tx_core_clkout),
.reset_n(1'b1),
.din(~src_user_clock_reset[1]),
.dout(src_user_clock_reset_sync)
);

altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
sync_snk_user_clock_reset
(
.clk(rx_core_clkout),
.reset_n(1'b1),
.din(~snk_user_clock_reset[1]),
.dout(snk_user_clock_reset_sync)
);

wire tx_core_rst_n1;
wire tx_core_rst_n2;
wire rx_core_rst_n1;
wire rx_core_rst_n2;

assign tx_avs_valid = src_ready & &tx_data_valid;
assign tx_core_rst_n = src_user_clock_reset_sync & tx_ehip_ready_coreclk_sync;
assign tx_core_rst_n1 = tx_core_rst_n & issp_tx_core_rst_sync;
assign tx_core_rst_n2 = tx_core_rst_n1 & ~ninit_done;

assign rx_core_rst_n = snk_user_clock_reset_sync & rx_pcs_ready_coreclk_sync;
assign rx_core_rst_n1 = rx_core_rst_n & issp_rx_core_rst_sync;
assign rx_core_rst_n2 = rx_core_rst_n1 & ~ninit_done;

always @(posedge rx_core_clkout) begin
snk_valid_d <= snk_valid;
snk_data_d <= snk_data;
rx_startofpacket_d <= rx_startofpacket;
rx_endofpacket_d <= rx_endofpacket;
tc_rx_is_usr_cmd_d <= tc_rx_is_usr_cmd;
snk_empty_d <= snk_empty;
rx_channel_d <= rx_channel;
tc_rx_num_valid_bytes_eob_d <= tc_rx_num_valid_bytes_eob;
end


always @ (posedge tx_core_clkout) begin
	 tx_core_rst_n_d   <= tx_core_rst_n2;
	 tx_core_rst_n_d1   <= tx_core_rst_n_d;
	 tx_core_clock_reset <= ~tx_core_reset_n_d[1];
end

always @ (posedge rx_core_clkout) begin
	 rx_core_rst_n_d   <= rx_core_rst_n2;
	 rx_core_rst_n_d1   <= rx_core_rst_n_d;
	 rx_core_clock_reset <= ~rx_core_reset_n_d[1];
end


seriallite4_dup seriallite_iv_streaming_inst
(
// Clocks and reset
.tx_core_clkout             (tx_core_clkout),
.rx_core_clkout             (rx_core_clkout),
.tx_core_rst_n              (tx_core_reset_n_d[0]),
.rx_core_rst_n              (rx_core_reset_n_d[0]),
.tx_pcs_fec_phy_reset_n      (~mgmt_clk_reset & issp_fec_phy_rst_sync),
.rx_pcs_fec_phy_reset_n      (~mgmt_clk_reset & issp_fec_phy_rst_sync),

// Source User Interface
.tx_avs_data                 (tx_avs_data[(lanes*64)-1:0]),
.tx_avs_valid                (tx_avs_valid),
.tx_avs_channel              (src_channel_in),
.tx_avs_startofpacket    	 (src_startofpacket_in),
.tx_avs_endofpacket          (src_endofpacket_in),
.tx_num_valid_bytes_eob      (tg_tx_num_valid_bytes_eob_in),
.tx_is_usr_cmd               (tg_tx_is_usr_cmd_in),
.tx_avs_empty              (src_empty_in),
.tx_pll_locked 			 (tx_pll_locked),
.tx_avs_ready              (src_ready),
.tx_link_up                (src_link_up),
.tx_error                  (src_error),
// Sink User Interface
.rx_avs_data               (snk_data),
.rx_avs_channel            (rx_channel),
.rx_avs_startofpacket      (rx_startofpacket),
.rx_avs_endofpacket        (rx_endofpacket),
.rx_num_valid_bytes_eob    (tc_rx_num_valid_bytes_eob),
.rx_is_usr_cmd             (tc_rx_is_usr_cmd),
.rx_avs_empty              (snk_empty),
.rx_avs_valid              (snk_valid),
.rx_avs_ready              ((enable_slave_mode_sync) ? src_link_up : snk_link_up),
.rx_link_up                (snk_link_up),
.rx_error                  (snk_error),

.rx_link_reinit				(sync_issp_sources_n4),
.tx_link_reinit				(sync_issp_sources_n5),

// Ethernet PHY IP xcvr reconfig interface
.reconfig_clk                   (mgmt_clk),
.reconfig_reset0                (mgmt_clk_reset),   // externally synchronized phy_mgmt_clk_reset
.xcvr_reconfig_address          (xcvr_reconfig_address),
.xcvr_reconfig_read             (xcvr_reconfig_read),
.xcvr_reconfig_readdata         (xcvr_reconfig_readdata),
.xcvr_reconfig_write            (xcvr_reconfig_write),
.xcvr_reconfig_writedata        (xcvr_reconfig_writedata),
.xcvr_reconfig_waitrequest      (xcvr_reconfig_waitrequest),
.phy_ehip_ready                 (phy_ehip_ready),
.phy_tx_lanes_stable            (phy_tx_lanes_stable),
.phy_rx_pcs_ready               (phy_rx_pcs_ready),
.phy_rx_block_lock              (phy_rx_block_lock),
.phy_rx_hi_ber					(phy_rx_hi_ber),
.rsfec_reconfig_read      	  	(rsfec_reconfig_read),
.rsfec_reconfig_write       	(rsfec_reconfig_write),
.rsfec_reconfig_address         (rsfec_reconfig_address),
.rsfec_reconfig_readdata        (rsfec_reconfig_readdata),
.rsfec_reconfig_waitrequest     (rsfec_reconfig_waitrequest),
.rsfec_reconfig_writedata       (rsfec_reconfig_writedata),
// Transceiver clock and data
.xcvr_ref_clk               	(pll_ref_clk), //input
.tx_serial_data            		(tx),
.rx_serial_data            		(rx)
);

genvar j;
generate
for (j = 0; j < (lanes*2+3) ; j = j + 1) begin: pulse_extend_loop
pulse_extension_sync #(
   .PULSE_WIDTH(5),
   .COMBI_INPUT(1),
   .RESET_VALUE(0),
   .SYNC_DEPTH(3)
) pulse_extension_sync_inst (
.source_clk (rx_core_clkout),
.source_clk_rstn (rx_core_reset_n_d[1]),
.dest_clk (snk_user_clock),
.dest_clk_rstn (~rx_user_clock_reset),
.datain (snk_error[j]),
.dataout (sink_error_out[j])
);
end
endgenerate

// RX DCFIFO
wire [5:0] rx_fifo_wr_pfull_out;  
assign tc_enable = snk_link_up & tg_enable;

always @(posedge rx_core_clkout) begin
snk_data_fifo_in <= {11'b0, snk_valid, rx_startofpacket, rx_endofpacket, rx_channel, snk_empty, tc_rx_num_valid_bytes_eob, tc_rx_is_usr_cmd, snk_data};
end

wire sync_rxfifo_reset;
altera_std_synchronizer_nocut #
(
.depth(3),
.rst_value(1)
)
rxfifo_reset_sync
(
.clk(snk_user_clock),
.reset_n(~rx_user_clock_reset),
.din(1'b0),
.dout(sync_rxfifo_reset)
);


seriallite4_dcfifo_seriallite4_dcfifo #
(
)
dcfifo_rx_data
(
.aclr(sync_rxfifo_reset),
.wrclk(rx_core_clkout),
.data(snk_data_fifo_in),
.wrreq(snk_data_fifo_in[(lanes*64)+20]),
//.wrfull(rx_adaptation_fifo_full),
.wrempty(),
.wrusedw(rx_fifo_wr_pfull_out),

.rdclk(snk_user_clock),
.q(snk_data_fifo_out),
.rdreq(rdreq),
.rdempty(rdempty),
.rdfull(),
.rdusedw(rdusedw)
);

always @(posedge snk_user_clock) begin
if (rx_user_clock_reset) begin
rx_fifo_full <= 1'd0;
rx_fifo_full_d <= 1'd0;
rx_adaptation_fifo_full <= 1'b0;
end else begin
rx_adaptation_fifo_full <= (rdusedw <= 6'd61) ? 1'b0 : 1'b1;
rx_fifo_full <= ~rdempty & rx_adaptation_fifo_full;
rx_fifo_full_d <= rx_fifo_full;
end
end

assign rdreq = (~rdempty) && tc_enable;
always @(posedge snk_user_clock) begin
if (rx_user_clock_reset) begin
valid <= 1'd0;
valid_d <= 1'd0;
end else begin
valid <= rdreq;
valid_d <= valid;
end
end

always @(posedge snk_user_clock) begin
snk_data_fifo_out_d <= snk_data_fifo_out;
end
assign snk_data_in                     = snk_data_fifo_out_d[(lanes*64)-1:0];
assign snk_valid_in                    = snk_data_fifo_out_d[(lanes*64)+20];
assign snk_empty_in                    = snk_data_fifo_out_d[(lanes*64)+9:(lanes*64)+5];
assign snk_startofpacket_in            = snk_data_fifo_out_d[(lanes*64)+19];
assign snk_endofpacket_in              = snk_data_fifo_out_d[(lanes*64)+18];
assign snk_channel_in                  = snk_data_fifo_out_d[(lanes*64)+17:(lanes*64)+10];
assign tc_rx_num_valid_bytes_eob_in    = snk_data_fifo_out_d[(lanes*64)+4:(lanes*64)+1];
assign tc_rx_is_usr_cmd_in             = snk_data_fifo_out_d[(lanes*64)];

//reserved
assign reserved_bits[31:0] = snk_data_fifo_out_d[(lanes*64)+31: (lanes*64)];

traffic_check #
(
.LANES(lanes),
.STREAM                    (STREAM_MODE)
)
traffic_check
(
// Clocks and reset
.user_clock(snk_user_clock),   //input
.user_clock_reset(rx_user_clock_reset),    //input

// Streaming data interface
.data(snk_data_fifo_out_d[(lanes*64)-1:0]),
.sync(snk_channel_in),
.start_of_burst(snk_startofpacket_in),
.end_of_burst(snk_endofpacket_in),
.tc_rx_num_valid_bytes_eob(tc_rx_num_valid_bytes_eob_in),
.tc_rx_is_usr_cmd(tc_rx_is_usr_cmd_in),
.valid(valid_d),
// Test control ports
.tc_enable(tc_enable),
.tc_test_mode(tg_test_mode),
.tc_enable_stalls(tg_enable_stalls),

// Status ports
.tc_burst_count(tc_burst_count),
.tc_words_received(tc_words_received),
.tc_lane_swap_error(tc_lane_swap_error),
.tc_lane_sequence_error(tc_lane_sequence_error),
.tc_lane_alignment_error(tc_lane_alignment_error),

//prbs data enable
.prbs_en_data(prbs_en_data)
);

reg sink_valid_d;
always @(posedge rx_core_clkout) begin
if (~rx_core_reset_n_d[1]) begin
sink_valid_d <= 1'b0;
end else begin 
sink_valid_d <= snk_valid;
		end
end

reg [31:0] start_time_cnt;
reg [31:0] start_time_cnt_p;
reg [63:0] payload;
wire [31:0] measure_time_pio;
reg count_valid;
reg bdw_count_valid;
reg stop_count;

always @(posedge rx_core_clkout) begin
	if (~rx_core_reset_n_d[1]) begin
stop_count <= 1'b0;
bdw_count_valid <= 1'b0;
end else begin 
bdw_count_valid <= sink_valid_d & count_valid;
stop_count <=   (start_time_cnt_p  == measure_time_pio) ? 1'b1 : 1'b0;
end
end

always @(posedge rx_core_clkout) begin
if (sync_issp_sources_n8) begin
start_time_cnt <= 32'h0;
start_time_cnt_p <= 32'h0;
count_valid <= 1'b0;
end else begin 
			if (sync_issp_sources_n7) begin
				if (stop_count) begin
					start_time_cnt <= start_time_cnt;
					count_valid <= 1'b0;
			   end else begin
				   start_time_cnt <= start_time_cnt + 32'h1;
					count_valid <= 1'b1;
				 end
			end
			start_time_cnt_p <= start_time_cnt;
	 end
end

always @(posedge rx_core_clkout) begin
if (sync_issp_sources_n8) begin
payload <= 64'h0;
end else begin 
			if (sync_issp_sources_n7) begin
				if (~bdw_count_valid) begin
					payload <= payload;
				end else begin
						if (bdw_count_valid)
						payload <= payload + 64'h1;
					end
			end
	 end
end

demo_control demo_control
(
// Clocks and reset
.clk_clk(mgmt_clk),
.reset_reset_n(mgmt_reset_n & ~ninit_done & issp_sources[0]),
// ISSP
.issp_sources_source (issp_sources),
.measure_payload_pio_h_export (payload[63:32]),
.measure_payload_pio_l_export (payload[31:0]),
.measure_time_pio_export (measure_time_pio),
.rx_coreclk_cnt_pio_export (start_time_cnt),
.rx_lanes_status_pio_export (rx_lanes_status_pio_in),
.system_status_pio_export (system_status_pio_in),
.tx_lanes_status_pio_export (tx_lanes_status_pio_in),
.txrx_lanes_status_pio_export (txrx_lanes_status_pio_in),
// Source core Ethernet HIP management interface
.source_mgmt_address(reconfig_addr),
.source_mgmt_write(status_write),
.source_mgmt_read(status_read),
.source_mgmt_readdata(status_readdata),
.source_mgmt_writedata(status_writedata),
.source_mgmt_waitrequest(status_waitrequest),
.source_mgmt_readdatavalid(status_readdata_valid),

// Demo management interface
.demo_mgmt_address(demo_mgmt_address),
.demo_mgmt_write(demo_mgmt_write),
.demo_mgmt_read(demo_mgmt_read),
.demo_mgmt_readdata(demo_mgmt_readdata),
.demo_mgmt_writedata(demo_mgmt_writedata),
.demo_mgmt_waitrequest(demo_mgmt_waitrequest),
.demo_mgmt_readdatavalid (demo_mgmt_readdata_valid),

// synchronized reset to coreclk
.rx_coreclk_sync_clk_clk (rx_core_clkout),
.tx_coreclk_sync_clk_clk (tx_core_clkout), 
.rx_coreclk_sync_in_reset_reset_n (rx_core_rst_n_d1),
.tx_coreclk_sync_in_reset_reset_n (tx_core_rst_n_d1),

.rx_coreclk_sync_out_reset_reset_n (rx_core_reset_n_d[0]),
.tx_coreclk_sync_out_reset_reset_n (tx_core_reset_n_d[0]),
.rx_coreclk_sync_out_reset_1_reset_n (rx_core_reset_n_d[1]),
.tx_coreclk_sync_out_reset_1_reset_n (tx_core_reset_n_d[1]),

// synchronized reset to mgmt_clk
.sync_reset_reset (mgmt_clk_reset)

);


seriallite4_io_pll u0 (
.locked   (iopll_locked),      //  output,  width = 1,  locked.export
.outclk_0 (src_user_clock),    //  output,  width = 1, outclk0.clk
.refclk   ( iopll_ref_clk ),   //   input,  width = 1,  refclk.clk
   .rst      ( mgmt_clk_reset )     //   input,  width = 1,   reset.reset
);

endmodule  // seriallite_iv_streaming_demo


