// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synopsys translate_off
`timescale 1ps/1ps
`include "testbench_defs.v"
// synopsys translate_on

`define  CONTINUOUS                       0
`define  BURST                            1


module traffic_check #
(
   parameter                     LANES                   = 2,             // Number of lanes on the link
   parameter                     STREAM                  = 1
)
(
   // Clocks and reset
   input                         user_clock,
   input                         user_clock_reset,

`ifdef SIMULATION
   // Traffic checker ports
   input       [63:0]            burst_descr,
   input                         burst_descr_ready,
   output reg                    burst_descr_read,
`endif

   // Streaming data interface
   input       [(LANES*64)-1:0]  data,
   input       [7:0]             sync,
   input       [4:0]             empty,
   input                         valid,
   input                         start_of_burst,
   input                         end_of_burst,
   input                         tc_rx_is_usr_cmd,
   input       [3:0]             tc_rx_num_valid_bytes_eob,
   // Test control ports
   input                         tc_enable,
   input                         tc_test_mode,
   input                         tc_enable_stalls,
	
   output reg  [LANES-1:0]       tc_non_monotic_increasing_sample_error,
   output reg  [LANES-1:0]       tc_no_last_valid_data_error,
   output reg  [LANES-1:0]       tc_prbs_data_error,
   output reg  [LANES-1:0]       tc_burst_number_inconsistent_at_sop_error,
   output reg  [LANES-1:0]       tc_burst_number_inconsistent_error,
   // Status interface
   output reg  [26:0]            tc_burst_count,
   output reg  [63:0]            tc_words_received,
   output reg  [LANES-1:0]       tc_lane_swap_error,
   output reg  [LANES-1:0]       tc_lane_sequence_error,
   output reg  [LANES-1:0]       tc_lane_alignment_error,

   //prbs data control
    input prbs_en_data
);


   reg         [(LANES*64)-1:0]  last_valid_lane_data;
   reg                           last_valid_lane_data_valid;
   wire        [63:0]            tc_words_received_comb;
   reg                          state, next_state;
   reg  [63:0]            		chk_tc_words_received;
   localparam  IDLE=1'b0, START_CNT=1'b1;

	reg tc_valid;
   always @(posedge user_clock) begin
      if (user_clock_reset) begin
		tc_valid <= 1'b0;
      end else begin
	  	tc_valid <= 1'b1;
      end
   end
   

  //
  // prbs_en_data synchronizer.
  //
   reg  tc_sync_prbs_en_data;
   reg  sync_tc_enable;
   always @(posedge user_clock) begin
      if (~tc_valid) begin
		tc_sync_prbs_en_data <= 1'b0;
		sync_tc_enable <= 1'b0;		
      end else begin
	  	tc_sync_prbs_en_data <= prbs_en_data;
	  	sync_tc_enable <= tc_enable;		
      end
   end
   
 //prbs generator for data
   wire        [63:0]  prbs_check;
   reg       [63:0]  prbs_check_data;
   reg first_valid_removal;
   
  //remove the first valid data.
    always @ (posedge user_clock) 
  begin
     if (user_clock_reset) begin
       first_valid_removal  <= 1'd0;
       end
    else if (valid == 1)begin
       first_valid_removal <= 1'd1;
    end
  end

	 wire prbs_check_valid;
     assign prbs_check_valid = first_valid_removal? valid:1'd0;
	 
      prbs_generator #
   (
      .PRBS_INITIAL_VALUE(97),
      .DATA_WIDTH(64),
      .PRBS(23)
   ) 
   prbs_data_generator_inst_3
   (
      .clk(user_clock),
      .nreset(~user_clock_reset),
      .xaui_word_align(1'b1),
      .dout(prbs_check),
      .pause(~prbs_check_valid),
      .insert_error(1'b0)
   );
 
  always @ (posedge user_clock) begin
    if (~tc_valid) begin
       prbs_check_data  <= 64'd1;
    end
    else begin 
	   if (first_valid_removal)begin
       prbs_check_data <= prbs_check;
	   end else begin
	   prbs_check_data <= 64'd1;
	   end
	end
  end
 

//
// Traffic Checker Storage
//
  wire full_stream;
  reg tcheck_words_count_enable;
  wire        [26:0]            tc_burst_count_comb_inc_out; 
  reg	tc_burst_count_enable;
  reg        [26:0]            tc_burst_count_reg;
  always @ (posedge user_clock) begin
     if (~tc_valid) begin
       tc_burst_count_enable  <= 1'b0;
       end
    else begin
       tc_burst_count_enable  <= (valid && tc_enable && start_of_burst) ? 1'b1 : 1'b0;
    end
  end
  
cascaded_counter #
(
.NUM_ADD_BLOCK  (3),
.ADD_BLOCK_BITS     (9)
) tc_burst_count_inst (
.reset (user_clock_reset),
.count_enable (tc_burst_count_enable),
.clock (user_clock),
.data_in (tc_burst_count_reg),
.data_out (tc_burst_count_comb_inc_out)

);

  generate
  
  always @ (posedge user_clock) begin
     if (~tc_valid) begin
       tc_burst_count_reg  <= 27'b0;
       end else begin
		if (STREAM == 1) begin	
		tc_burst_count_reg  <= tc_burst_count_comb_inc_out;
		end else begin
		tc_burst_count_reg  <= 27'b0;
		end
    end
  end
  
  if (STREAM == 1) begin
  assign full_stream = 1'b1;
  end else begin 
  assign full_stream = 1'b0;
  end
  
  endgenerate

  always @ (posedge user_clock) begin
     if (~tc_valid) begin
       tcheck_words_count_enable  <= 1'b0;
       end else begin
       tcheck_words_count_enable  <= (valid && tc_enable) ? 1'b1 : 1'b0;
    end
  end
  
       //
      // tc_words_received_comb counter
      //

  cascaded_counter #
   (
      .NUM_ADD_BLOCK                    (8),
      .ADD_BLOCK_BITS                   (8)
   )
   tcheck_word_counter_inst
   ( .clock(user_clock),.reset(user_clock_reset),.count_enable(tcheck_words_count_enable),.data_in(chk_tc_words_received),.data_out(tc_words_received_comb) );
   
    
   always @(posedge user_clock) begin
      if (~tc_valid) begin
         last_valid_lane_data       <= {(LANES<<6){1'b0}};
         last_valid_lane_data_valid <= 1'b0;
		 tc_burst_count		        <= 27'd0;
		 chk_tc_words_received 		    <= 64'd0;
		 tc_words_received    		<= 64'd0;
      end else begin
         last_valid_lane_data       <= (valid == 1'b1) ? data : last_valid_lane_data;
         last_valid_lane_data_valid <= (valid == 1'b1) ? 1'b1 : last_valid_lane_data_valid;
		 tc_burst_count             <= tc_burst_count_reg;  		 
         chk_tc_words_received      <= tc_words_received_comb;
         tc_words_received          <= chk_tc_words_received;		 
      end

   end

   genvar lane;

   generate

      for (lane = 0; lane < LANES; lane = lane + 1) begin : lane_traffic_check_inst

         wire        [4:0]             data_lane_number;
         wire        [26:0]            data_burst_number;
         wire        [31:0]            data_sample_number;
         wire        [31:0]            last_valid_lane_data_sample_number;
         wire        [63:0]            lane_a_data;
         wire        [63:0]            lane_b_data;
         wire        [63:0]            prbs_data_sample_number;
         assign data_lane_number                   = data[((LANES<<6)- 1)-(lane<<6)-: 5];
         assign data_burst_number                  = data[((LANES<<6)- 6)-(lane<<6)-:27];
         assign data_sample_number                 = data[((LANES<<6)-33)-(lane<<6)-:32];
         assign last_valid_lane_data_sample_number = last_valid_lane_data[((LANES<<6)-33)-(lane<<6)-:32];
         assign prbs_data_sample_number            = {data_lane_number,data_burst_number,data_sample_number};
         // Fixed at lane 0
         assign lane_a_data = data[63:0] & 64'h07FFFFFFFFFFFFFF ;
         assign lane_b_data = (lane <= LANES-1) ? data[(((lane+1)<<6)-1):((lane)<<6)] & 64'h07FFFFFFFFFFFFFF : 64'd0;

         
     //next cycle (nc), plus one(po) 

     reg [4:0] data_lane_number_nc;
     reg [LANES-1:0] last_valid_lane_data_valid_nc /* synthesis preserve_syn_only */;
     reg [LANES-1:0] valid_nc /* synthesis preserve_syn_only */;
	 reg [LANES-1:0] valid_nc_d /* synthesis preserve_syn_only */;
	 reg [LANES-1:0] valid_nc_d1 /* synthesis preserve_syn_only */;
	 reg [LANES-1:0] valid_nc_d2 /* synthesis preserve_syn_only */;
	 reg [LANES-1:0] valid_nc_d3 /* synthesis preserve_syn_only */;
     reg [LANES-1:0] tc_enable_nc /* synthesis preserve_syn_only */;
     reg [31:0]  data_sample_number_nc;
     reg [31:0] last_valid_lane_data_sample_number_nc;
     reg [LANES-1:0] prbs_en_data_nc /* synthesis preserve_syn_only */;
     
     reg [63:0] prbs_check_data_nc;
     reg [63:0] prbs_data_sample_number_nc;
	 reg [63:0] prbs_data_sample_number_nc_d;
	 reg [63:0] prbs_data_sample_number_nc_d1;	 
     reg [63:0] lane_a_data_nc;
     reg [63:0] lane_b_data_nc ;
     reg [63:0] lane_a_data_nc_d /* synthesis preserve_syn_only */;
     reg [63:0] lane_b_data_nc_d /* synthesis preserve_syn_only */;
     reg [26:0] tc_burst_count_nc;  
     reg [26:0] data_burst_number_nc; 
     reg [26:0] data_burst_number_nc_d; 
     reg [26:0] data_burst_number_nc_d1; 
     reg [26:0] data_burst_number_nc_d2;
     reg start_of_burst_nc, start_of_burst_nc_d, start_of_burst_nc_d1 /* synthesis preserve_syn_only */;   
     reg alignment_mismatch;
     reg sequence_mismatch;
     reg prbs_mismatch;
     reg burstcount_mismatch;
     reg start_of_burstcount_mismatch;
	 reg prbs_first_check_removal;
  
     always @(posedge user_clock) begin
     valid_nc                 <= {LANES{valid}};
     valid_nc_d                 <= valid_nc;
     valid_nc_d1                 <= valid_nc_d;
     valid_nc_d2                 <= valid_nc_d1;
     valid_nc_d3                 <= valid_nc_d2;
     data_lane_number_nc          <= data_lane_number;
     last_valid_lane_data_valid_nc      <= {LANES{last_valid_lane_data_valid}};
     tc_enable_nc               <= {LANES{sync_tc_enable}};
     data_sample_number_nc          <= data_sample_number;
     last_valid_lane_data_sample_number_nc  <= last_valid_lane_data_sample_number;
     prbs_en_data_nc            <= {LANES{tc_sync_prbs_en_data}};
     prbs_check_data_nc           <= prbs_check_data;
     prbs_data_sample_number_nc       <= prbs_data_sample_number;
     prbs_data_sample_number_nc_d       <= prbs_data_sample_number_nc;
     prbs_data_sample_number_nc_d1       <= prbs_data_sample_number_nc_d;
     lane_a_data_nc             <= lane_a_data;
	 lane_a_data_nc_d            <= lane_a_data_nc;
     lane_b_data_nc             <= lane_b_data;
	 lane_b_data_nc_d            <= lane_b_data_nc;
     tc_burst_count_nc            <= tc_burst_count;
     data_burst_number_nc         <= data_burst_number;
	 data_burst_number_nc_d         <= data_burst_number_nc;
	 data_burst_number_nc_d1         <= data_burst_number_nc_d;	 
	 data_burst_number_nc_d2         <= data_burst_number_nc_d1;	 	 
     start_of_burst_nc            <= start_of_burst;	
     start_of_burst_nc_d            <= start_of_burst_nc;	
     start_of_burst_nc_d1            <= start_of_burst_nc_d;	 	 
     end
	 
	 //remove first check
		always @ (posedge user_clock) begin
		 if (tc_valid) begin
		   prbs_first_check_removal  <= 1'd0;
		   end
		else if (valid_nc_d == 1)begin
		   prbs_first_check_removal <= 1'd1;
		end
	  end

	 
     always @(posedge user_clock) begin
			 
             // Check for lane swap error
             if ((valid_nc[lane] == 1'b1) && (data_lane_number_nc != lane) && (tc_enable_nc[lane] == 1'b1)&& (prbs_en_data_nc[lane] == 1'b0)) begin
                 tc_lane_swap_error[lane] <= 1'b1;
                  `ifdef SIMULATION

                     $write("   *** ERROR *** Lane Swap Error on lane %0d at %0t\n", lane, $time);
                     $write("                 Expected lane number %0d got lane number %0d\n", lane, data_lane_number);

                  `endif

             end else begin
                 tc_lane_swap_error[lane] <= 1'b0;
             end

             if (data_sample_number_nc != last_valid_lane_data_sample_number_nc + 32'd1) begin
				sequence_mismatch <= 1'b1 & valid_nc[lane]; //the last cycle check report before valid deassert is invalid which is expected delay
             end else begin
                 sequence_mismatch <= 1'b0;
             end
			 
             // Check for non-monotonicly increasing sample number
			 	if ((last_valid_lane_data_valid_nc[lane] == 1'b1) && (valid_nc_d[lane] == 1'b1) && sequence_mismatch && (tc_enable_nc[lane] == 1'b1) && (prbs_en_data_nc[lane] == 1'b0)) begin
                 tc_lane_sequence_error[lane] <= 1'b1;
                 tc_non_monotic_increasing_sample_error[lane] <= 1'b1;

                 `ifdef SIMULATION
                    $write("   *** ERROR *** Lane Sequence Error on lane %0d at %0t\n", lane, $time);
                    $write("                 Expected sample number %0d got sample number %0d\n", last_valid_lane_data_sample_number + 32'd1, data_sample_number);
                 `endif
             end  

             // No last valid data (for first counter data check)

             `ifdef SIMULATION
             else if ( (valid_nc_d[lane] == 1'b1) && sequence_mismatch && (tc_enable_nc[lane] == 1'b1) && (prbs_en_data_nc[lane] == 1'b0)) begin			 
                 tc_lane_sequence_error[lane] <= 1'b1;
                 tc_no_last_valid_data_error[lane] <= 1'b1;

                    $write("   *** ERROR *** There is no last valid Lane Sequence Error on lane %0d at %0t\n", lane, $time);
                    $write("                 Expected sample number %0d got sample number %0d\n", last_valid_lane_data_sample_number + 32'd1, data_sample_number);

             end  
             `endif   
			 if ((data_burst_number_nc_d != tc_burst_count_nc + 27'd1) && start_of_burst_nc_d) begin
				 start_of_burstcount_mismatch <= 1'b1 & valid_nc_d[lane];
             end else begin
                 start_of_burstcount_mismatch <= 1'b0;
             end
			 
			 if ((data_burst_number_nc_d2 != tc_burst_count_nc) && tc_valid) begin
				burstcount_mismatch <= 1'b1 & valid_nc_d2[lane];
             end else begin
                 burstcount_mismatch <= 1'b0;
             end

			 if (prbs_data_sample_number_nc_d != prbs_check_data_nc) begin 
			 	prbs_mismatch <= 1'b1 & valid_nc_d[lane] & prbs_en_data_nc[lane] & prbs_first_check_removal;
             end else begin
                 prbs_mismatch <= 1'b0;
             end
			 
        //check prbs data
		 if ((last_valid_lane_data_valid_nc[lane] == 1'b1) && prbs_mismatch && (tc_enable_nc[lane] == 1'b1)&&(prbs_en_data_nc[lane] == 1'b1)) begin
                 tc_lane_sequence_error[lane] <= 1'b1;
                 tc_prbs_data_error[lane] <= 1'b1;

                 `ifdef SIMULATION

                 $write("   *** ERROR *** PRBS Data error Lane Sequence Error on lane %0d at %0t\n", lane, $time);
                 $write("                 Expected sample number %0d got sample number %0d\n", prbs_check_data, prbs_data_sample_number);

                 `endif

             end    // Check for burst number inconsistancy at start of burst
			 else if (start_of_burstcount_mismatch && (tc_enable_nc[lane] == 1'b1) && (prbs_en_data_nc[lane] == 1'b0)&&(full_stream == 1'b1)) begin
                 tc_lane_sequence_error[lane] <= 1'b1;
                 tc_burst_number_inconsistent_at_sop_error[lane] <= 1'b1;

                 `ifdef SIMULATION

                  $write("   *** ERROR *** Burst number inconsistency at start of burst Lane Sequence Error on lane %0d during burst start at %0t\n", lane, $time);
                  $write("                 Expected burst number %0d got burst number %0d\n", tc_burst_count + 27'd1, data_burst_number);

                 `endif

             end


               // Check for burst number inconsistancy in burst
			   else if (burstcount_mismatch && (tc_enable_nc[lane] == 1'b1) && (prbs_en_data_nc[lane] == 1'b0)) begin		
                 tc_lane_sequence_error[lane] <= 1'b1;
                 tc_burst_number_inconsistent_error[lane] <= 1'b1;

                 `ifdef SIMULATION

                  $write("   *** ERROR *** Burst number inconsintency in burst Lane Sequence Error on lane %0d at %0t\n", lane, $time);
                  $write("                 Expected burst number %0d got burst number %0d\n", tc_burst_count, data_burst_number);

                 `endif

             end else begin
                 tc_lane_sequence_error[lane] <= 1'b0;
                 tc_non_monotic_increasing_sample_error[lane] <= 1'b0;
                 tc_no_last_valid_data_error[lane] <= 1'b0;
                 tc_prbs_data_error[lane] <= 1'b0;
                 tc_burst_number_inconsistent_at_sop_error[lane] <= 1'b0;
                 tc_burst_number_inconsistent_error[lane] <= 1'b0;
             end

	 
             if (lane_a_data_nc != lane_b_data_nc) begin
				alignment_mismatch <= 1'b1 & valid_nc[lane];
             end else begin
                 alignment_mismatch <= 1'b0;
             end
			 
             // Check for an alignment error between lane (lanes-1) and lane 0.
            if ((valid_nc_d[lane] == 1'b1) && alignment_mismatch && (tc_enable_nc[lane] == 1'b1)) begin
                 tc_lane_alignment_error[lane] <= 1'b1;

                 `ifdef SIMULATION

                  $write("   *** ERROR *** Lane Alignment Error on lane %0d at %0t\n", lane, $time);
                  $write("                 Expected data %16.16h got data %16.16h\n", lane_a_data, lane_b_data);

                 `endif

             end else begin
                 tc_lane_alignment_error[lane] <= 1'b0;
             end
     end
      end

   endgenerate

   
   
   
   
   
   
   

`ifdef SIMULATION

   wire     [7:0]             burst_descr_type;
   wire     [7:0]             burst_descr_inter_burst_interval;
   wire     [7:0]             burst_descr_sync_value;
   wire     [4:0]             burst_descr_empty_value;
   wire     [34:0]            burst_descr_sample_number;

   reg      [0:255]           burst_descr_type_encode;

   assign burst_descr_type                   = burst_descr[63:56];
   assign burst_descr_stall_duration         = burst_descr[55:48];
   assign burst_descr_inter_burst_interval   = burst_descr[55:48];
   assign burst_descr_sync_value             = burst_descr[47:40];
   assign burst_descr_empty_value            = burst_descr[39:35];
   assign burst_descr_sample_number          = burst_descr[34:00];


   always @* begin

      case (burst_descr_type)

         `BURST_DESCR_START_END: burst_descr_type_encode   <= "START_END";
         `BURST_DESCR_START:     burst_descr_type_encode   <= "START";
         `BURST_DESCR_END:       burst_descr_type_encode   <= "END";
         `BURST_DESCR_STALL:     burst_descr_type_encode   <= "STALL";
         default:                burst_descr_type_encode   <= "reserved";

     endcase

   end


   //
   // Burst Descriptor Checking clocked process
   //
   always @(posedge user_clock or posedge user_clock_reset) begin

      if (user_clock_reset == 1'b1) begin

      end else begin

         // Burst end descriptor
         if ((burst_descr_ready == 1'b1) && (tc_enable == 1'b1)&&(tc_sync_prbs_en_data == 1'b0)) begin

            case (burst_descr_type)

               // Burst start/end descriptor
               `BURST_DESCR_START_END: begin

                  // Wait for the next end of burst and check consistency
                  if ((start_of_burst == 1'b1) && (end_of_burst == 1'b1) && (valid == 1'b1)) begin

                     $write("\n");
                     $write("   Traffic Checker: Burst start/end descriptor read at time %0t\n", $time);
                     $write("                    Inter-burst interval %0d\n", burst_descr_inter_burst_interval);
                     $write("                    Sync value %0d\n",           burst_descr_sync_value);
                     $write("                    Empty value %0d\n",          burst_descr_empty_value);
                     $write("                    Sample number %0d\n",        burst_descr_sample_number);

                     if (burst_descr_sync_value != sync) begin

                        $write("\n");
                        $write("   Traffic Checker: *** ERROR *** Sync value verification error at time %0t\n", $time);
                        $write("                    Expected sync value %0d got sync value %0d on incoming burst\n", burst_descr_sync_value, sync);

                        test_env.testbench.test_abort = 1;

                     end

                     if (burst_descr_empty_value != empty) begin

                        $write("\n");
                        $write("   Traffic Checker: *** ERROR *** Empty value verification error at time %0t\n", $time);
                        $write("                    Expected empty value %0d got empty value %0d on incoming burst\n", burst_descr_empty_value, empty);

                        test_env.testbench.test_abort = 1;

                     end

                     if (burst_descr_sample_number != data[31:0]) begin

                        $write("\n");
                        $write("   Traffic Checker: *** ERROR *** Sample number verification error at time %0t\n", $time);
                        $write("                    Expected sample number %0d got sample number %0d on incoming burst\n", burst_descr_sample_number, data[31:0]);

                        test_env.testbench.test_abort = 1;

                     end

                  end

               end


               // Burst start descriptor
               `BURST_DESCR_START: begin

                  // Wait for the next end of burst and check consistency
                  if ((start_of_burst == 1'b1) && (valid == 1'b1)&&(tc_sync_prbs_en_data == 1'b0)) begin

                     $write("\n");
                     $write("   Traffic Checker: Burst start descriptor read at time %0t\n", $time);
                     $write("                    Sync value %0d\n",           burst_descr_sync_value);
                     $write("                    Sample number %0d\n",        burst_descr_sample_number);

                     if (burst_descr_sync_value != sync) begin

                        $write("\n");
                        $write("   Traffic Checker: *** ERROR *** Sync value verification error at time %0t\n", $time);
                        $write("                    Expected sync value %0d got sync value %0d on incoming burst\n", burst_descr_sync_value, sync);

                        test_env.testbench.test_abort = 1;

                     end

                     if ((burst_descr_sample_number != data[31:0]) && (tc_sync_prbs_en_data == 1'b0)) begin

                        $write("\n");
                        $write("   Traffic Checker: *** ERROR *** Sample number verification error at time %0t\n", $time);
                        $write("                    Expected sample number %0d got sample number %0d on incoming burst\n", burst_descr_sample_number, data[31:0]);

                        test_env.testbench.test_abort = 1;

                     end

                  end

               end


               // Burst end descriptor
               `BURST_DESCR_END: begin

                  // Wait for the next end of burst and check consistency
                  if ((end_of_burst == 1'b1) && (valid == 1'b1)&&(tc_sync_prbs_en_data == 1'b0)) begin

                     $write("\n");
                     $write("   Traffic Checker: Burst end descriptor read at time %0t\n", $time);
                     $write("                    Inter-burst interval %0d\n", burst_descr_inter_burst_interval);
                     $write("                    Empty value %0d\n",          burst_descr_empty_value);
                     $write("                    Sample number %0d\n",        burst_descr_sample_number);

                     if (burst_descr_empty_value != empty) begin

                        $write("\n");
                        $write("   Traffic Checker: *** ERROR *** Empty value verification error at time %0t\n", $time);
                        $write("                    Expected empty value %0d got empty value %0d on incoming burst\n", burst_descr_empty_value, empty);

                        test_env.testbench.test_abort = 1;

                     end

                  end

               end


               // Stall descriptor
               `BURST_DESCR_STALL: begin

                  // Wait for the next stall and check consistency
                  if (valid == 1'b0) begin

                     $write("\n");
                     $write("   Traffic Checker: Burst stall descriptor read at time %0t\n", $time);
                     $write("                    Stall duration %0d\n", burst_descr_stall_duration);
                     $write("                    Sample Number %0d\n",  burst_descr_sample_number);

                  end

               end


               default: begin

               end

            endcase

         end else begin

         end

      end

   end


   //
   // Burst Descriptor Checking combinatorial process
   //
   always @(*) begin

      if (user_clock_reset == 1'b1) begin

         burst_descr_read  = 1'b0;

      end else begin

         // Burst end descriptor
         if (burst_descr_ready == 1'b1) begin

            case (burst_descr_type)

               // Burst start/end descriptor
               `BURST_DESCR_START_END: begin

                  // Wait for the next end of burst and check consistency
                  burst_descr_read  = ((start_of_burst == 1'b1) && (end_of_burst == 1'b1) && (valid == 1'b1)) ? 1'b1 : 1'b0;

               end


               // Burst start descriptor
               `BURST_DESCR_START: begin

                  // Wait for the next end of burst and check consistency
                  burst_descr_read  = ((start_of_burst == 1'b1) && (valid == 1'b1)) ? 1'b1 : 1'b0;

               end


               // Burst end descriptor
               `BURST_DESCR_END: begin

                  // Wait for the next end of burst and check consistency
                  burst_descr_read  = ((end_of_burst == 1'b1) && (valid == 1'b1)) ? 1'b1 : 1'b0;

               end


               // Stall descriptor
               `BURST_DESCR_STALL: begin

                  // Wait for the next stall and check consistency
                  burst_descr_read  = (valid == 1'b0) ? 1'b1 : 1'b0;

               end


               default: begin

                  burst_descr_read  = 1'b0;

               end

            endcase

         end else if (tc_enable == 1'b0) begin

            burst_descr_read  = 1'b1;

         end else begin

            burst_descr_read  = 1'b0;

         end

      end

   end

`endif


endmodule

